Package: treeSS
Type: Package
Title: Tree-Spatial Scan Statistic for Cluster Detection
Version: 0.2.6
Authors@R: c(
    person("Allan", "Quadros", email = "allanvcq@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3250-5380")),
    person("Andre L. F.", "Cançado",
           role = "aut",
           comment = c(ORCID = "0000-0002-7019-8521"))
  )
Description: Implements the tree-spatial scan statistic for detecting clusters
    that combine both spatial and hierarchical structures, as proposed by
    Cancado et al. (2025) <doi:10.1007/s10651-025-00670-w>. The method extends
    Kulldorff (1997) <doi:10.1080/03610929708831995> circular spatial scan
    statistic and the tree-based scan statistic of Kulldorff et al. (2003)
    <doi:10.1111/1541-0420.00039> by searching for anomalies in both
    geographic regions and branches of hierarchical trees simultaneously. The
    package also provides standalone implementations of Kulldorff's circular
    spatial scan statistic and the tree-based scan statistic. Statistical
    significance is assessed via Monte Carlo simulation under a Poisson or
    binomial model, with optional 'OpenMP' parallelization.
License: GPL (>= 3)
URL: https://github.com/allanvc/treeSS
BugReports: https://github.com/allanvc/treeSS/issues
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.0), stats
LinkingTo: Rcpp
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, sf
SystemRequirements: GNU make
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-07-28 04:01:31 UTC; allan
Author: Allan Quadros [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3250-5380>),
  Andre L. F. Cançado [aut] (ORCID:
    <https://orcid.org/0000-0002-7019-8521>)
Maintainer: Allan Quadros <allanvcq@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-28 12:40:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(get_cluster_regions,default)
S3method(get_cluster_regions,sequential_scan)
S3method(print,circular_scan)
S3method(print,sequential_scan)
S3method(print,tree_scan)
S3method(print,treespatial_scan)
S3method(summary,circular_scan)
S3method(summary,sequential_scan)
S3method(summary,tree_scan)
S3method(summary,treespatial_scan)
export(aggregate_tree)
export(build_zones)
export(circular_scan)
export(filter_clusters)
export(generate_example_data)
export(get_cluster_regions)
export(sequential_scan)
export(tree_scan)
export(treespatial_scan)
importFrom(Rcpp,sourceCpp)
importFrom(utils,head)
useDynLib(treeSS, .registration = TRUE)

# treeSS

**Tree-Spatial Scan Statistic for Cluster Detection**

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/treeSS)](https://CRAN.R-project.org/package=treeSS)
[![R-CMD-check](https://github.com/allanvc/treeSS/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/allanvc/treeSS/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/allanvc/treeSS/graph/badge.svg)](https://app.codecov.io/gh/allanvc/treeSS)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
<!-- badges: end -->

Implements the tree-spatial scan statistic (Cançado et al., 2025), which
detects clusters that are anomalous in both geographic space and a
hierarchical tree simultaneously. The method searches over circular spatial
zones and branches of a classification tree to find regions where observed
cases significantly exceed expectations under a Poisson or binomial model,
selectable via the `model` argument.

## Installation

```r
# CRAN version
install.packages("treeSS")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("allanvc/treeSS")
```

## Quick start

```r
library(treeSS)

# Example: London road collisions
data(london_collisions)
data(london_tree)

# The scan functions take a data.frame as the first argument and refer to
# its columns by name. This keeps the choice of denominator,
# coordinates, etc. transparent.
result <- treespatial_scan(
  london_collisions,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = london_tree,
  nsim        = 999, seed = 42,
  n_cores     = 4L                 # parallelize the MC over 4 threads
)
print(result)

# Extract cluster membership for visualization
cr <- get_cluster_regions(result, n_clusters = 3, overlap = FALSE)
```

## Included datasets

| Dataset | Country | Domain | Regions | Tree |
|:--------|:--------|:-------|:--------|:-----|
| `rj_mortality` + `rj_tree` | Brazil | Infant mortality | 92 municipalities | ICD-10 (622 nodes) |
| `fl_deaths` | USA | General mortality | 65 counties | raw (built by user) |
| `london_collisions` + `london_tree` | UK | Road collisions | 33 boroughs | Light x Road x Junction (81 nodes) |
| `chicago_crimes` + `chicago_tree` | USA | Crime | 77 community areas | Type x Description x Location (2841 nodes) |
| `rj_map`, `london_boroughs_map`, `chicago_map` | Brazil / UK / USA | Polygon boundaries | 92 / 33 / 77 | -- |

## Key functions

- `treespatial_scan()` — tree-spatial scan (main function)
- `circular_scan()` — Kulldorff's spatial scan
- `tree_scan()` — tree-based scan
- `filter_clusters()` — non-overlapping secondary clusters (Cançado et al. 2025)
- `sequential_scan()` — sequential adjustment for secondary clusters (Zhang, Assunção & Kulldorff 2010)
- `get_cluster_regions()` — cluster membership for any visualization package

## Visualization

The package is visualization-agnostic. `get_cluster_regions()` returns a
data.frame that can be merged with any spatial object for plotting with
ggplot2, leaflet, tmap, or any other mapping package. The bundled `sf`
boundary datasets (`rj_map`, `london_boroughs_map`, `chicago_map`) let the
examples map clusters without any external boundary download. See
`vignette("introduction")` for worked examples with ggplot2 + `rj_map`
(Brazil), leaflet + tigris (USA), and leaflet + `london_boroughs_map`
(London).

## References

Cançado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
(2025). A tree-spatial scan statistic. *Environmental and Ecological
Statistics*, 32, 953–978. doi:10.1007/s10651-025-00670-w

##=============================================================================
## Example 4: General Mortality in Florida, USA (2016)
##
## Two secondary-cluster analyses are shown:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025,
##      Sec. 5.1.1): single scan + filter_clusters /
##      get_cluster_regions(n_clusters = N).
##   2. sequential_scan() -- Zhang, Assuncao & Kulldorff (2010).
##
## Pedagogical workflow:
##   - Load raw long-format data (fl_deaths)
##   - Build ICD-10 tree directly from the codes that actually appear
##   - Download county polygons + centroids from tigris
##   - Assemble a long-format data.frame (cases, population, region_id, x, y, node_id)
##   - Run the standard scan plus both secondary-cluster procedures
##
## Data: CDC WONDER Compressed Mortality File 1999-2016
##=============================================================================

options(bitmapType = "cairo")

library(treeSS)
library(ggplot2)
library(sf)


## ---- 1. Load raw data ----
data(fl_deaths)

cat("=== Florida Mortality 2016 ===\n")
cat("Rows (raw):  ", nrow(fl_deaths), "\n")
cat("Counties:    ", length(unique(fl_deaths$county_fips)), "\n")
cat("ICD-10 codes:", length(unique(fl_deaths$icd10_code)), "\n")
cat("Total deaths:", sum(fl_deaths$deaths), "\n")

icd_lookup <- unique(fl_deaths[, c("icd10_code", "icd10_desc")])
icd_desc <- function(code) {
  d <- icd_lookup$icd10_desc[icd_lookup$icd10_code == code]
  if (length(d) == 0) return(code)
  d[1]
}


## ---- 2. Build ICD-10 tree directly from fl_deaths ----
##
## fl_deaths uses two kinds of codes:
##   * 3-char (e.g. "C56", "R54") - leaf at the chapter level
##   * 5-char with a dot (e.g. "A04.7") - leaf under a 3-char internal parent

cat("\nBuilding ICD-10 tree from fl_deaths codes...\n")
icd_codes <- sort(unique(fl_deaths$icd10_code))

parsed <- data.frame(
  code = icd_codes,
  three = ifelse(grepl("\\.", icd_codes),
                  sub("\\..*", "", icd_codes),
                  icd_codes),
  is_three_leaf = !grepl("\\.", icd_codes) & nchar(icd_codes) == 3,
  stringsAsFactors = FALSE
)

icd_chapter_of <- function(three) {
  L <- substr(three, 1, 1)
  num <- suppressWarnings(as.integer(substr(three, 2, 3)))
  switch(
    L,
    A = "I_AB", B = "I_AB",
    C = "II_C_D",
    D = if (!is.na(num) && num <= 48) "II_C_D" else "III_D",
    E = "IV_E",
    F = "V_F",
    G = "VI_G",
    H = if (!is.na(num) && num <= 59) "VII_H" else "VIII_H",
    I = "IX_I",
    J = "X_J",
    K = "XI_K",
    L = "XII_L",
    M = "XIII_M",
    N = "XIV_N",
    O = "XV_O",
    P = "XVI_P",
    Q = "XVII_Q",
    R = "XVIII_R",
    S = "XIX_S_T", T = "XIX_S_T",
    V = "XX_V_Y", W = "XX_V_Y", X = "XX_V_Y", Y = "XX_V_Y",
    "Other"
  )
}
parsed$chapter <- sapply(parsed$three, icd_chapter_of)

fl_tree <- unique(rbind(
  data.frame(node_id = "Root", parent_id = NA_character_,
             stringsAsFactors = FALSE),
  data.frame(node_id = unique(parsed$chapter), parent_id = "Root",
             stringsAsFactors = FALSE),
  unique(data.frame(node_id = parsed$three, parent_id = parsed$chapter,
                    stringsAsFactors = FALSE)),
  data.frame(
    node_id   = parsed$code[!parsed$is_three_leaf],
    parent_id = parsed$three[!parsed$is_three_leaf],
    stringsAsFactors = FALSE
  )
))

parents <- unique(fl_tree$parent_id[!is.na(fl_tree$parent_id)])
leaves  <- setdiff(fl_tree$node_id, parents)
stopifnot(length(setdiff(icd_codes, leaves)) == 0)
cat("  Total nodes:", nrow(fl_tree),
    "| Chapters:", length(unique(parsed$chapter)),
    "| Leaves:", length(leaves), "\n")


## ---- 3. Download county polygons + centroids ----
if (!requireNamespace("tigris", quietly = TRUE)) {
  stop("Install 'tigris': install.packages('tigris')")
}
library(tigris)
cat("\nDownloading FL county boundaries...\n")
fl_map <- counties(state = "FL", cb = TRUE, year = 2016)
fl_map <- st_transform(fl_map, 4326)
centroids <- st_centroid(st_geometry(fl_map))
coords <- st_coordinates(centroids)
fl_map$x <- coords[, "X"]
fl_map$y <- coords[, "Y"]


## ---- 4. Assemble the long-format data.frame ----
fl_pop <- aggregate(population ~ county_fips, data = fl_deaths, FUN = max)
xy <- data.frame(county_fips = fl_map$GEOID, x = fl_map$x, y = fl_map$y,
                  stringsAsFactors = FALSE)
dat <- merge(fl_deaths[, c("county_fips", "icd10_code", "deaths")],
             fl_pop, by = "county_fips")
dat <- merge(dat, xy, by = "county_fips")
dat$region_id <- as.integer(factor(dat$county_fips,
                                    levels = sort(unique(dat$county_fips))))
dat <- dat[dat$deaths > 0, ]

region_info <- unique(dat[, c("region_id", "county_fips")])
cat("Long rows:", nrow(dat), "\n")


## ---- 5. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4, max_pop_pct=0.05)...\n")
system.time({
  result_fl <- treespatial_scan(
    dat,
    cases       = deaths,
    population  = population,
    region_id   = region_id,
    x           = x,
    y           = y,
    node_id     = icd10_code,
    tree        = fl_tree,
    max_pop_pct = 0.05,
    nsim        = 999, seed = 2016,
    n_cores     = 4L
  )
})
print(result_fl)


## ---- 6. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_fl)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 7. Sequential scan (Zhang, Assuncao & Kulldorff, 2010) ----
cat("\n=== Sequential scan (Zhang et al. 2010) ===\n")
seq_fl <- sequential_scan(
  dat,
  cases       = deaths,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = icd10_code,
  tree        = fl_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2016,
  max_pop_pct = 0.05, n_cores = 4L
)
print(seq_fl)


## ---- 8. Cluster membership ----
cr1    <- merge(get_cluster_regions(result_fl, n_clusters = 1, overlap = FALSE),
                region_info, by = "region_id")
cr2    <- merge(get_cluster_regions(result_fl, n_clusters = 2, overlap = TRUE),
                region_info, by = "region_id")
cr_seq <- merge(get_cluster_regions(seq_fl, overlap = TRUE),
                region_info, by = "region_id")


## ---- 9. Plot 1: Most likely cluster ----
fl1 <- merge(fl_map, cr1, by.x = "GEOID", by.y = "county_fips", all.x = TRUE)
mlc <- result_fl$most_likely_cluster

p1 <- ggplot(fl1) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.15, alpha = 0.75) +
  scale_fill_manual(
    values   = c("1" = "#C44E52"),
    na.value = "gray95",
    labels   = c("1" = paste0(mlc$node_id, " (",
                              length(mlc$region_ids), " counties)")),
    name     = "Cluster"
  ) +
  labs(
    title    = paste0("Tree-Spatial Scan: most likely cluster (", mlc$node_id, ")"),
    subtitle = paste0(icd_desc(mlc$node_id),
                      " | LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_fl$pvalue, digits = 3),
                      " | ", mlc$cases, " deaths (expected ",
                      round(mlc$expected, 1), ")")
  ) +
  theme_minimal(base_size = 12) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 8))

ggsave("fl_cluster_mlc.png", p1, width = 9, height = 8, dpi = 300)
cat("Saved: fl_cluster_mlc.png\n")


## ---- 10. Plot 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
fl2 <- merge(fl_map, cr2, by.x = "GEOID", by.y = "county_fips")
palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")

p2 <- ggplot(fl2) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(values = palette2, na.value = "gray95",
                    name = "Cluster", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  labs(title    = "Tree-Spatial Scan: Top 2 Distinct Clusters",
       subtitle = "Cancado et al. (2025), Sec. 5.1.1 - Florida mortality, 2016") +
  theme_minimal(base_size = 11) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        strip.text    = element_text(face = "bold", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 7),
        legend.position = "none")

ggsave("fl_clusters_top2.png", p2, width = 14, height = 6, dpi = 300)
cat("Saved: fl_clusters_top2.png\n")


## ---- 11. Plot 3: Sequential scan clusters ----
n_iter <- seq_fl$n_iter
if (n_iter > 0) {
  # Cross-join the map polygons with every panel so all counties are
  # drawn in each iteration (those outside the analysis dataset show
  # up NA-coloured rather than as an extra empty "NA" panel).
  panels <- unique(cr_seq$panel)
  cr_seq_keys <- unique(cr_seq[, c("county_fips", "cluster", "node_id",
                                    "llr", "pvalue", "panel")])
  fl_seq <- merge(
    do.call(rbind,
            lapply(panels, function(p) cbind(fl_map, panel = p))),
    cr_seq_keys,
    by.x = c("GEOID", "panel"), by.y = c("county_fips", "panel"),
    all.x = TRUE
  )
  palette_seq <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_seq) <- as.character(seq_len(n_iter))

  n_sig <- sum(seq_fl$clusters$significant, na.rm = TRUE)
  p_seq <- ggplot(fl_seq) +
    geom_sf(aes(fill = factor(cluster)), color = "gray40",
            linewidth = 0.12, alpha = 0.75) +
    scale_fill_manual(values = palette_seq, na.value = "gray95",
                      name = "Iteration", na.translate = FALSE) +
    facet_wrap(~ panel, nrow = 1) +
    labs(title    = paste0("Sequential Scan (Zhang et al. 2010): ",
                            n_iter, " iterations, ",
                            n_sig, " significant"),
         subtitle = "Each panel = one iteration (regions removed before next).") +
    theme_minimal(base_size = 11) +
    theme(plot.title    = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray40"),
          strip.text    = element_text(face = "bold", size = 9),
          axis.title    = element_blank(),
          axis.text     = element_text(color = "gray50", size = 7),
          legend.position = "none")

  ggsave("fl_clusters_sequential.png", p_seq,
         width = max(8, 4 * n_iter), height = 6, dpi = 300)
  cat("Saved: fl_clusters_sequential.png (", n_iter, " iterations)\n")
}


## ---- 12. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "(", icd_desc(mlc$node_id), ")\n")
cat("  Counties:", length(mlc$region_ids), "\n")
cat("  Deaths:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_fl$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Sequential scan (Zhang et al. 2010): ", n_iter, " iterations, ",
    sum(seq_fl$clusters$significant, na.rm = TRUE),
    " significant\n", sep = "")
cat("\nDone!\n")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>",
  eval = FALSE
)

## ----fl-load------------------------------------------------------------------
#  library(treeSS)
#  data(fl_deaths)
#  
#  str(fl_deaths)
#  #> 'data.frame': N obs. of 5 variables:
#  #>  $ county_fips: chr  ...
#  #>  $ icd10_code : chr  "C56" "C61" "I10" ...
#  #>  $ icd10_desc : chr  ...
#  #>  $ deaths     : int  ...
#  #>  $ population : int  ...
#  
#  c(rows         = nrow(fl_deaths),
#    counties     = length(unique(fl_deaths$county_fips)),
#    icd10_codes  = length(unique(fl_deaths$icd10_code)),
#    total_deaths = sum(fl_deaths$deaths))

## ----fl-tree------------------------------------------------------------------
#  icd_codes <- sort(unique(fl_deaths$icd10_code))
#  
#  parsed <- data.frame(
#    code  = icd_codes,
#    three = ifelse(grepl("\\.", icd_codes),
#                    sub("\\..*", "", icd_codes),
#                    icd_codes),
#    is_three_leaf = !grepl("\\.", icd_codes) & nchar(icd_codes) == 3,
#    stringsAsFactors = FALSE
#  )
#  
#  # Map 3-character codes to ICD-10 chapters
#  icd_chapter_of <- function(three) {
#    L   <- substr(three, 1, 1)
#    num <- suppressWarnings(as.integer(substr(three, 2, 3)))
#    switch(L,
#      A = "I_AB", B = "I_AB",
#      C = "II_C_D",
#      D = if (!is.na(num) && num <= 48) "II_C_D" else "III_D",
#      E = "IV_E",  F = "V_F",  G = "VI_G",
#      H = if (!is.na(num) && num <= 59) "VII_H" else "VIII_H",
#      I = "IX_I",  J = "X_J",  K = "XI_K", L = "XII_L",
#      M = "XIII_M", N = "XIV_N", O = "XV_O", P = "XVI_P",
#      Q = "XVII_Q", R = "XVIII_R",
#      S = "XIX_S_T", T = "XIX_S_T",
#      V = "XX_V_Y", W = "XX_V_Y", X = "XX_V_Y", Y = "XX_V_Y",
#      "Other"
#    )
#  }
#  parsed$chapter <- sapply(parsed$three, icd_chapter_of)
#  
#  fl_tree <- unique(rbind(
#    data.frame(node_id = "Root", parent_id = NA_character_,
#               stringsAsFactors = FALSE),
#    data.frame(node_id = unique(parsed$chapter), parent_id = "Root",
#               stringsAsFactors = FALSE),
#    unique(data.frame(node_id = parsed$three, parent_id = parsed$chapter,
#                      stringsAsFactors = FALSE)),
#    data.frame(
#      node_id   = parsed$code[!parsed$is_three_leaf],
#      parent_id = parsed$three[!parsed$is_three_leaf],
#      stringsAsFactors = FALSE
#    )
#  ))
#  
#  # Sanity check: every code in fl_deaths is a leaf of fl_tree
#  parents <- unique(fl_tree$parent_id[!is.na(fl_tree$parent_id)])
#  leaves  <- setdiff(fl_tree$node_id, parents)
#  stopifnot(length(setdiff(icd_codes, leaves)) == 0)
#  
#  c(total_nodes = nrow(fl_tree),
#    chapters    = length(unique(parsed$chapter)),
#    leaves      = length(leaves))

## ----fl-polygons--------------------------------------------------------------
#  library(sf)
#  library(tigris)
#  
#  fl_map <- counties(state = "FL", cb = TRUE, year = 2016)
#  fl_map <- st_transform(fl_map, 4326)
#  
#  centroids   <- st_centroid(st_geometry(fl_map))
#  coords      <- st_coordinates(centroids)
#  fl_map$x    <- coords[, "X"]
#  fl_map$y    <- coords[, "Y"]

## ----fl-vectors---------------------------------------------------------------
#  # Per-county population: same value across all leaves, so take any one
#  fl_pop <- aggregate(population ~ county_fips, data = fl_deaths,
#                       FUN = max)
#  
#  # Per-county centroids from the polygon layer
#  xy <- data.frame(county_fips = fl_map$GEOID,
#                    x = fl_map$x, y = fl_map$y,
#                    stringsAsFactors = FALSE)
#  
#  dat <- merge(fl_deaths[, c("county_fips", "icd10_code", "deaths")],
#               fl_pop, by = "county_fips")
#  dat <- merge(dat, xy, by = "county_fips")
#  
#  # Integer region_id is convenient downstream; here we use the rank
#  # of county_fips in alphabetical order
#  dat$region_id <- as.integer(factor(
#    dat$county_fips, levels = sort(unique(dat$county_fips))
#  ))
#  
#  # Keep only rows with at least one death (sparse-by-default scan)
#  dat <- dat[dat$deaths > 0, ]
#  
#  c(long_rows = nrow(dat),
#    counties  = length(unique(dat$region_id)))

## ----fl-scan------------------------------------------------------------------
#  result_fl <- treespatial_scan(
#    cases       = dat$deaths,
#    population  = dat$population,
#    region_id   = dat$region_id,
#    x           = dat$x,
#    y           = dat$y,
#    node_id     = dat$icd10_code,
#    tree        = fl_tree,
#    max_pop_pct = 0.05,
#    nsim        = 999, seed = 2016,
#    n_cores     = 4L
#  )
#  print(result_fl)

## ----fl-filter----------------------------------------------------------------
#  filter_clusters(result_fl)[1:5,
#                              c("node_id", "n_regions",
#                                "cases", "expected", "llr",
#                                "pvalue")]

## ----fl-seq-------------------------------------------------------------------
#  seq_fl <- sequential_scan(
#    cases       = dat$deaths,
#    population  = dat$population,
#    region_id   = dat$region_id,
#    x           = dat$x,
#    y           = dat$y,
#    node_id     = dat$icd10_code,
#    tree        = fl_tree,
#    max_iter    = 5L, alpha = 0.05,
#    nsim        = 999, seed = 2016,
#    max_pop_pct = 0.05, n_cores = 4L
#  )
#  print(seq_fl)

## ----fl-plot------------------------------------------------------------------
#  library(ggplot2)
#  library(dplyr)
#  
#  region_info <- unique(dat[, c("region_id", "county_fips")])
#  
#  cr_seq <- merge(
#    get_cluster_regions(seq_fl, overlap = TRUE),
#    region_info, by = "region_id"
#  )
#  
#  # Cross-join the polygon layer with every iteration panel so all
#  # counties appear in every panel (those outside the analysis
#  # dataset show as NA-coloured rather than as an extra empty panel)
#  panels       <- unique(cr_seq$panel)
#  cr_seq_keys  <- unique(cr_seq[, c("county_fips", "cluster",
#                                     "node_id", "llr", "pvalue",
#                                     "panel")])
#  fl_seq_plot <- merge(
#    do.call(rbind,
#            lapply(panels, function(p) cbind(fl_map, panel = p))),
#    cr_seq_keys,
#    by.x = c("GEOID", "panel"),
#    by.y = c("county_fips", "panel"),
#    all.x = TRUE
#  )
#  
#  ggplot(fl_seq_plot) +
#    geom_sf(aes(fill = factor(cluster)), color = "gray40",
#            linewidth = 0.12, alpha = 0.75) +
#    scale_fill_manual(
#      values = c("#C44E52", "#4C72B0", "#55A868",
#                 "#8172B2", "#CCB974"),
#      na.value = "gray95", na.translate = FALSE
#    ) +
#    facet_wrap(~ panel, nrow = 1) +
#    theme_minimal() +
#    theme(legend.position = "none")

##=============================================================================
## Example 3: Road Traffic Collisions in London, UK (2022)
##
## Two secondary-cluster analyses are shown:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025,
##      Sec. 5.1.1): single scan + filter_clusters /
##      get_cluster_regions(n_clusters = N).
##   2. sequential_scan() -- Zhang, Assuncao & Kulldorff (2010).
##
## Maps: leaflet (interactive, HTML)
## Polygons: london_boroughs_map (included in the package)
##=============================================================================

library(treeSS)
library(leaflet)
library(sf)
library(htmlwidgets)
library(htmltools)


## ---- 1. Load data ----
data(london_collisions)
data(london_tree)
data(london_boroughs_map)

cat("=== London Road Collisions 2022 ===\n")
cat("Rows (long):     ", nrow(london_collisions), "\n")
cat("Unique boroughs: ", length(unique(london_collisions$region_id)), "\n")
cat("Tree nodes:      ", nrow(london_tree), "\n")
cat("Total collisions:", sum(london_collisions$cases), "\n")


## ---- 2. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_ldn <- treespatial_scan(
    cases       = london_collisions$cases,
    population  = london_collisions$population,
    region_id   = london_collisions$region_id,
    x           = london_collisions$x,
    y           = london_collisions$y,
    node_id     = london_collisions$node_id,
    tree        = london_tree,
    max_pop_pct = 0.25,
    nsim        = 999, seed = 42,
    n_cores     = 4L
  )
})
print(result_ldn)


## ---- 3. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_ldn)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 4. Sequential scan (Zhang, Assuncao & Kulldorff, 2010) ----
cat("\n=== Sequential scan (Zhang et al. 2010) ===\n")
seq_ldn <- sequential_scan(
  cases       = london_collisions$cases,
  population  = london_collisions$population,
  region_id   = london_collisions$region_id,
  x           = london_collisions$x,
  y           = london_collisions$y,
  node_id     = london_collisions$node_id,
  tree        = london_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 42,
  max_pop_pct = 0.25, n_cores = 4L
)
print(seq_ldn)


## ---- 5. Cluster membership + descriptive lookup ----
region_info <- unique(london_collisions[, c("region_id", "borough")])

cr1    <- merge(get_cluster_regions(result_ldn, n_clusters = 1, overlap = FALSE),
                region_info, by = "region_id")
cr2    <- merge(get_cluster_regions(result_ldn, n_clusters = 2, overlap = TRUE),
                region_info, by = "region_id")
cr_seq <- merge(get_cluster_regions(seq_ldn, overlap = TRUE),
                region_info, by = "region_id")

JOIN_X <- "NAME"     # column in london_boroughs_map
JOIN_Y <- "borough"  # column in cluster results


## ---- 6. Map 1: Most likely cluster (leaflet) ----
ldn1 <- merge(london_boroughs_map, cr1,
              by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)
mlc <- result_ldn$most_likely_cluster

ldn1$label <- ifelse(
  is.na(ldn1$cluster),
  paste0("<b>", ldn1$NAME, "</b><br>Not in cluster"),
  paste0("<b>", ldn1$NAME, "</b><br>",
         "<span style='color:#C44E52'><b>", ldn1$node_id, "</b></span><br>",
         "LR: ", round(ldn1$llr, 1),
         " | p = ", round(ldn1$pvalue, 3))
)

map1 <- leaflet(ldn1) %>%
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(
    fillColor   = ~ifelse(is.na(cluster), "#E8E8E8", "#C44E52"),
    fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
    color       = "gray40", weight = 1.5,
    label       = ~lapply(label, HTML)
  ) %>%
  setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
  addLegend(
    position = "bottomright",
    colors   = c("#C44E52", "#E8E8E8"),
    labels   = c(paste0(mlc$node_id, " (",
                        length(mlc$region_ids), " boroughs)"),
                 "Not in cluster"),
    title    = paste0("LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_ldn$pvalue, digits = 3)),
    opacity  = 0.8
  )

saveWidget(map1, "london_cluster_mlc.html", selfcontained = TRUE)
cat("Saved: london_cluster_mlc.html\n")


## ---- 7. Map 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
# A borough may appear in multiple cluster panels in overlap mode; for the
# leaflet map we keep the lowest-ranked cluster (cluster 1 wins over 2).
cr2_first <- cr2[order(cr2$cluster), ]
cr2_first <- cr2_first[!duplicated(cr2_first$borough), ]

ldn2 <- merge(london_boroughs_map, cr2_first,
              by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)

palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")
top2 <- filter_clusters(result_ldn)
top2 <- head(top2, 2)

ldn2$fillc <- ifelse(is.na(ldn2$cluster), "#E8E8E8",
                     palette2[as.character(ldn2$cluster)])
ldn2$label <- ifelse(
  is.na(ldn2$cluster),
  paste0("<b>", ldn2$NAME, "</b><br>Not in any top-2 cluster"),
  paste0("<b>", ldn2$NAME, "</b><br>",
         "Cluster ", ldn2$cluster, ": ",
         "<span style='color:", palette2[as.character(ldn2$cluster)],
         "'><b>", ldn2$node_id, "</b></span><br>",
         "LR: ", round(ldn2$llr, 1),
         " | p = ", round(ldn2$pvalue, 3))
)

map2 <- leaflet(ldn2) %>%
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(
    fillColor   = ~fillc,
    fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
    color       = "gray40", weight = 1.5,
    label       = ~lapply(label, HTML)
  ) %>%
  setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
  addLegend(
    position = "bottomright",
    colors   = c(palette2, "#E8E8E8"),
    labels   = c(paste0("Cluster ", seq_along(top2$node_id), ": ",
                        top2$node_id),
                 "Other"),
    title    = "Top 2 distinct (paper Sec. 5.1.1)",
    opacity  = 0.8
  )

saveWidget(map2, "london_clusters_top2.html", selfcontained = TRUE)
cat("Saved: london_clusters_top2.html\n")


## ---- 8. Map 3: Sequential scan clusters ----
n_iter <- seq_ldn$n_iter
if (n_iter > 0) {
  cr_seq_first <- cr_seq[order(cr_seq$cluster), ]
  cr_seq_first <- cr_seq_first[!duplicated(cr_seq_first$borough), ]

  ldn_seq <- merge(london_boroughs_map, cr_seq_first,
                   by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)

  palette_seq <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_seq) <- as.character(seq_len(n_iter))

  n_sig <- sum(seq_ldn$clusters$significant, na.rm = TRUE)
  ldn_seq$fillc <- ifelse(is.na(ldn_seq$cluster), "#E8E8E8",
                          palette_seq[as.character(ldn_seq$cluster)])
  ldn_seq$label <- ifelse(
    is.na(ldn_seq$cluster),
    paste0("<b>", ldn_seq$NAME, "</b><br>Not detected"),
    paste0("<b>", ldn_seq$NAME, "</b><br>",
           "Iteration ", ldn_seq$cluster, ": ",
           "<span style='color:", palette_seq[as.character(ldn_seq$cluster)],
           "'><b>", ldn_seq$node_id, "</b></span><br>",
           "LR: ", round(ldn_seq$llr, 1),
           " | p = ", round(ldn_seq$pvalue, 3))
  )

  map_seq <- leaflet(ldn_seq) %>%
    addProviderTiles(providers$CartoDB.Positron) %>%
    addPolygons(
      fillColor   = ~fillc,
      fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
      color       = "gray40", weight = 1.5,
      label       = ~lapply(label, HTML)
    ) %>%
    setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
    addLegend(
      position = "bottomright",
      colors   = c(palette_seq, "#E8E8E8"),
      labels   = c(paste0("Iteration ", seq_len(n_iter)), "Other"),
      title    = paste0("Sequential scan (Zhang et al. 2010)<br>",
                        n_sig, " of ", n_iter, " significant"),
      opacity  = 0.8
    )

  saveWidget(map_seq, "london_clusters_sequential.html",
             selfcontained = TRUE)
  cat("Saved: london_clusters_sequential.html\n")
}


## ---- 9. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "\n")
cat("  Boroughs:", length(mlc$region_ids), "\n")
cat("  Collisions:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_ldn$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Sequential scan (Zhang et al. 2010): ", n_iter, " iterations, ",
    sum(seq_ldn$clusters$significant, na.rm = TRUE),
    " significant\n", sep = "")
cat("\nDone!\n")

##=============================================================================
## Example 3: Road Traffic Collisions in London, UK (2022)
##
## Two secondary-cluster analyses are shown:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025,
##      Sec. 5.1.1): single scan + filter_clusters /
##      get_cluster_regions(n_clusters = N).
##   2. sequential_scan() -- Zhang, Assuncao & Kulldorff (2010).
##
## Maps: leaflet (interactive, HTML)
## Polygons: london_boroughs_map (included in the package)
##=============================================================================

library(treeSS)
library(leaflet)
library(sf)
library(htmlwidgets)
library(htmltools)


## ---- 1. Load data ----
data(london_collisions)
data(london_tree)
data(london_boroughs_map)

cat("=== London Road Collisions 2022 ===\n")
cat("Rows (long):     ", nrow(london_collisions), "\n")
cat("Unique boroughs: ", length(unique(london_collisions$region_id)), "\n")
cat("Tree nodes:      ", nrow(london_tree), "\n")
cat("Total collisions:", sum(london_collisions$cases), "\n")


## ---- 2. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_ldn <- treespatial_scan(
    london_collisions,
    cases       = cases,
    population  = population,
    region_id   = region_id,
    x           = x,
    y           = y,
    node_id     = node_id,
    tree        = london_tree,
    max_pop_pct = 0.25,
    nsim        = 999, seed = 42,
    n_cores     = 4L
  )
})
print(result_ldn)


## ---- 3. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_ldn)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 4. Sequential scan (Zhang, Assuncao & Kulldorff, 2010) ----
cat("\n=== Sequential scan (Zhang et al. 2010) ===\n")
seq_ldn <- sequential_scan(
  london_collisions,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = london_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 42,
  max_pop_pct = 0.25, n_cores = 4L
)
print(seq_ldn)


## ---- 5. Cluster membership + descriptive lookup ----
region_info <- unique(london_collisions[, c("region_id", "borough")])

cr1    <- merge(get_cluster_regions(result_ldn, n_clusters = 1, overlap = FALSE),
                region_info, by = "region_id")
cr2    <- merge(get_cluster_regions(result_ldn, n_clusters = 2, overlap = TRUE),
                region_info, by = "region_id")
cr_seq <- merge(get_cluster_regions(seq_ldn, overlap = TRUE),
                region_info, by = "region_id")

JOIN_X <- "NAME"     # column in london_boroughs_map
JOIN_Y <- "borough"  # column in cluster results


## ---- 6. Map 1: Most likely cluster (leaflet) ----
ldn1 <- merge(london_boroughs_map, cr1,
              by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)
mlc <- result_ldn$most_likely_cluster

ldn1$label <- ifelse(
  is.na(ldn1$cluster),
  paste0("<b>", ldn1$NAME, "</b><br>Not in cluster"),
  paste0("<b>", ldn1$NAME, "</b><br>",
         "<span style='color:#C44E52'><b>", ldn1$node_id, "</b></span><br>",
         "LR: ", round(ldn1$llr, 1),
         " | p = ", round(ldn1$pvalue, 3))
)

map1 <- leaflet(ldn1) %>%
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(
    fillColor   = ~ifelse(is.na(cluster), "#E8E8E8", "#C44E52"),
    fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
    color       = "gray40", weight = 1.5,
    label       = ~lapply(label, HTML)
  ) %>%
  setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
  addLegend(
    position = "bottomright",
    colors   = c("#C44E52", "#E8E8E8"),
    labels   = c(paste0(mlc$node_id, " (",
                        length(mlc$region_ids), " boroughs)"),
                 "Not in cluster"),
    title    = paste0("LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_ldn$pvalue, digits = 3)),
    opacity  = 0.8
  )

saveWidget(map1, "london_cluster_mlc.html", selfcontained = TRUE)
cat("Saved: london_cluster_mlc.html\n")


## ---- 7. Map 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
# A borough may appear in multiple cluster panels in overlap mode; for the
# leaflet map we keep the lowest-ranked cluster (cluster 1 wins over 2).
cr2_first <- cr2[order(cr2$cluster), ]
cr2_first <- cr2_first[!duplicated(cr2_first$borough), ]

ldn2 <- merge(london_boroughs_map, cr2_first,
              by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)

palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")
top2 <- filter_clusters(result_ldn)
top2 <- head(top2, 2)

ldn2$fillc <- ifelse(is.na(ldn2$cluster), "#E8E8E8",
                     palette2[as.character(ldn2$cluster)])
ldn2$label <- ifelse(
  is.na(ldn2$cluster),
  paste0("<b>", ldn2$NAME, "</b><br>Not in any top-2 cluster"),
  paste0("<b>", ldn2$NAME, "</b><br>",
         "Cluster ", ldn2$cluster, ": ",
         "<span style='color:", palette2[as.character(ldn2$cluster)],
         "'><b>", ldn2$node_id, "</b></span><br>",
         "LR: ", round(ldn2$llr, 1),
         " | p = ", round(ldn2$pvalue, 3))
)

map2 <- leaflet(ldn2) %>%
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(
    fillColor   = ~fillc,
    fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
    color       = "gray40", weight = 1.5,
    label       = ~lapply(label, HTML)
  ) %>%
  setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
  addLegend(
    position = "bottomright",
    colors   = c(palette2, "#E8E8E8"),
    labels   = c(paste0("Cluster ", seq_along(top2$node_id), ": ",
                        top2$node_id),
                 "Other"),
    title    = "Top 2 distinct (paper Sec. 5.1.1)",
    opacity  = 0.8
  )

saveWidget(map2, "london_clusters_top2.html", selfcontained = TRUE)
cat("Saved: london_clusters_top2.html\n")


## ---- 8. Map 3: Sequential scan clusters ----
n_iter <- seq_ldn$n_iter
if (n_iter > 0) {
  cr_seq_first <- cr_seq[order(cr_seq$cluster), ]
  cr_seq_first <- cr_seq_first[!duplicated(cr_seq_first$borough), ]

  ldn_seq <- merge(london_boroughs_map, cr_seq_first,
                   by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)

  palette_seq <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_seq) <- as.character(seq_len(n_iter))

  n_sig <- sum(seq_ldn$clusters$significant, na.rm = TRUE)
  ldn_seq$fillc <- ifelse(is.na(ldn_seq$cluster), "#E8E8E8",
                          palette_seq[as.character(ldn_seq$cluster)])
  ldn_seq$label <- ifelse(
    is.na(ldn_seq$cluster),
    paste0("<b>", ldn_seq$NAME, "</b><br>Not detected"),
    paste0("<b>", ldn_seq$NAME, "</b><br>",
           "Iteration ", ldn_seq$cluster, ": ",
           "<span style='color:", palette_seq[as.character(ldn_seq$cluster)],
           "'><b>", ldn_seq$node_id, "</b></span><br>",
           "LR: ", round(ldn_seq$llr, 1),
           " | p = ", round(ldn_seq$pvalue, 3))
  )

  map_seq <- leaflet(ldn_seq) %>%
    addProviderTiles(providers$CartoDB.Positron) %>%
    addPolygons(
      fillColor   = ~fillc,
      fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
      color       = "gray40", weight = 1.5,
      label       = ~lapply(label, HTML)
    ) %>%
    setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
    addLegend(
      position = "bottomright",
      colors   = c(palette_seq, "#E8E8E8"),
      labels   = c(paste0("Iteration ", seq_len(n_iter)), "Other"),
      title    = paste0("Sequential scan (Zhang et al. 2010)<br>",
                        n_sig, " of ", n_iter, " significant"),
      opacity  = 0.8
    )

  saveWidget(map_seq, "london_clusters_sequential.html",
             selfcontained = TRUE)
  cat("Saved: london_clusters_sequential.html\n")
}


## ---- 9. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "\n")
cat("  Boroughs:", length(mlc$region_ids), "\n")
cat("  Collisions:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_ldn$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Sequential scan (Zhang et al. 2010): ", n_iter, " iterations, ",
    sum(seq_ldn$clusters$significant, na.rm = TRUE),
    " significant\n", sep = "")
cat("\nDone!\n")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>",
  eval = FALSE
)

## ----rj-data------------------------------------------------------------------
#  library(treeSS)
#  data(rj_mortality)
#  data(rj_tree)
#  
#  c(rows           = nrow(rj_mortality),
#    municipalities = length(unique(rj_mortality$region_id)),
#    tree_nodes     = nrow(rj_tree),
#    total_deaths   = sum(rj_mortality$cases),
#    total_births   = sum(unique(
#      rj_mortality[, c("region_id", "live_births")]
#    )$live_births))
#  #> rows           municipalities  tree_nodes  total_deaths  total_births
#  #> 1440           89              622         2981          219124

## ----rj-scan------------------------------------------------------------------
#  result_rj <- treespatial_scan(
#    cases       = rj_mortality$cases,
#    population  = rj_mortality$live_births,
#    region_id   = rj_mortality$region_id,
#    x           = rj_mortality$x,
#    y           = rj_mortality$y,
#    node_id     = rj_mortality$node_id,
#    tree        = rj_tree,
#    max_pop_pct = 0.50,
#    nsim        = 999, seed = 2016,
#    n_cores     = 4L
#  )
#  print(result_rj)

## ----rj-spatial-only----------------------------------------------------------
#  rj_agg <- unique(rj_mortality[, c("region_id", "x", "y", "live_births")])
#  rj_agg$cases <- tapply(rj_mortality$cases,
#                          rj_mortality$region_id,
#                          sum)[as.character(rj_agg$region_id)]
#  
#  result_spatial <- circular_scan(
#    cases       = rj_agg$cases,
#    population  = rj_agg$live_births,
#    region_id   = rj_agg$region_id,
#    x           = rj_agg$x,
#    y           = rj_agg$y,
#    max_pop_pct = 0.50,
#    nsim        = 999, seed = 2016
#  )
#  result_spatial$most_likely_cluster$region_ids

## ----rj-filter----------------------------------------------------------------
#  filter_clusters(result_rj)[1:5,
#                              c("node_id", "n_regions",
#                                "cases", "expected", "llr")]

## ----rj-getregions------------------------------------------------------------
#  # Most likely cluster only (single map)
#  cr1 <- get_cluster_regions(result_rj, n_clusters = 1, overlap = FALSE)
#  
#  # Top-2 distinct clusters (faceted map)
#  cr2 <- get_cluster_regions(result_rj, n_clusters = 2, overlap = TRUE)

## ----rj-plot------------------------------------------------------------------
#  library(ggplot2)
#  library(geobr)
#  library(sf)
#  
#  mun       <- read_municipality(code_muni = "RJ", year = 2016)
#  mun$code6 <- as.integer(substr(mun$code_muni, 1, 6))
#  
#  cr2 <- merge(get_cluster_regions(result_rj, n_clusters = 2,
#                                    overlap = TRUE),
#               unique(rj_mortality[, c("region_id", "ibge_code")]),
#               by = "region_id")
#  mun_facet <- merge(mun, cr2, by.x = "code6", by.y = "ibge_code")
#  
#  ggplot(mun_facet) +
#    geom_sf(aes(fill = factor(cluster)), color = "gray40",
#            alpha = 0.75) +
#    scale_fill_manual(values = c("1" = "#C44E52", "2" = "#4C72B0"),
#                      na.value = "gray95", na.translate = FALSE) +
#    facet_wrap(~ panel, nrow = 1) +
#    theme_minimal() +
#    theme(legend.position = "none")

## ----rj-seq-------------------------------------------------------------------
#  seq_rj <- sequential_scan(
#    cases       = rj_mortality$cases,
#    population  = rj_mortality$live_births,
#    region_id   = rj_mortality$region_id,
#    x           = rj_mortality$x,
#    y           = rj_mortality$y,
#    node_id     = rj_mortality$node_id,
#    tree        = rj_tree,
#    max_iter    = 5L, alpha = 0.05,
#    nsim        = 999, seed = 2016,
#    max_pop_pct = 0.50, n_cores = 4L
#  )
#  print(seq_rj)

## ----rj-seq-table-------------------------------------------------------------
#  seq_rj$clusters[, c("iteration", "node_id", "n_regions",
#                      "llr", "pvalue", "significant")]

##=============================================================================
## Example 2: Crime in Chicago, USA (2023)
##
## Two secondary-cluster analyses are shown:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025,
##      Sec. 5.1.1): single scan + filter_clusters /
##      get_cluster_regions(n_clusters = N).
##   2. sequential_scan() -- Zhang, Assuncao & Kulldorff (2010).
##
## Population denominator: ACS 2020 5-year residential population per
## community area (replaces the bundled compositional 'population' column).
##=============================================================================

options(bitmapType = "cairo")

library(treeSS)
library(ggplot2)
library(sf)


## ---- 1. Load data ----
data(chicago_crimes)
data(chicago_tree)
data(chicago_map)

cat("=== Chicago Crime 2023 ===\n")
cat("Rows (long):           ", nrow(chicago_crimes), "\n")
cat("Unique community areas:", length(unique(chicago_crimes$region_id)), "\n")
cat("Tree nodes:            ", nrow(chicago_tree), "\n")
cat("Total incidents:       ", sum(chicago_crimes$cases), "\n")

crime_desc <- function(node_id) {
  parts <- strsplit(node_id, " \\| ")[[1]]
  if (length(parts) == 3) return(paste0(parts[2], " (", parts[3], ")"))
  if (length(parts) == 2) return(parts[2])
  node_id
}


## ---- 2. Choose denominator: residents (not total incidents) ----
##
## chicago_crimes ships with two denominator columns:
##   * population:      total incidents per area (compositional - useful
##                       for "which crime types over-occur where", but
##                       not a population at risk)
##   * pop_residential: ACS 2020 5-year residential population. This is
##                       the appropriate denominator for an incidence-rate
##                       analysis (incidents per resident), analogous to
##                       deaths/live_births in the paper's RJ application.
##
## We use pop_residential below.
cat("Total Chicago residents:",
    sum(unique(chicago_crimes[, c("area_number", "pop_residential")])$pop_residential),
    "\n")


## ---- 3. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_chi <- treespatial_scan(
    cases       = chicago_crimes$cases,
    population  = chicago_crimes$pop_residential,
    region_id   = chicago_crimes$region_id,
    x           = chicago_crimes$x,
    y           = chicago_crimes$y,
    node_id     = chicago_crimes$node_id,
    tree        = chicago_tree,
    max_pop_pct = 0.25,
    nsim        = 999, seed = 2023,
    n_cores     = 4L
  )
})
print(result_chi)


## ---- 4. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_chi)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 5. Sequential scan (Zhang, Assuncao & Kulldorff, 2010) ----
cat("\n=== Sequential scan (Zhang et al. 2010) ===\n")
seq_chi <- sequential_scan(
  cases       = chicago_crimes$cases,
  population  = chicago_crimes$pop_residential,
  region_id   = chicago_crimes$region_id,
  x           = chicago_crimes$x,
  y           = chicago_crimes$y,
  node_id     = chicago_crimes$node_id,
  tree        = chicago_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2023,
  max_pop_pct = 0.25, n_cores = 4L
)
print(seq_chi)


## ---- 6. Cluster membership + descriptive lookup ----
region_info <- unique(chicago_crimes[, c("region_id", "area_number", "name")])

cr1    <- merge(get_cluster_regions(result_chi, n_clusters = 1, overlap = FALSE),
                region_info, by = "region_id")
cr2    <- merge(get_cluster_regions(result_chi, n_clusters = 2, overlap = TRUE),
                region_info, by = "region_id")
cr_seq <- merge(get_cluster_regions(seq_chi, overlap = TRUE),
                region_info, by = "region_id")


## ---- 7. Plot 1: Most likely cluster ----
chi1 <- merge(chicago_map, cr1, by.x = "AREA_NUM", by.y = "area_number",
              all.x = TRUE)
mlc <- result_chi$most_likely_cluster

p1 <- ggplot(chi1) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.15, alpha = 0.75) +
  scale_fill_manual(
    values   = c("1" = "#C44E52"),
    na.value = "gray95",
    labels   = c("1" = paste0(crime_desc(mlc$node_id), " (",
                              length(mlc$region_ids), " areas)")),
    name     = "Cluster"
  ) +
  labs(
    title    = "Tree-Spatial Scan: most likely cluster",
    subtitle = paste0(crime_desc(mlc$node_id),
                      " | LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_chi$pvalue, digits = 3),
                      " | ", mlc$cases, " incidents (expected ",
                      round(mlc$expected, 1), ")")
  ) +
  theme_minimal(base_size = 12) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 8))

ggsave("chicago_cluster_mlc.png", p1, width = 9, height = 9, dpi = 300)
cat("Saved: chicago_cluster_mlc.png\n")


## ---- 8. Plot 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
chi2 <- merge(chicago_map, cr2, by.x = "AREA_NUM", by.y = "area_number")
palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")

p2 <- ggplot(chi2) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(values = palette2, na.value = "gray95",
                    name = "Cluster", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  labs(title    = "Tree-Spatial Scan: Top 2 Distinct Clusters",
       subtitle = "Cancado et al. (2025), Sec. 5.1.1 - denominator: residents") +
  theme_minimal(base_size = 11) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        strip.text    = element_text(face = "bold", size = 9),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 7),
        legend.position = "none")

ggsave("chicago_clusters_top2.png", p2, width = 14, height = 6, dpi = 300)
cat("Saved: chicago_clusters_top2.png\n")


## ---- 9. Plot 3: Sequential scan clusters ----
n_iter <- seq_chi$n_iter
if (n_iter > 0) {
  # Cross-join the map polygons with every panel so all areas are drawn
  # in each iteration (those outside the analysis dataset show up
  # NA-coloured rather than as an extra empty "NA" panel).
  panels <- unique(cr_seq$panel)
  cr_seq_keys <- unique(cr_seq[, c("area_number", "cluster", "node_id",
                                    "llr", "pvalue", "panel")])
  chi_seq <- merge(
    do.call(rbind,
            lapply(panels, function(p) cbind(chicago_map, panel = p))),
    cr_seq_keys,
    by.x = c("AREA_NUM", "panel"), by.y = c("area_number", "panel"),
    all.x = TRUE
  )
  palette_seq <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_seq) <- as.character(seq_len(n_iter))

  n_sig <- sum(seq_chi$clusters$significant, na.rm = TRUE)
  p_seq <- ggplot(chi_seq) +
    geom_sf(aes(fill = factor(cluster)), color = "gray40",
            linewidth = 0.12, alpha = 0.75) +
    scale_fill_manual(values = palette_seq, na.value = "gray95",
                      name = "Iteration", na.translate = FALSE) +
    facet_wrap(~ panel, nrow = 1) +
    labs(title    = paste0("Sequential Scan (Zhang et al. 2010): ",
                            n_iter, " iterations, ",
                            n_sig, " significant"),
         subtitle = "Each panel = one iteration (regions removed before next).") +
    theme_minimal(base_size = 11) +
    theme(plot.title    = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray40"),
          strip.text    = element_text(face = "bold", size = 9),
          axis.title    = element_blank(),
          axis.text     = element_text(color = "gray50", size = 7),
          legend.position = "none")

  ggsave("chicago_clusters_sequential.png", p_seq,
         width = max(8, 4 * n_iter), height = 6, dpi = 300)
  cat("Saved: chicago_clusters_sequential.png (", n_iter, " iterations)\n")
}


## ---- 10. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "\n")
cat("  Description:", crime_desc(mlc$node_id), "\n")
cat("  Areas:", length(mlc$region_ids), "\n")
cat("  Incidents:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_chi$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Sequential scan (Zhang et al. 2010): ", n_iter, " iterations, ",
    sum(seq_chi$clusters$significant, na.rm = TRUE),
    " significant\n", sep = "")
cat("\nDone!\n")

##=============================================================================
## Example 2: Crime in Chicago, USA (2023)
##
## Two secondary-cluster analyses are shown:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025,
##      Sec. 5.1.1): single scan + filter_clusters /
##      get_cluster_regions(n_clusters = N).
##   2. sequential_scan() -- Zhang, Assuncao & Kulldorff (2010).
##
## Population denominator: the compositional 'population' column (total
## incidents per community area). This asks which (crime type, area)
## combinations are over-represented relative to the citywide mix of
## incidents -- an inherently tree-spatial question, for which a specific
## branch (not the root) is the expected most-likely cluster.
##=============================================================================

options(bitmapType = "cairo")

library(treeSS)
library(ggplot2)
library(sf)


## ---- 1. Load data ----
data(chicago_crimes)
data(chicago_tree)
data(chicago_map)

cat("=== Chicago Crime 2023 ===\n")
cat("Rows (long):           ", nrow(chicago_crimes), "\n")
cat("Unique community areas:", length(unique(chicago_crimes$region_id)), "\n")
cat("Tree nodes:            ", nrow(chicago_tree), "\n")
cat("Total incidents:       ", sum(chicago_crimes$cases), "\n")

crime_desc <- function(node_id) {
  parts <- strsplit(node_id, " \\| ")[[1]]
  if (length(parts) == 3) return(paste0(parts[2], " (", parts[3], ")"))
  if (length(parts) == 2) return(parts[2])
  node_id
}


## ---- 2. Choose denominator: compositional (total incidents per area) ----
##
## chicago_crimes ships with two denominator columns:
##   * population:      total incidents per area. With this denominator the
##                       expected count of the root branch equals its
##                       observed count, so the root cannot win and the scan
##                       reports a specific (crime type, area) combination --
##                       the tree-spatial question of interest.
##   * pop_residential: ACS 2020 5-year residential population. With this
##                       denominator the scan answers an incidence-rate
##                       question (incidents per resident); on these data the
##                       most-likely cluster is a broad-spectrum spatial
##                       hotspot reported at the root, not a specific branch.
##
## We use the compositional 'population' below.
cat("Denominator: compositional (total incidents per area)\n")


## ---- 3. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_chi <- treespatial_scan(
    chicago_crimes,
    cases       = cases,
    population  = population,
    region_id   = region_id,
    x           = x,
    y           = y,
    node_id     = node_id,
    tree        = chicago_tree,
    max_pop_pct = 0.25,
    nsim        = 999, seed = 2023,
    n_cores     = 4L
  )
})
print(result_chi)


## ---- 4. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_chi)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
           5))


## ---- 5. Sequential scan (Zhang, Assuncao & Kulldorff, 2010) ----
cat("\n=== Sequential scan (Zhang et al. 2010) ===\n")
seq_chi <- sequential_scan(
  chicago_crimes,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = chicago_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2023,
  max_pop_pct = 0.25, n_cores = 4L
)
print(seq_chi)


## ---- 6. Cluster membership + descriptive lookup ----
region_info <- unique(chicago_crimes[, c("region_id", "area_number", "name")])

cr1    <- merge(get_cluster_regions(result_chi, n_clusters = 1, overlap = FALSE),
                region_info, by = "region_id")
cr2    <- merge(get_cluster_regions(result_chi, n_clusters = 2, overlap = TRUE),
                region_info, by = "region_id")
cr_seq <- merge(get_cluster_regions(seq_chi, overlap = TRUE),
                region_info, by = "region_id")


## ---- 7. Plot 1: Most likely cluster ----
chi1 <- merge(chicago_map, cr1, by.x = "AREA_NUM", by.y = "area_number",
              all.x = TRUE)
mlc <- result_chi$most_likely_cluster

p1 <- ggplot(chi1) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.15, alpha = 0.75) +
  scale_fill_manual(
    values   = c("1" = "#C44E52"),
    na.value = "gray95",
    labels   = c("1" = paste0(crime_desc(mlc$node_id), " (",
                              length(mlc$region_ids), " areas)")),
    name     = "Cluster"
  ) +
  labs(
    title    = "Tree-Spatial Scan: most likely cluster",
    subtitle = paste0(crime_desc(mlc$node_id),
                      " | LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_chi$pvalue, digits = 3),
                      " | ", mlc$cases, " incidents (expected ",
                      round(mlc$expected, 1), ")")
  ) +
  theme_minimal(base_size = 12) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 8))

ggsave("chicago_cluster_mlc.png", p1, width = 9, height = 9, dpi = 300)
cat("Saved: chicago_cluster_mlc.png\n")


## ---- 8. Plot 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
chi2 <- merge(chicago_map, cr2, by.x = "AREA_NUM", by.y = "area_number")
palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")

p2 <- ggplot(chi2) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(values = palette2, na.value = "gray95",
                    name = "Cluster", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  labs(title    = "Tree-Spatial Scan: Top 2 Distinct Clusters",
       subtitle = "Cancado et al. (2025), Sec. 5.1.1 - denominator: total incidents per area") +
  theme_minimal(base_size = 11) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        strip.text    = element_text(face = "bold", size = 9),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 7),
        legend.position = "none")

ggsave("chicago_clusters_top2.png", p2, width = 14, height = 6, dpi = 300)
cat("Saved: chicago_clusters_top2.png\n")


## ---- 9. Plot 3: Sequential scan clusters ----
n_iter <- seq_chi$n_iter
if (n_iter > 0) {
  # Cross-join the map polygons with every panel so all areas are drawn
  # in each iteration (those outside the analysis dataset show up
  # NA-coloured rather than as an extra empty "NA" panel).
  panels <- unique(cr_seq$panel)
  cr_seq_keys <- unique(cr_seq[, c("area_number", "cluster", "node_id",
                                   "llr", "pvalue", "panel")])
  chi_seq <- merge(
    do.call(rbind,
            lapply(panels, function(p) cbind(chicago_map, panel = p))),
    cr_seq_keys,
    by.x = c("AREA_NUM", "panel"), by.y = c("area_number", "panel"),
    all.x = TRUE
  )
  palette_seq <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_seq) <- as.character(seq_len(n_iter))
  
  n_sig <- sum(seq_chi$clusters$significant, na.rm = TRUE)
  p_seq <- ggplot(chi_seq) +
    geom_sf(aes(fill = factor(cluster)), color = "gray40",
            linewidth = 0.12, alpha = 0.75) +
    scale_fill_manual(values = palette_seq, na.value = "gray95",
                      name = "Iteration", na.translate = FALSE) +
    facet_wrap(~ panel, nrow = 1) +
    labs(title    = paste0("Sequential Scan (Zhang et al. 2010): ",
                           n_iter, " iterations, ",
                           n_sig, " significant"),
         subtitle = "Each panel = one iteration (regions removed before next).") +
    theme_minimal(base_size = 11) +
    theme(plot.title    = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray40"),
          strip.text    = element_text(face = "bold", size = 9),
          axis.title    = element_blank(),
          axis.text     = element_text(color = "gray50", size = 7),
          legend.position = "none")
  
  ggsave("chicago_clusters_sequential.png", p_seq,
         width = max(8, 4 * n_iter), height = 6, dpi = 300)
  cat("Saved: chicago_clusters_sequential.png (", n_iter, " iterations)\n")
}


## ---- 10. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "\n")
cat("  Description:", crime_desc(mlc$node_id), "\n")
cat("  Areas:", length(mlc$region_ids), "\n")
cat("  Incidents:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_chi$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Sequential scan (Zhang et al. 2010): ", n_iter, " iterations, ",
    sum(seq_chi$clusters$significant, na.rm = TRUE),
    " significant\n", sep = "")
cat("\nDone!\n")

##=============================================================================
## Example 1: Infant Mortality in Rio de Janeiro, Brazil (2016)
##
## Reproduces Section 5.2 of:
##   Cancado et al. (2025). Environmental and Ecological Statistics, 32, 953-978.
##
## Two secondary-cluster analyses are shown:
##   1. The PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025,
##      Sec. 5.1.1): single scan + filter_clusters /
##      get_cluster_regions(n_clusters = N).
##   2. sequential_scan() -- Zhang, Assuncao & Kulldorff (2010).
##=============================================================================

options(bitmapType = "cairo")     # robust PNG rendering on Linux servers

library(treeSS)
library(ggplot2)
library(geobr)
library(sf)


## ---- 1. Load data ----
data(rj_mortality)
data(rj_tree)

cat("=== Rio de Janeiro Infant Mortality 2016 ===\n")
cat("Rows (long):          ", nrow(rj_mortality), "\n")
cat("Unique municipalities:", length(unique(rj_mortality$region_id)), "\n")
cat("Tree nodes:           ", nrow(rj_tree), "\n")
cat("Total deaths:         ", sum(rj_mortality$cases), "\n")


## ---- 2. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_rj <- treespatial_scan(
    cases       = rj_mortality$cases,
    population  = rj_mortality$live_births,
    region_id   = rj_mortality$region_id,
    x           = rj_mortality$x,
    y           = rj_mortality$y,
    node_id     = rj_mortality$node_id,
    tree        = rj_tree,
    max_pop_pct = 0.50,
    nsim        = 999, seed = 2016,
    n_cores     = 4L
  )
})
print(result_rj)


## ---- 3. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_rj)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 4. Sequential scan (Zhang, Assuncao & Kulldorff, 2010) ----
cat("\n=== Sequential scan (Zhang et al. 2010) ===\n")
seq_rj <- sequential_scan(
  cases       = rj_mortality$cases,
  population  = rj_mortality$live_births,
  region_id   = rj_mortality$region_id,
  x           = rj_mortality$x,
  y           = rj_mortality$y,
  node_id     = rj_mortality$node_id,
  tree        = rj_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2016,
  max_pop_pct = 0.50, n_cores = 4L
)
print(seq_rj)


## ---- 5. Polygons + cluster membership ----
cat("\nDownloading RJ municipal boundaries...\n")
mun <- read_municipality(code_muni = "RJ", year = 2016)
mun$code6 <- as.integer(substr(mun$code_muni, 1, 6))

region_info <- unique(rj_mortality[, c("region_id", "ibge_code", "name")])

cr1    <- merge(get_cluster_regions(result_rj, n_clusters = 1, overlap = FALSE),
                region_info, by = "region_id")
cr2    <- merge(get_cluster_regions(result_rj, n_clusters = 2, overlap = TRUE),
                region_info, by = "region_id")
cr_seq <- merge(get_cluster_regions(seq_rj, overlap = TRUE),
                region_info, by = "region_id")


## ---- 6. Plot 1: Most likely cluster ----
mun1 <- merge(mun, cr1, by.x = "code6", by.y = "ibge_code", all.x = TRUE)
mlc  <- result_rj$most_likely_cluster

p1 <- ggplot(mun1) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.15, alpha = 0.75) +
  scale_fill_manual(
    values   = c("1" = "#C44E52"),
    na.value = "gray95",
    labels   = c("1" = paste0(mlc$node_id, " (", length(mlc$region_ids),
                              " municipalities)")),
    name     = "Cluster"
  ) +
  labs(
    title    = paste0("Tree-Spatial Scan: most likely cluster (", mlc$node_id, ")"),
    subtitle = paste0("LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_rj$pvalue, digits = 3),
                      " | ", mlc$cases, " cases (expected ",
                      round(mlc$expected, 1), ")")
  ) +
  theme_minimal(base_size = 12) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 8))

ggsave("rj_cluster_mlc.png", p1, width = 9, height = 8, dpi = 300)
cat("Saved: rj_cluster_mlc.png\n")


## ---- 7. Plot 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
mun2 <- merge(mun, cr2, by.x = "code6", by.y = "ibge_code")
palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")

p2 <- ggplot(mun2) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(values = palette2, na.value = "gray95",
                    name = "Cluster", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  labs(title    = "Tree-Spatial Scan: Top 2 Distinct Clusters",
       subtitle = "Cancado et al. (2025), Sec. 5.1.1") +
  theme_minimal(base_size = 11) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        strip.text    = element_text(face = "bold", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 7),
        legend.position = "none")

ggsave("rj_clusters_top2.png", p2, width = 14, height = 6, dpi = 300)
cat("Saved: rj_clusters_top2.png\n")


## ---- 8. Plot 3: Sequential scan clusters ----
n_iter <- seq_rj$n_iter
if (n_iter > 0) {
  # Cross-join the map polygons with the panels of the sequential scan,
  # so that every municipality appears in every panel (those that are
  # outside the dataset of 89 municipalities show up as NA-coloured).
  panels <- unique(cr_seq$panel)
  cr_seq_keys <- unique(cr_seq[, c("ibge_code", "cluster", "node_id",
                                    "llr", "pvalue", "panel")])
  mun_panels <- merge(
    do.call(rbind,
            lapply(panels, function(p) cbind(mun, panel = p))),
    cr_seq_keys,
    by.x = c("code6", "panel"), by.y = c("ibge_code", "panel"),
    all.x = TRUE
  )

  palette_seq <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_seq) <- as.character(seq_len(n_iter))

  n_sig <- sum(seq_rj$clusters$significant, na.rm = TRUE)
  p_seq <- ggplot(mun_panels) +
    geom_sf(aes(fill = factor(cluster)), color = "gray40",
            linewidth = 0.12, alpha = 0.75) +
    scale_fill_manual(values = palette_seq, na.value = "gray95",
                      name = "Iteration", na.translate = FALSE) +
    facet_wrap(~ panel, nrow = 1) +
    labs(title    = paste0("Sequential Scan (Zhang et al. 2010): ",
                            n_iter, " iterations, ",
                            n_sig, " significant"),
         subtitle = "Each panel = one iteration (regions removed before next).") +
    theme_minimal(base_size = 11) +
    theme(plot.title    = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray40"),
          strip.text    = element_text(face = "bold", size = 9),
          axis.title    = element_blank(),
          axis.text     = element_text(color = "gray50", size = 7),
          legend.position = "none")

  ggsave("rj_clusters_sequential.png", p_seq,
         width = max(8, 4 * n_iter), height = 6, dpi = 300)
  cat("Saved: rj_clusters_sequential.png (", n_iter, " iterations)\n")
}


## ---- 9. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "(ICD-10)\n")
cat("  Municipalities:", length(mlc$region_ids), "\n")
cat("  Cases:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  Relative risk:", round(mlc$rr, 2), "\n")
cat("  LR:", round(mlc$llr, 2), "  p-value:", result_rj$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Sequential scan (Zhang et al. 2010): ", n_iter, " iterations, ",
    sum(seq_rj$clusters$significant, na.rm = TRUE),
    " significant\n", sep = "")
cat("\nDone!\n")

##=============================================================================
## Example 1: Infant Mortality in Rio de Janeiro, Brazil (2016)
##
## Reproduces Section 5.2 of:
##   Cancado et al. (2025). Environmental and Ecological Statistics, 32, 953-978.
##
## Two secondary-cluster analyses are shown:
##   1. The PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025,
##      Sec. 5.1.1): single scan + filter_clusters /
##      get_cluster_regions(n_clusters = N).
##   2. sequential_scan() -- Zhang, Assuncao & Kulldorff (2010).
##=============================================================================

options(bitmapType = "cairo")     # robust PNG rendering on Linux servers

library(treeSS)
library(ggplot2)
library(sf)


## ---- 1. Load data ----
data(rj_mortality)
data(rj_tree)

cat("=== Rio de Janeiro Infant Mortality 2016 ===\n")
cat("Rows (long):          ", nrow(rj_mortality), "\n")
cat("Unique municipalities:", length(unique(rj_mortality$region_id)), "\n")
cat("Tree nodes:           ", nrow(rj_tree), "\n")
cat("Total deaths:         ", sum(rj_mortality$cases), "\n")


## ---- 2. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_rj <- treespatial_scan(
    rj_mortality,
    cases       = cases,
    population  = live_births,
    region_id   = region_id,
    x           = x,
    y           = y,
    node_id     = node_id,
    tree        = rj_tree,
    max_pop_pct = 0.50,
    nsim        = 999, seed = 2016,
    n_cores     = 4L
  )
})
print(result_rj)


## ---- 3. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_rj)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 4. Sequential scan (Zhang, Assuncao & Kulldorff, 2010) ----
cat("\n=== Sequential scan (Zhang et al. 2010) ===\n")
seq_rj <- sequential_scan(
  rj_mortality,
  cases       = cases,
  population  = live_births,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = rj_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2016,
  max_pop_pct = 0.50, n_cores = 4L
)
print(seq_rj)


## ---- 5. Polygons + cluster membership ----
## RJ municipal boundaries ship with the package (IBGE Malhas Municipais),
## so no download is needed -- mirrors chicago_map / london_boroughs_map.
data(rj_map)
mun <- rj_map        # columns: ibge_code (6-digit), NAME, geometry

region_info <- unique(rj_mortality[, c("region_id", "ibge_code", "name")])

cr1    <- merge(get_cluster_regions(result_rj, n_clusters = 1, overlap = FALSE),
                region_info, by = "region_id")
cr2    <- merge(get_cluster_regions(result_rj, n_clusters = 2, overlap = TRUE),
                region_info, by = "region_id")
cr_seq <- merge(get_cluster_regions(seq_rj, overlap = TRUE),
                region_info, by = "region_id")


## ---- 6. Plot 1: Most likely cluster ----
mun1 <- merge(mun, cr1, by = "ibge_code", all.x = TRUE)
mlc  <- result_rj$most_likely_cluster

p1 <- ggplot(mun1) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.15, alpha = 0.75) +
  scale_fill_manual(
    values   = c("1" = "#C44E52"),
    na.value = "gray95",
    labels   = c("1" = paste0(mlc$node_id, " (", length(mlc$region_ids),
                              " municipalities)")),
    name     = "Cluster"
  ) +
  labs(
    title    = paste0("Tree-Spatial Scan: most likely cluster (", mlc$node_id, ")"),
    subtitle = paste0("LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_rj$pvalue, digits = 3),
                      " | ", mlc$cases, " cases (expected ",
                      round(mlc$expected, 1), ")")
  ) +
  theme_minimal(base_size = 12) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 8))

ggsave("rj_cluster_mlc.png", p1, width = 9, height = 8, dpi = 300)
cat("Saved: rj_cluster_mlc.png\n")


## ---- 7. Plot 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
mun2 <- merge(mun, cr2, by = "ibge_code")
palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")

p2 <- ggplot(mun2) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(values = palette2, na.value = "gray95",
                    name = "Cluster", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  labs(title    = "Tree-Spatial Scan: Top 2 Distinct Clusters",
       subtitle = "Cancado et al. (2025), Sec. 5.1.1") +
  theme_minimal(base_size = 11) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        strip.text    = element_text(face = "bold", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 7),
        legend.position = "none")

ggsave("rj_clusters_top2.png", p2, width = 14, height = 6, dpi = 300)
cat("Saved: rj_clusters_top2.png\n")


## ---- 8. Plot 3: Sequential scan clusters ----
n_iter <- seq_rj$n_iter
if (n_iter > 0) {
  # Cross-join the map polygons with the panels of the sequential scan,
  # so that every municipality appears in every panel (those that are
  # outside the dataset of 89 municipalities show up as NA-coloured).
  panels <- unique(cr_seq$panel)
  cr_seq_keys <- unique(cr_seq[, c("ibge_code", "cluster", "node_id",
                                    "llr", "pvalue", "panel")])
  mun_panels <- merge(
    do.call(rbind,
            lapply(panels, function(p) cbind(mun, panel = p))),
    cr_seq_keys,
    by = c("ibge_code", "panel"),
    all.x = TRUE
  )

  palette_seq <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_seq) <- as.character(seq_len(n_iter))

  n_sig <- sum(seq_rj$clusters$significant, na.rm = TRUE)
  p_seq <- ggplot(mun_panels) +
    geom_sf(aes(fill = factor(cluster)), color = "gray40",
            linewidth = 0.12, alpha = 0.75) +
    scale_fill_manual(values = palette_seq, na.value = "gray95",
                      name = "Iteration", na.translate = FALSE) +
    facet_wrap(~ panel, nrow = 1) +
    labs(title    = paste0("Sequential Scan (Zhang et al. 2010): ",
                            n_iter, " iterations, ",
                            n_sig, " significant"),
         subtitle = "Each panel = one iteration (regions removed before next).") +
    theme_minimal(base_size = 11) +
    theme(plot.title    = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray40"),
          strip.text    = element_text(face = "bold", size = 9),
          axis.title    = element_blank(),
          axis.text     = element_text(color = "gray50", size = 7),
          legend.position = "none")

  ggsave("rj_clusters_sequential.png", p_seq,
         width = max(8, 4 * n_iter), height = 6, dpi = 300)
  cat("Saved: rj_clusters_sequential.png (", n_iter, " iterations)\n")
}


## ---- 9. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "(ICD-10)\n")
cat("  Municipalities:", length(mlc$region_ids), "\n")
cat("  Cases:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  Relative risk:", round(mlc$rr, 2), "\n")
cat("  LR:", round(mlc$llr, 2), "  p-value:", result_rj$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Sequential scan (Zhang et al. 2010): ", n_iter, " iterations, ",
    sum(seq_rj$clusters$significant, na.rm = TRUE),
    " significant\n", sep = "")
cat("\nDone!\n")

# CSR format conversion helpers for the C++ backend

#' @keywords internal
.zones_to_csr <- function(zones) {
  # Convert zones list to CSR (compressed sparse row) format for C++
  # Returns: zone_region_idx (0-based), zone_ptr, zone_pop
  n_zones <- length(zones)

  lens <- vapply(zones, function(z) length(z$region_idx), integer(1))
  zone_ptr <- c(0L, cumsum(lens))

  if (n_zones > 0L && sum(lens) > 0L) {
    all_idx <- unlist(lapply(zones, `[[`, "region_idx"),
                      use.names = FALSE) - 1L  # 0-based for C++
  } else {
    all_idx <- integer(0)
  }

  zone_pop <- vapply(zones, function(z) as.numeric(z$population), numeric(1))

  list(
    zone_region_idx = as.integer(all_idx),
    zone_ptr = as.integer(zone_ptr),
    zone_pop = as.numeric(zone_pop)
  )
}

#' @keywords internal
.tree_to_csr_children <- function(tree) {
  # Convert tree children to CSR format for C++
  # Returns: children_idx (0-based), children_ptr
  all_nodes <- tree$node_id
  n_nodes <- length(all_nodes)

  # Row position of each node's parent within all_nodes (NA for the root and
  # for any parent_id not present in node_id, matching the previous
  # behaviour of dropping such rows).
  parent_pos <- match(tree$parent_id, all_nodes)
  child_rows <- which(!is.na(parent_pos))

  if (length(child_rows) > 0L) {
    # Group children by parent position; within a parent, keep increasing
    # row order (same order the previous per-node which() scan produced).
    ord <- order(parent_pos[child_rows], child_rows)
    children_sorted <- child_rows[ord]
    counts <- tabulate(parent_pos[child_rows], nbins = n_nodes)
  } else {
    children_sorted <- integer(0)
    counts <- integer(n_nodes)
  }

  children_ptr <- c(0L, cumsum(counts))

  list(
    children_idx = as.integer(children_sorted - 1L),  # 0-based for C++
    children_ptr = as.integer(children_ptr)
  )
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>",
  eval = FALSE
)

## ----rj-data------------------------------------------------------------------
#  library(treeSS)
#  data(rj_mortality)
#  data(rj_tree)
#  
#  c(rows           = nrow(rj_mortality),
#    municipalities = length(unique(rj_mortality$region_id)),
#    tree_nodes     = nrow(rj_tree),
#    total_deaths   = sum(rj_mortality$cases),
#    total_births   = sum(unique(
#      rj_mortality[, c("region_id", "live_births")]
#    )$live_births))
#  #> rows           municipalities  tree_nodes  total_deaths  total_births
#  #> 1440           89              622         2981          219124

## ----rj-scan------------------------------------------------------------------
#  result_rj <- treespatial_scan(
#    rj_mortality,
#    cases       = cases,
#    population  = live_births,
#    region_id   = region_id,
#    x           = x,
#    y           = y,
#    node_id     = node_id,
#    tree        = rj_tree,
#    max_pop_pct = 0.50,
#    nsim        = 999, seed = 2016,
#    n_cores     = 4L
#  )
#  print(result_rj)

## ----rj-spatial-only----------------------------------------------------------
#  rj_agg <- unique(rj_mortality[, c("region_id", "x", "y", "live_births")])
#  rj_agg$cases <- tapply(rj_mortality$cases,
#                          rj_mortality$region_id,
#                          sum)[as.character(rj_agg$region_id)]
#  
#  result_spatial <- circular_scan(
#    rj_agg,
#    cases       = cases,
#    population  = live_births,
#    region_id   = region_id,
#    x           = x,
#    y           = y,
#    max_pop_pct = 0.50,
#    nsim        = 999, seed = 2016
#  )
#  result_spatial$most_likely_cluster$region_ids

## ----rj-filter----------------------------------------------------------------
#  filter_clusters(result_rj)[1:5,
#                              c("node_id", "n_regions",
#                                "cases", "expected", "llr")]

## ----rj-getregions------------------------------------------------------------
#  # Most likely cluster only (single map)
#  cr1 <- get_cluster_regions(result_rj, n_clusters = 1, overlap = FALSE)
#  
#  # Top-2 distinct clusters (faceted map)
#  cr2 <- get_cluster_regions(result_rj, n_clusters = 2, overlap = TRUE)

## ----rj-plot------------------------------------------------------------------
#  library(ggplot2)
#  library(sf)
#  
#  data(rj_map)   # sf polygons: ibge_code, NAME, geometry
#  
#  cr2 <- merge(get_cluster_regions(result_rj, n_clusters = 2,
#                                    overlap = TRUE),
#               unique(rj_mortality[, c("region_id", "ibge_code")]),
#               by = "region_id")
#  mun_facet <- merge(rj_map, cr2, by = "ibge_code")
#  
#  ggplot(mun_facet) +
#    geom_sf(aes(fill = factor(cluster)), color = "gray40",
#            alpha = 0.75) +
#    scale_fill_manual(values = c("1" = "#C44E52", "2" = "#4C72B0"),
#                      na.value = "gray95", na.translate = FALSE) +
#    facet_wrap(~ panel, nrow = 1) +
#    theme_minimal() +
#    theme(legend.position = "none")

## ----rj-seq-------------------------------------------------------------------
#  seq_rj <- sequential_scan(
#    rj_mortality,
#    cases       = cases,
#    population  = live_births,
#    region_id   = region_id,
#    x           = x,
#    y           = y,
#    node_id     = node_id,
#    tree        = rj_tree,
#    max_iter    = 5L, alpha = 0.05,
#    nsim        = 999, seed = 2016,
#    max_pop_pct = 0.50, n_cores = 4L
#  )
#  print(seq_rj)

## ----rj-seq-table-------------------------------------------------------------
#  seq_rj$clusters[, c("iteration", "node_id", "n_regions",
#                      "llr", "pvalue", "significant")]

#' Filter Overlapping Secondary Clusters
#'
#' Removes overlapping secondary clusters from the output of
#' \code{\link{treespatial_scan}}, \code{\link{circular_scan}}, or
#' \code{\link{tree_scan}}.
#'
#' @param result An object of class \code{"treespatial_scan"},
#'   \code{"circular_scan"}, or \code{"tree_scan"}.
#' @param alpha Numeric. Significance level for filtering. Default is
#'   \code{NULL}, which uses the alpha from the original analysis.
#'
#' @return A \code{data.frame} of distinct significant clusters ordered by
#'   descending LLR.
#'
#' @details
#' For \code{treespatial_scan} objects, two clusters overlap if they have
#' BOTH tree overlap (one node is ancestor/descendant of the other) AND
#' spatial overlap (same center or identical set of regions). This follows
#' Cancado et al. (2025).
#'
#' For \code{circular_scan} objects, the criterion follows Kulldorff (1997):
#' a secondary cluster is distinct if its center is not contained in any
#' previously retained cluster's zone, and no previously retained cluster's
#' center is contained in the candidate zone.
#'
#' For \code{tree_scan} objects, a secondary cluster is distinct if its node
#' is NOT an ancestor or descendant of any previously retained node. This
#' follows Kulldorff et al. (2003).
#'
#' When a dominant signal saturates the candidate pool (making distinct
#' secondary clusters hard to find), or when clusters shadow each other,
#' consider using \code{\link{sequential_scan}} (Zhang, Assuncao and
#' Kulldorff, 2010), which re-runs the scan after removing each detected
#' cluster.
#'
#' @references
#' Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
#' Statistics - Theory and Methods}, 26(6), 1481-1496.
#'
#' Kulldorff, M., Fang, Z., & Walsh, S. J. (2003). A tree-based scan
#' statistic for database disease surveillance. \emph{Biometrics}, 59(2),
#' 323-331.
#'
#' Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
#' (2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
#' Statistics}, 32, 953-978. \doi{10.1007/s10651-025-00670-w}
#'
#' @seealso \code{\link{treespatial_scan}}, \code{\link{circular_scan}},
#'   \code{\link{tree_scan}}, \code{\link{sequential_scan}}
#'
#' @examples
#' data(london_collisions); data(london_tree)
#' result <- treespatial_scan(
#'   london_collisions,
#'   cases       = cases,
#'   population  = population,
#'   region_id   = region_id,
#'   x           = x,
#'   y           = y,
#'   node_id     = node_id,
#'   tree        = london_tree,
#'   nsim        = 99, seed = 42
#' )
#' filter_clusters(result)
#'
#' @export
filter_clusters <- function(result, alpha = NULL) {

  is_treespatial <- inherits(result, "treespatial_scan")
  is_circular <- inherits(result, "circular_scan")
  is_tree <- inherits(result, "tree_scan")

  if (!is_treespatial && !is_circular && !is_tree) {
    stop("'result' must be of class 'treespatial_scan', 'circular_scan', ",
         "or 'tree_scan'.", call. = FALSE)
  }

  if (is.null(alpha)) alpha <- result$alpha

  # --- tree_scan: use all_cuts (p-values already computed) ---
  if (is_tree) {
    sig <- result$all_cuts[result$all_cuts$pvalue < alpha, ]
    if (nrow(sig) == 0) {
      message("No significant clusters found at alpha = ", alpha, ".")
      return(data.frame())
    }
    sig <- sig[order(-sig$llr), ]
    return(.filter_tree_only(sig, result$tree))
  }

  # --- circular_scan / treespatial_scan: use secondary_clusters ---
  all_pairs <- result$secondary_clusters
  if (is.null(all_pairs) || nrow(all_pairs) == 0) {
    message("No secondary clusters to filter.")
    return(data.frame())
  }

  # Compute p-values (vectorized)
  sim_llr <- result$simulated_llr
  nsim <- result$nsim
  sim_sorted <- sort(sim_llr)
  n_geq <- nsim - findInterval(all_pairs$llr, sim_sorted, left.open = TRUE)
  all_pairs$pvalue <- (n_geq + 1) / (nsim + 1)

  sig_pairs <- all_pairs[all_pairs$pvalue < alpha, ]
  if (nrow(sig_pairs) == 0) {
    message("No significant clusters found at alpha = ", alpha, ".")
    return(data.frame())
  }
  sig_pairs <- sig_pairs[order(-sig_pairs$llr), ]

  # Pre-compute zone regions
  zones <- build_zones(result$regions,
                       max_pop = result$total_population * 0.5)
  pair_regions <- vector("list", nrow(sig_pairs))
  for (i in seq_len(nrow(sig_pairs))) {
    zi <- .find_zone(zones, sig_pairs$center[i], sig_pairs$n_regions[i])
    if (!is.null(zi)) {
      pair_regions[[i]] <- zones[[zi]]$region_idx
    } else {
      pair_regions[[i]] <- integer(0)
    }
  }

  if (is_circular) {
    .filter_circular(sig_pairs, pair_regions)
  } else {
    .filter_treespatial(sig_pairs, pair_regions, result$tree)
  }
}


# ---- Tree-only criterion (Kulldorff et al. 2003) ----
# Distinct if NOT ancestor/descendant of any retained node.
.filter_tree_only <- function(sig_cuts, tree) {

  all_descendants <- list()
  for (nd in tree$node_id) {
    all_descendants[[nd]] <- .get_descendants(tree, nd)
  }

  retained <- list(sig_cuts[1, ])

  if (nrow(sig_cuts) > 1) {
    for (i in 2:nrow(sig_cuts)) {
      cand_node <- sig_cuts$node_id[i]
      cand_desc <- all_descendants[[cand_node]]

      is_distinct <- TRUE
      for (j in seq_along(retained)) {
        kept_node <- retained[[j]]$node_id
        kept_desc <- all_descendants[[kept_node]]

        if (cand_node %in% kept_desc || kept_node %in% cand_desc) {
          is_distinct <- FALSE
          break
        }
      }

      if (is_distinct) {
        retained[[length(retained) + 1]] <- sig_cuts[i, ]
      }
    }
  }

  do.call(rbind, retained)
}


# ---- Kulldorff (1997) criterion for circular scan ----
.filter_circular <- function(sig_pairs, pair_regions) {

  retained <- list(sig_pairs[1, ])
  retained_regions <- list(pair_regions[[1]])
  retained_centers <- sig_pairs$center[1]

  if (nrow(sig_pairs) > 1) {
    for (i in 2:nrow(sig_pairs)) {
      cand_center <- sig_pairs$center[i]
      cand_regs <- pair_regions[[i]]

      is_distinct <- TRUE
      for (j in seq_along(retained)) {
        if (cand_center %in% retained_regions[[j]] ||
            retained_centers[j] %in% cand_regs) {
          is_distinct <- FALSE
          break
        }
      }

      if (is_distinct) {
        retained[[length(retained) + 1]] <- sig_pairs[i, ]
        retained_regions[[length(retained_regions) + 1]] <- cand_regs
        retained_centers <- c(retained_centers, cand_center)
      }
    }
  }

  do.call(rbind, retained)
}


# ---- Tree-spatial overlap criterion (Cancado et al. 2025, Sec 5.1.1) ----
# A candidate (z, g) is REJECTED as redundant if:
#   * the candidate's branch g overlaps with the retained branch (one is
#     ancestor/descendant of the other in the tree), AND
#   * the candidate's spatial zone intersects the retained zone (any
#     region in common).
# Equivalently, a candidate is RETAINED as distinct if at least one of:
#   * branches are disjoint (no ancestor/descendant relation), OR
#   * spatial zones are disjoint (no region in common).
# This matches the three scenarios listed in Sec. 5.1.1 of Cancado et al.
.filter_treespatial <- function(sig_pairs, pair_regions, tree) {

  all_descendants <- list()
  for (nd in tree$node_id) {
    all_descendants[[nd]] <- .get_descendants(tree, nd)
  }

  retained <- list(sig_pairs[1, ])
  retained_regions <- list(pair_regions[[1]])

  if (nrow(sig_pairs) > 1) {
    for (i in 2:nrow(sig_pairs)) {
      candidate <- sig_pairs[i, ]
      cand_node <- candidate$node_id
      cand_desc <- all_descendants[[cand_node]]
      cand_regs <- pair_regions[[i]]

      is_distinct <- TRUE
      for (j in seq_along(retained)) {
        kept <- retained[[j]]
        kept_node <- kept$node_id
        kept_desc <- all_descendants[[kept_node]]
        kept_regs <- retained_regions[[j]]

        # Branch overlap: same node OR ancestor/descendant
        node_overlap <- (cand_node == kept_node) ||
                         (cand_node %in% kept_desc) ||
                         (kept_node %in% cand_desc)
        # Spatial overlap: any region in common
        spatial_overlap <- length(intersect(cand_regs, kept_regs)) > 0

        if (node_overlap && spatial_overlap) {
          is_distinct <- FALSE
          break
        }
      }

      if (is_distinct) {
        retained[[length(retained) + 1]] <- candidate
        retained_regions[[length(retained_regions) + 1]] <- cand_regs
      }
    }
  }

  do.call(rbind, retained)
}

#' Aggregate Case Counts from Leaves to All Nodes in a Hierarchical Tree
#'
#' Given case counts at the leaf level (supplied as columns of a long-format
#' \code{data} data.frame) and a tree, aggregates case counts upward from
#' child nodes to parent nodes, producing a complete case matrix indexed by
#' all nodes (rows) and regions (columns).
#'
#' @param data A \code{data.frame} in long format with one row per
#'   (region, leaf) observation. The \code{cases}, \code{region_id} and
#'   \code{node_id} arguments name columns of this data.frame.
#' @param cases Column of \code{data} giving case counts. Given as an
#'   unquoted column name (a string or expression also works).
#' @param region_id Column of \code{data} giving the region identifier.
#' @param node_id Column of \code{data} giving the leaf identifier.
#' @param tree A \code{data.frame} with columns \code{node_id} and
#'   \code{parent_id}. As an alternative, pass \code{tree_node_id} and
#'   \code{tree_parent_id}.
#' @param tree_node_id,tree_parent_id Optional. Parallel vectors as an
#'   alternative to \code{tree}.
#'
#' @return A matrix of dimensions \eqn{m \times k} (nodes x regions), with
#'   rows ordered by \code{tree$node_id} and columns by region.
#'
#' @details
#' This function is exposed for inspection and pedagogical use; the scan
#' functions call it internally on the matrix they build from your input
#' data.
#'
#' @references
#' Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
#' (2025). A tree-spatial scan statistic.
#' \emph{Environmental and Ecological Statistics}, 32, 953-978.
#' \doi{10.1007/s10651-025-00670-w}
#'
#' @export
#' @examples
#' tree <- data.frame(
#'   node_id   = c(1, 2, 3, 4, 5, 6, 7),
#'   parent_id = c(NA, 1, 1, 2, 2, 3, 3)
#' )
#' # Leaves are 4, 5, 6, 7; long-format data.frame
#' dat <- data.frame(
#'   cases     = c(10, 5, 3, 8,  2, 7, 4, 1,  6, 3, 9, 2),
#'   region_id = rep(1:3, each = 4),
#'   node_id   = rep(c(4, 5, 6, 7), times = 3)
#' )
#' aggregate_tree(dat, cases = cases, region_id = region_id,
#'                node_id = node_id, tree = tree)
aggregate_tree <- function(data, cases, region_id, node_id,
                           tree = NULL,
                           tree_node_id = NULL, tree_parent_id = NULL) {

  ## --- resolve data/column inputs (substitutive interface, treeSS >= 0.2.0) ---
  if (missing(data)) {
    stop("`data` is required: a data.frame with one row per (region, leaf).",
         call. = FALSE)
  }
  data <- as.data.frame(data)
  .env <- parent.frame()
  cases     <- .resolve_col(substitute(cases),     data, .env, "cases")
  region_id <- .resolve_col(substitute(region_id), data, .env, "region_id")
  node_id   <- .resolve_col(substitute(node_id),   data, .env, "node_id")

  tree <- .normalize_tree(tree, tree_node_id, tree_parent_id)
  .validate_tree(tree)

  if (length(cases) != length(region_id) ||
      length(cases) != length(node_id)) {
    stop("'cases', 'region_id', 'node_id' must have the same length.",
         call. = FALSE)
  }
  if (any(is.na(cases))) stop("'cases' contains NA values.", call. = FALSE)
  if (any(cases < 0))    stop("'cases' must be non-negative.", call. = FALSE)

  leaves <- .get_leaves(tree)
  reg_ids <- unique(region_id)
  n_leaves <- length(leaves)
  n_regions <- length(reg_ids)

  tree_leaves <- leaves
  if (is.character(tree$node_id) || is.character(node_id)) {
    node_id     <- as.character(node_id)
    tree_leaves <- as.character(tree_leaves)
  }
  reg_ids_chr_check <- is.character(reg_ids) || is.character(region_id)
  if (reg_ids_chr_check) {
    region_id <- as.character(region_id)
    reg_ids   <- as.character(reg_ids)
  }

  unknown_nodes <- setdiff(unique(node_id), tree_leaves)
  if (length(unknown_nodes) > 0) {
    stop("'node_id' contains values that are not leaves of the tree: ",
         paste(utils::head(unknown_nodes, 5), collapse = ", "),
         if (length(unknown_nodes) > 5) ", ..." else "",
         ".", call. = FALSE)
  }

  row_idx <- match(node_id, tree_leaves)
  col_idx <- match(region_id, reg_ids)

  cases_mat <- matrix(0, nrow = n_leaves, ncol = n_regions)
  rownames(cases_mat) <- as.character(tree_leaves)
  colnames(cases_mat) <- as.character(reg_ids)

  keep <- cases > 0
  if (any(keep)) {
    key <- paste(row_idx[keep], col_idx[keep], sep = "_")
    if (anyDuplicated(key)) {
      agg <- stats::aggregate(cases[keep],
                              by = list(r = row_idx[keep],
                                        c = col_idx[keep]),
                              FUN = sum)
      cases_mat[cbind(agg$r, agg$c)] <- agg$x
    } else {
      cases_mat[cbind(row_idx[keep], col_idx[keep])] <- cases[keep]
    }
  }

  .aggregate_leaves_to_all(cases_mat, tree)
}

test_that("aggregate_tree sums correctly", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5, 6, 7),
    parent_id = c(NA, 1, 1, 2, 2, 3, 3)
  )
  dat <- data.frame(
    cases     = c(10, 5, 3, 8,  2, 7, 4, 1,  6, 3, 9, 2),
    region_id = rep(1:3, each = 4),
    node_id   = rep(c(4, 5, 6, 7), times = 3)
  )
  full <- aggregate_tree(dat, cases, region_id, node_id, tree = tree)

  expect_equal(nrow(full), 7)
  expect_equal(ncol(full), 3)

  leaf4 <- which(rownames(full) == "4")
  leaf5 <- which(rownames(full) == "5")
  node2 <- which(rownames(full) == "2")
  expect_equal(full[node2, ], full[leaf4, ] + full[leaf5, ])

  root <- which(rownames(full) == "1")
  per_region <- c(10 + 5 + 3 + 8,
                   2 + 7 + 4 + 1,
                   6 + 3 + 9 + 2)
  expect_equal(unname(full[root, ]), per_region)
})

test_that("aggregate_tree errors on node_id that is not a leaf", {
  tree <- data.frame(node_id = c(1, 2, 3), parent_id = c(NA, 1, 1))
  dat <- data.frame(cases = 1:3, region_id = 1:3, node_id = c(2, 3, 99))
  expect_error(
    aggregate_tree(dat, cases, region_id, node_id, tree = tree),
    "not leaves"
  )
})

test_that("aggregate_tree accepts tree_node_id/tree_parent_id form", {
  dat <- data.frame(
    cases     = c(10, 5, 3, 8),
    region_id = rep(1:2, each = 2),
    node_id   = c(2, 3, 2, 3)
  )
  res1 <- aggregate_tree(dat, cases, region_id, node_id,
                         tree = data.frame(node_id = c(1, 2, 3),
                                           parent_id = c(NA, 1, 1)))
  res2 <- aggregate_tree(dat, cases, region_id, node_id,
                         tree_node_id = c(1, 2, 3),
                         tree_parent_id = c(NA, 1, 1))
  expect_equal(res1, res2)
})

#' Tree-Based Scan Statistic
#'
#' Performs the tree-based scan statistic for detecting clusters in
#' hierarchical data. Uses a Poisson or binomial model with Monte Carlo
#' simulation (implemented in C++ via Rcpp) for significance testing.
#'
#' Inputs are supplied through a single \code{data} data.frame with one row
#' per leaf (rows are matched to the tree by \code{node_id}, so they need
#' not be pre-ordered); counts are summed within leaf.
#'
#' @param data A \code{data.frame} with one row per leaf node. The
#'   \code{cases}, \code{node_id} and (optional) \code{population} arguments
#'   name columns of this data.frame.
#' @param cases Column of \code{data} giving case counts at the leaf level.
#'   Given as an unquoted column name (a string or expression also works).
#' @param node_id Column of \code{data} giving the leaf identifier for each
#'   row. Each value must match a leaf of the tree.
#' @param tree A \code{data.frame} with columns \code{node_id} and
#'   \code{parent_id}. Root node(s) must have \code{parent_id = NA}.
#'   Alternatively, pass the tree as parallel vectors via
#'   \code{tree_node_id} and \code{tree_parent_id}.
#' @param population Optional column of \code{data} giving the population at
#'   the leaf level. For the binomial model, \code{population} is the number
#'   of trials (cases + controls) per leaf and is required. For the
#'   Poisson model, defaults to \code{1} per leaf when omitted.
#' @param tree_node_id,tree_parent_id Optional parallel vectors describing
#'   the tree as an alternative to the \code{tree} data.frame. Both must
#'   have the same length, and the root node(s) must have
#'   \code{tree_parent_id = NA}. Ignored when \code{tree} is supplied.
#' @param nsim Integer. Number of Monte Carlo simulations. Default is
#'   \code{999}.
#' @param alpha Numeric. Significance level. Default is \code{0.05}.
#' @param model Character. Likelihood model: either \code{"poisson"}
#'   (default) or \code{"binomial"}.
#' @param seed Integer or \code{NULL}. Random seed for the Monte
#'   Carlo loop. When non-\code{NULL}, the user's pre-existing RNG
#'   state is saved on entry and restored on exit, so the seed
#'   argument affects only this call and does not leak into
#'   subsequent draws in the user's session.
#' @param n_cores Integer. Number of OpenMP threads for the Monte Carlo
#'   loop. Default is \code{1L} (serial). Set higher to parallelize.
#'
#' @return An object of class \code{"tree_scan"} (see package help for
#'   details).
#'
#' @references
#' Kulldorff, M., Fang, Z., & Walsh, S. J. (2003). A tree-based scan
#' statistic for database disease surveillance. \emph{Biometrics}, 59(2),
#' 323–331.
#'
#' @seealso \code{\link{circular_scan}}, \code{\link{treespatial_scan}},
#'   \code{\link{aggregate_tree}}
#'
#' @export
#' @examples
#' tree <- data.frame(
#'   node_id   = c(1, 2, 3, 4, 5, 6, 7, 8),
#'   parent_id = c(NA, 1, 1, 2, 2, 3, 3, 3)
#' )
#' # One row per leaf (leaves are 4, 5, 6, 7, 8)
#' leaf_data <- data.frame(
#'   node_id = c(4, 5, 6, 7, 8),
#'   cases   = c(50, 5, 3, 2, 4),
#'   pop     = c(100, 100, 100, 100, 100)
#' )
#'
#' result <- tree_scan(leaf_data, cases = cases, node_id = node_id,
#'                     tree = tree, population = pop, nsim = 99)
#' print(result)
tree_scan <- function(data, cases, node_id, tree = NULL, population = NULL,
                      tree_node_id = NULL, tree_parent_id = NULL,
                      nsim = 999L, alpha = 0.05,
                      model = c("poisson", "binomial"),
                      seed = NULL, n_cores = 1L) {

  ## --- resolve data/column inputs (substitutive interface, treeSS >= 0.2.0) ---
  if (missing(data)) {
    stop("`data` is required: a data.frame with one row per leaf.",
         call. = FALSE)
  }
  data <- as.data.frame(data)
  .env <- parent.frame()
  .cases_in <- .resolve_col(substitute(cases),   data, .env, "cases")
  .node_in  <- .resolve_col(substitute(node_id), data, .env, "node_id")
  .q_pop    <- substitute(population)
  .pop_in   <- if (is.null(.q_pop)) NULL else
    .resolve_col(.q_pop, data, .env, "population")

  model <- match.arg(model)
  model_int <- if (model == "binomial") 1L else 0L

  tree <- .normalize_tree(tree, tree_node_id, tree_parent_id)
  .validate_tree(tree)

  leaves <- .get_leaves(tree)
  n_leaves <- length(leaves)

  ## Match the leaf rows of `data` onto the canonical leaf order of the
  ## tree. Cases are summed within leaf; the denominator is taken from the
  ## first occurrence per leaf.
  node_chr   <- as.character(.node_in)
  leaves_chr <- as.character(leaves)
  unknown <- setdiff(unique(node_chr), leaves_chr)
  if (length(unknown) > 0) {
    stop("'node_id' contains values that are not leaves of the tree: ",
         paste(utils::head(unknown, 5), collapse = ", "),
         if (length(unknown) > 5) ", ..." else "", ".", call. = FALSE)
  }
  if (any(is.na(.cases_in))) stop("'cases' contains NA values.", call. = FALSE)
  if (any(.cases_in < 0))    stop("'cases' must be non-negative.", call. = FALSE)

  fac   <- factor(node_chr, levels = leaves_chr)
  cases <- as.numeric(tapply(as.numeric(.cases_in), fac, sum))
  cases[is.na(cases)] <- 0

  if (is.null(.pop_in)) {
    if (model == "binomial") {
      stop("The binomial model requires a `population` column (number of ",
           "trials per leaf, i.e., cases + controls).", call. = FALSE)
    }
    population <- rep(1, n_leaves)
  } else {
    population <- as.numeric(tapply(as.numeric(.pop_in), fac,
                                    function(z) z[1L]))
    population[is.na(population)] <- 0
  }

  if (!is.null(seed)) {
    .snap__ <- .seed_save_and_set(seed)
    on.exit(.seed_restore(.snap__), add = TRUE)
  }

  all_nodes <- tree$node_id
  n_nodes <- length(all_nodes)

  # Aggregate cases and population upward
  leaf_idx <- match(leaves, all_nodes)
  node_cases <- rep(0, n_nodes)
  node_pop <- rep(0, n_nodes)
  node_cases[leaf_idx] <- cases
  node_pop[leaf_idx] <- population

  depths <- .compute_depths(tree)
  proc_order <- order(depths, decreasing = TRUE)

  for (idx in proc_order) {
    node <- all_nodes[idx]
    children_idx_v <- which(tree$parent_id == node & !is.na(tree$parent_id))
    if (length(children_idx_v) > 0) {
      child_match <- match(tree$node_id[children_idx_v], all_nodes)
      node_cases[idx] <- sum(node_cases[child_match])
      node_pop[idx] <- sum(node_pop[child_match])
    }
  }

  C <- sum(cases)
  N <- sum(population)

  # --- C++ backend for MC simulation ---
  tree_csr <- .tree_to_csr_children(tree)
  leaf_idx_0 <- as.integer(leaf_idx - 1L)
  proc_order_0 <- as.integer(proc_order - 1L)

  cpp_result <- mc_treescan_cpp(
    as.numeric(node_cases),
    as.numeric(node_pop),
    as.numeric(C),
    as.numeric(N),
    leaf_idx_0,
    as.numeric(population),
    tree_csr$children_idx,
    tree_csr$children_ptr,
    proc_order_0,
    as.integer(nsim),
    model_int,
    as.integer(max(1L, n_cores))
  )

  obs_llr_vec <- cpp_result$obs_llr
  sim_llr <- cpp_result$sim_llr
  best_idx <- cpp_result$obs_best  # 1-based

  # P-values for all cuts
  pvalues <- numeric(n_nodes)
  for (i in seq_len(n_nodes)) {
    pvalues[i] <- (sum(sim_llr >= obs_llr_vec[i]) + 1) / (nsim + 1)
  }

  # Build cuts table
  expected_vec <- C * node_pop / N

  all_cuts <- data.frame(
    node_id = all_nodes,
    cases = node_cases,
    expected = expected_vec,
    population = node_pop,
    llr = obs_llr_vec,
    pvalue = pvalues,
    stringsAsFactors = FALSE
  )
  all_cuts <- all_cuts[order(-all_cuts$llr), ]
  sig_cuts <- all_cuts[all_cuts$pvalue < alpha, ]

  branches <- .get_branches(tree)
  best_node <- all_nodes[best_idx]
  best_leaves <- branches[[as.character(best_node)]]$leaves

  result <- list(
    most_likely_cluster = list(
      node_id = best_node,
      leaf_ids = best_leaves,
      cases = node_cases[best_idx],
      expected = expected_vec[best_idx],
      population = node_pop[best_idx],
      llr = obs_llr_vec[best_idx]
    ),
    all_cuts = all_cuts,
    significant_cuts = sig_cuts,
    pvalue = pvalues[best_idx],
    alpha = alpha,
    nsim = nsim,
    total_cases = C,
    total_population = N,
    tree = tree,
    simulated_llr = sim_llr
  )

  class(result) <- "tree_scan"
  result
}

test_that("circular_scan binomial runs", {
  set.seed(123)
  n <- 15
  cases <- rpois(n, lambda = 10)
  cases[1:4] <- rpois(4, lambda = 50)
  dat <- data.frame(cases = cases, population = rep(1000, n),
                    region_id = 1:n, x = runif(n, 0, 10), y = runif(n, 0, 10))
  res <- circular_scan(dat, cases, population, region_id, x, y,
                       nsim = 49, seed = 42, model = "binomial")
  expect_s3_class(res, "circular_scan")
  expect_equal(res$model, "binomial")
})

test_that("tree_scan binomial requires population", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5, 6, 7, 8),
    parent_id = c(NA, 1, 1, 2, 2, 3, 3, 3)
  )
  # leaves are 4, 5, 6, 7, 8
  dat <- data.frame(node_id = c(4, 5, 6, 7, 8), cases = c(50, 5, 3, 2, 4),
                    pop = c(100, 100, 100, 100, 100))

  expect_error(
    tree_scan(dat, cases, node_id, tree = tree, population = NULL,
              nsim = 9, seed = 1, model = "binomial"),
    "binomial model requires"
  )

  res <- tree_scan(dat, cases, node_id, tree = tree, population = pop,
                   nsim = 49, seed = 42, model = "binomial")
  expect_s3_class(res, "tree_scan")
})

test_that("treespatial_scan binomial runs", {
  ex <- generate_example_data(n_regions = 16, seed = 77)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res <- treespatial_scan(dat, cases, population, region_id, x, y, node_id,
                          tree = ex$tree, nsim = 49, seed = 1,
                          model = "binomial")
  expect_s3_class(res, "treespatial_scan")
  expect_equal(res$model, "binomial")
})

test_that("n_cores > 1 path runs (treespatial)", {
  skip_on_cran()
  ex <- generate_example_data(n_regions = 16, seed = 11)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  args <- list(dat, cases = "cases", population = "population",
               region_id = "region_id", x = "x", y = "y", node_id = "node_id",
               tree = ex$tree, nsim = 49, seed = 1)
  r1 <- do.call(treespatial_scan, c(args, list(n_cores = 1L)))
  r2 <- do.call(treespatial_scan, c(args, list(n_cores = 2L)))
  expect_equal(r1$most_likely_cluster$llr, r2$most_likely_cluster$llr)
  expect_setequal(r1$most_likely_cluster$region_ids,
                  r2$most_likely_cluster$region_ids)
})

test_that("sequential_scan accepts model = 'binomial' and n_cores", {
  skip_on_cran()
  ex <- generate_example_data(n_regions = 12, seed = 33)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res <- sequential_scan(dat, cases, population, region_id, x, y, node_id,
                         tree = ex$tree, max_iter = 2, nsim = 29, seed = 1,
                         model = "binomial", n_cores = 2L, verbose = FALSE)
  expect_s3_class(res, "sequential_scan")
})

test_that("tree_scan returns correct class and structure", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5, 6, 7),
    parent_id = c(NA, 1, 1, 2, 2, 3, 3)
  )
  # leaves are 4, 5, 6, 7
  dat <- data.frame(node_id = c(4, 5, 6, 7), cases = c(50, 5, 3, 2),
                    pop = c(100, 100, 100, 100))

  result <- tree_scan(dat, cases, node_id, tree = tree, population = pop,
                      nsim = 49, seed = 42)

  expect_s3_class(result, "tree_scan")
  expect_true("most_likely_cluster" %in% names(result))
  expect_true("all_cuts" %in% names(result))
  expect_true("significant_cuts" %in% names(result))
  expect_equal(nrow(result$all_cuts), 7)
})

test_that("tree_scan detects obvious cluster", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5, 6, 7),
    parent_id = c(NA, 1, 1, 2, 2, 3, 3)
  )
  # Leaf 4 has far more cases
  dat <- data.frame(node_id = c(4, 5, 6, 7), cases = c(200, 5, 3, 2),
                    pop = c(100, 100, 100, 100))

  result <- tree_scan(dat, cases, node_id, tree = tree, population = pop,
                      nsim = 199, seed = 77)

  # Node 4 or its parent (2) should be the most likely cluster
  expect_true(result$most_likely_cluster$node_id %in% c(2, 4))
  expect_lt(result$pvalue, 0.05)
})

test_that("tree_scan handles default population", {
  tree <- data.frame(
    node_id   = c(1, 2, 3),
    parent_id = c(NA, 1, 1)
  )
  # leaves are 2, 3
  dat <- data.frame(node_id = c(2, 3), cases = c(10, 5))

  result <- tree_scan(dat, cases, node_id, tree = tree, nsim = 19, seed = 1)
  expect_s3_class(result, "tree_scan")
  expect_equal(result$total_population, 2)  # default pop = 1 per leaf
})

test_that("tree_scan errors on node_id that is not a leaf", {
  tree <- data.frame(
    node_id   = c(1, 2, 3),
    parent_id = c(NA, 1, 1)
  )
  dat <- data.frame(node_id = c(2, 3, 99), cases = c(10, 5, 3))
  expect_error(tree_scan(dat, cases, node_id, tree = tree),
               "not leaves of the tree")
})

test_that("tree_scan print method works", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5),
    parent_id = c(NA, 1, 1, 2, 2)
  )
  # leaves are 3, 4, 5
  dat <- data.frame(node_id = c(3, 4, 5), cases = c(30, 5, 2),
                    pop = rep(100, 3))
  result <- tree_scan(dat, cases, node_id, tree = tree, population = pop,
                      nsim = 19, seed = 1)
  expect_output(print(result), "Tree-Based Scan")
})

##=============================================================================
## Example 2: Crime in Chicago, USA (2023)
##
## Two analyses:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025, Sec. 5.1.1):
##      single scan + filter_clusters / get_cluster_regions(n_clusters = N).
##   2. (Optional) iterative_scan with Holm-Bonferroni - NOT in paper.
##
## Population denominator: ACS 2020 5-year residential population per
## community area (replaces the bundled compositional 'population' column).
##=============================================================================

options(bitmapType = "cairo")

library(treeSS)
library(ggplot2)
library(sf)


## ---- 1. Load data ----
data(chicago_crimes)
data(chicago_tree)
data(chicago_map)

cat("=== Chicago Crime 2023 ===\n")
cat("Rows (long):           ", nrow(chicago_crimes), "\n")
cat("Unique community areas:", length(unique(chicago_crimes$region_id)), "\n")
cat("Tree nodes:            ", nrow(chicago_tree), "\n")
cat("Total incidents:       ", sum(chicago_crimes$cases), "\n")

crime_desc <- function(node_id) {
  parts <- strsplit(node_id, " \\| ")[[1]]
  if (length(parts) == 3) return(paste0(parts[2], " (", parts[3], ")"))
  if (length(parts) == 2) return(parts[2])
  node_id
}


## ---- 2. Choose denominator: residents (not total incidents) ----
##
## chicago_crimes ships with two denominator columns:
##   * population:      total incidents per area (compositional - useful
##                       for "which crime types over-occur where", but
##                       not a population at risk)
##   * pop_residential: ACS 2020 5-year residential population. This is
##                       the appropriate denominator for an incidence-rate
##                       analysis (incidents per resident), analogous to
##                       deaths/live_births in the paper's RJ application.
##
## We use pop_residential below.
cat("Total Chicago residents:",
    sum(unique(chicago_crimes[, c("area_number", "pop_residential")])$pop_residential),
    "\n")


## ---- 3. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_chi <- treespatial_scan(
    cases       = chicago_crimes$cases,
    population  = chicago_crimes$pop_residential,
    region_id   = chicago_crimes$region_id,
    x           = chicago_crimes$x,
    y           = chicago_crimes$y,
    node_id     = chicago_crimes$node_id,
    tree        = chicago_tree,
    max_pop_pct = 0.25,
    nsim        = 999, seed = 2023,
    n_cores     = 4L
  )
})
print(result_chi)


## ---- 4. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_chi)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 5. (Optional) Iterative scan with Holm-Bonferroni ----
cat("\n=== Iterative scan (extension, with Holm-Bonferroni) ===\n")
iter_chi <- iterative_scan(
  cases       = chicago_crimes$cases,
  population  = chicago_crimes$pop_residential,
  region_id   = chicago_crimes$region_id,
  x           = chicago_crimes$x,
  y           = chicago_crimes$y,
  node_id     = chicago_crimes$node_id,
  tree        = chicago_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2023,
  max_pop_pct = 0.25, n_cores = 4L
)
print(iter_chi)


## ---- 6. Cluster membership + descriptive lookup ----
region_info <- unique(chicago_crimes[, c("region_id", "area_number", "name")])

cr1   <- merge(get_cluster_regions(result_chi, n_clusters = 1, overlap = FALSE),
               region_info, by = "region_id")
cr2   <- merge(get_cluster_regions(result_chi, n_clusters = 2, overlap = TRUE),
               region_info, by = "region_id")
cr_it <- merge(get_cluster_regions(iter_chi, overlap = TRUE),
               region_info, by = "region_id")


## ---- 7. Plot 1: Most likely cluster ----
chi1 <- merge(chicago_map, cr1, by.x = "AREA_NUM", by.y = "area_number",
              all.x = TRUE)
mlc <- result_chi$most_likely_cluster

p1 <- ggplot(chi1) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.15, alpha = 0.75) +
  scale_fill_manual(
    values   = c("1" = "#C44E52"),
    na.value = "gray95",
    labels   = c("1" = paste0(crime_desc(mlc$node_id), " (",
                              length(mlc$region_ids), " areas)")),
    name     = "Cluster"
  ) +
  labs(
    title    = "Tree-Spatial Scan: most likely cluster",
    subtitle = paste0(crime_desc(mlc$node_id),
                      " | LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_chi$pvalue, digits = 3),
                      " | ", mlc$cases, " incidents (expected ",
                      round(mlc$expected, 1), ")")
  ) +
  theme_minimal(base_size = 12) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 8))

ggsave("chicago_cluster_mlc.png", p1, width = 9, height = 9, dpi = 300)
cat("Saved: chicago_cluster_mlc.png\n")


## ---- 8. Plot 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
chi2 <- merge(chicago_map, cr2, by.x = "AREA_NUM", by.y = "area_number")
palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")

p2 <- ggplot(chi2) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(values = palette2, na.value = "gray95",
                    name = "Cluster", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  labs(title    = "Tree-Spatial Scan: Top 2 Distinct Clusters",
       subtitle = "Cancado et al. (2025), Sec. 5.1.1 - denominator: residents") +
  theme_minimal(base_size = 11) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        strip.text    = element_text(face = "bold", size = 9),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 7),
        legend.position = "none")

ggsave("chicago_clusters_top2.png", p2, width = 14, height = 6, dpi = 300)
cat("Saved: chicago_clusters_top2.png\n")


## ---- 9. Plot 3 (optional): Iterative scan clusters ----
n_iter <- iter_chi$n_iter
if (n_iter > 0) {
  chi_it <- merge(chicago_map, cr_it,
                  by.x = "AREA_NUM", by.y = "area_number", all.x = TRUE)
  palette_it <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_it) <- as.character(seq_len(n_iter))

  n_sig <- sum(iter_chi$clusters$significant, na.rm = TRUE)
  p_it <- ggplot(chi_it) +
    geom_sf(aes(fill = factor(cluster)), color = "gray40",
            linewidth = 0.12, alpha = 0.75) +
    scale_fill_manual(values = palette_it, na.value = "gray95",
                      name = "Iteration", na.translate = FALSE) +
    facet_wrap(~ panel, nrow = 1) +
    labs(title    = paste0("Iterative Scan (extension): ", n_iter,
                            " iterations, ", n_sig,
                            " significant after Holm-Bonferroni"),
         subtitle = "Each panel = one iteration (cases removed before next). NOT in paper.") +
    theme_minimal(base_size = 11) +
    theme(plot.title    = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray40"),
          strip.text    = element_text(face = "bold", size = 9),
          axis.title    = element_blank(),
          axis.text     = element_text(color = "gray50", size = 7),
          legend.position = "none")

  ggsave("chicago_clusters_iterative.png", p_it,
         width = max(8, 4 * n_iter), height = 6, dpi = 300)
  cat("Saved: chicago_clusters_iterative.png (", n_iter, " iterations)\n")
}


## ---- 10. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "\n")
cat("  Description:", crime_desc(mlc$node_id), "\n")
cat("  Areas:", length(mlc$region_ids), "\n")
cat("  Incidents:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_chi$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Iterative scan: ", n_iter, " iterations, ",
    sum(iter_chi$clusters$significant, na.rm = TRUE),
    " significant after Holm-Bonferroni\n", sep = "")
cat("\nDone!\n")

#' @keywords internal
.validate_regions <- function(regions) {
  required <- c("region_id", "population", "x", "y")
  missing_cols <- setdiff(required, names(regions))
  if (length(missing_cols) > 0) {
    stop("'regions' is missing required columns: ",
         paste(missing_cols, collapse = ", "),
         ". Required: region_id, population, x, y",
         call. = FALSE)
  }
  if (any(regions$population < 0)) {
    stop("'population' must be non-negative.", call. = FALSE)
  }
  if (nrow(regions) < 2) {
    stop("At least 2 regions are required.", call. = FALSE)
  }
  invisible(TRUE)
}

#' @keywords internal
.validate_tree <- function(tree) {
  required <- c("node_id", "parent_id")
  missing_cols <- setdiff(required, names(tree))
  if (length(missing_cols) > 0) {
    stop("'tree' is missing required columns: ",
         paste(missing_cols, collapse = ", "),
         ". Required: node_id, parent_id",
         call. = FALSE)
  }
  # Check for a single root (parent_id is NA)
  roots <- is.na(tree$parent_id)
  if (sum(roots) == 0) {
    stop("Tree must have at least one root node (parent_id = NA).",
         call. = FALSE)
  }
  invisible(TRUE)
}

#' @keywords internal
.validate_cases_matrix <- function(cases, tree, regions) {
  if (!is.matrix(cases) && !is.data.frame(cases)) {
    stop("'cases' must be a matrix or data.frame.", call. = FALSE)
  }
  cases <- as.matrix(cases)
  leaves <- .get_leaves(tree)
  if (nrow(cases) != length(leaves)) {
    stop("Number of rows in 'cases' (", nrow(cases), ") must match the ",
         "number of leaf nodes in the tree (", length(leaves), ").",
         call. = FALSE)
  }
  if (ncol(cases) != nrow(regions)) {
    stop("Number of columns in 'cases' (", ncol(cases), ") must match the ",
         "number of regions (", nrow(regions), ").",
         call. = FALSE)
  }
  if (any(cases < 0, na.rm = TRUE)) {
    stop("'cases' must contain non-negative values.", call. = FALSE)
  }
  invisible(TRUE)
}

#' @keywords internal
.get_leaves <- function(tree) {
  all_nodes <- tree$node_id
  parent_nodes <- unique(tree$parent_id[!is.na(tree$parent_id)])
  leaves <- setdiff(all_nodes, parent_nodes)
  leaves
}

#' @keywords internal
.get_branches <- function(tree) {
  # Returns a list where each element is a branch (node + all its descendants)
  all_nodes <- tree$node_id
  branches <- list()

  for (node in all_nodes) {
    descendants <- .get_descendants(tree, node)
    branches[[as.character(node)]] <- list(
      node = node,
      leaves = intersect(descendants, .get_leaves(tree))
    )
  }

  branches
}

#' @keywords internal
.get_descendants <- function(tree, node) {
  children <- tree$node_id[!is.na(tree$parent_id) & tree$parent_id == node]
  if (length(children) == 0) {
    return(node)
  }
  desc <- node
  for (child in children) {
    desc <- c(desc, .get_descendants(tree, child))
  }
  desc
}

#' @keywords internal
.poisson_llr <- function(cz, nz, C, N) {
  # Log-likelihood ratio for the Poisson model
  # cz: cases inside zone, nz: population inside zone
  # C: total cases, N: total population
  cz_bar <- C - cz
  nz_bar <- N - nz

  expected <- C * nz / N

  if (cz <= expected || cz == 0 || nz == 0 || nz_bar == 0) {
    return(0)
  }

  llr <- 0
  if (cz > 0) {
    llr <- llr + cz * log(cz / expected)
  }
  if (cz_bar > 0) {
    llr <- llr + cz_bar * log(cz_bar / (C - expected))
  }

  llr
}


#' @keywords internal
#' Bottom-up aggregation: given a leaf x region matrix, return a
#' (all_nodes) x region matrix where each non-leaf row is the sum of its
#' children rows.
.aggregate_leaves_to_all <- function(cases_matrix, tree) {
  leaves    <- .get_leaves(tree)
  all_nodes <- tree$node_id
  n_nodes   <- length(all_nodes)
  n_regions <- ncol(cases_matrix)

  full_cases <- matrix(0, nrow = n_nodes, ncol = n_regions)
  rownames(full_cases) <- as.character(all_nodes)
  colnames(full_cases) <- colnames(cases_matrix)

  leaf_idx <- match(leaves, all_nodes)
  full_cases[leaf_idx, ] <- cases_matrix

  depths <- .compute_depths(tree)
  processing_order <- order(depths, decreasing = TRUE)

  for (idx in processing_order) {
    node <- all_nodes[idx]
    children_idx <- which(tree$parent_id == node & !is.na(tree$parent_id))
    if (length(children_idx) > 0) {
      child_rows <- match(tree$node_id[children_idx], all_nodes)
      if (length(child_rows) == 1) {
        full_cases[idx, ] <- full_cases[child_rows, ]
      } else {
        full_cases[idx, ] <- colSums(full_cases[child_rows, , drop = FALSE])
      }
    }
  }
  full_cases
}


#' @keywords internal
#' Compute the depth of each node in the tree (root = 0).
.compute_depths <- function(tree) {
  depths <- rep(NA_integer_, nrow(tree))
  roots <- which(is.na(tree$parent_id))
  depths[roots] <- 0L

  changed <- TRUE
  while (changed) {
    changed <- FALSE
    for (i in seq_len(nrow(tree))) {
      if (is.na(depths[i]) && !is.na(tree$parent_id[i])) {
        parent_idx <- match(tree$parent_id[i], tree$node_id)
        if (!is.na(parent_idx) && !is.na(depths[parent_idx])) {
          depths[i] <- depths[parent_idx] + 1L
          changed <- TRUE
        }
      }
    }
  }
  depths
}


## .seed_save_and_set(): save the user's current RNG state (or signal
## its absence), and set a new seed. Returns a "snapshot" object that
## the caller must pass to .seed_restore() in an on.exit() handler.
##
## Idiomatic usage from a user-facing function:
##
##   if (!is.null(seed)) {
##     snap <- .seed_save_and_set(seed)
##     on.exit(.seed_restore(snap), add = TRUE)
##   }
##
## This guarantees that the user's RNG state is restored when the
## function returns, whether normally or via an error. The user's
## session-level set.seed() is therefore never silently overridden
## by the seed argument to scan functions.
.seed_save_and_set <- function(seed) {
  has_prev <- exists(".Random.seed", envir = .GlobalEnv,
                     inherits = FALSE)
  prev <- if (has_prev) {
    get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  } else {
    NULL
  }
  set.seed(seed)
  list(has_prev = has_prev, prev = prev)
}

.seed_restore <- function(snap) {
  if (snap$has_prev) {
    assign(".Random.seed", snap$prev, envir = .GlobalEnv)
  } else if (exists(".Random.seed", envir = .GlobalEnv,
                    inherits = FALSE)) {
    rm(".Random.seed", envir = .GlobalEnv)
  }
  invisible()
}

test_that("validate_regions catches missing columns", {
  bad_regions <- data.frame(id = 1:3, pop = c(100, 200, 300))
  expect_error(treeSS:::.validate_regions(bad_regions), "missing required columns")
})

test_that("validate_regions catches negative population", {
  regions <- data.frame(
    region_id = 1:3, population = c(100, -1, 300), x = 1:3, y = 1:3
  )
  expect_error(treeSS:::.validate_regions(regions), "non-negative")
})

test_that("validate_regions catches single region", {
  regions <- data.frame(region_id = 1, population = 100, x = 1, y = 1)
  expect_error(treeSS:::.validate_regions(regions), "At least 2")
})

test_that("validate_tree catches missing columns", {
  bad_tree <- data.frame(id = 1:3, parent = c(NA, 1, 1))
  expect_error(treeSS:::.validate_tree(bad_tree), "missing required columns")
})

test_that("validate_tree catches missing root", {
  tree <- data.frame(node_id = 1:3, parent_id = c(2, 3, 1))
  expect_error(treeSS:::.validate_tree(tree), "root node")
})

test_that("get_leaves works correctly", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5),
    parent_id = c(NA, 1, 1, 2, 2)
  )
  leaves <- treeSS:::.get_leaves(tree)
  expect_setequal(leaves, c(3, 4, 5))
})

test_that("get_descendants works correctly", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5, 6, 7),
    parent_id = c(NA, 1, 1, 2, 2, 3, 3)
  )
  desc <- treeSS:::.get_descendants(tree, 2)
  expect_setequal(desc, c(2, 4, 5))
  desc_root <- treeSS:::.get_descendants(tree, 1)
  expect_setequal(desc_root, 1:7)
})

test_that("poisson_llr returns 0 when cases <= expected", {
  expect_equal(treeSS:::.poisson_llr(5, 100, 100, 1000), 0)
})

test_that("poisson_llr returns positive for excess cases", {
  llr <- treeSS:::.poisson_llr(50, 100, 100, 1000)
  expect_gt(llr, 0)
})

##=============================================================================
## Example 3: Road Traffic Collisions in London, UK (2022)
##
## Two analyses:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025, Sec. 5.1.1):
##      single scan + filter_clusters / get_cluster_regions(n_clusters = N).
##   2. (Optional) iterative_scan with Holm-Bonferroni - NOT in paper.
##
## Maps: leaflet (interactive, HTML)
## Polygons: london_boroughs_map (included in the package)
##=============================================================================

library(treeSS)
library(leaflet)
library(sf)
library(htmlwidgets)
library(htmltools)


## ---- 1. Load data ----
data(london_collisions)
data(london_tree)
data(london_boroughs_map)

cat("=== London Road Collisions 2022 ===\n")
cat("Rows (long):     ", nrow(london_collisions), "\n")
cat("Unique boroughs: ", length(unique(london_collisions$region_id)), "\n")
cat("Tree nodes:      ", nrow(london_tree), "\n")
cat("Total collisions:", sum(london_collisions$cases), "\n")


## ---- 2. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4)...\n")
system.time({
  result_ldn <- treespatial_scan(
    cases       = london_collisions$cases,
    population  = london_collisions$population,
    region_id   = london_collisions$region_id,
    x           = london_collisions$x,
    y           = london_collisions$y,
    node_id     = london_collisions$node_id,
    tree        = london_tree,
    max_pop_pct = 0.25,
    nsim        = 999, seed = 42,
    n_cores     = 4L
  )
})
print(result_ldn)


## ---- 3. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_ldn)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 4. (Optional) Iterative scan with Holm-Bonferroni ----
cat("\n=== Iterative scan (extension, with Holm-Bonferroni) ===\n")
iter_ldn <- iterative_scan(
  cases       = london_collisions$cases,
  population  = london_collisions$population,
  region_id   = london_collisions$region_id,
  x           = london_collisions$x,
  y           = london_collisions$y,
  node_id     = london_collisions$node_id,
  tree        = london_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 42,
  max_pop_pct = 0.25, n_cores = 4L
)
print(iter_ldn)


## ---- 5. Cluster membership + descriptive lookup ----
region_info <- unique(london_collisions[, c("region_id", "borough")])

cr1   <- merge(get_cluster_regions(result_ldn, n_clusters = 1, overlap = FALSE),
               region_info, by = "region_id")
cr2   <- merge(get_cluster_regions(result_ldn, n_clusters = 2, overlap = TRUE),
               region_info, by = "region_id")
cr_it <- merge(get_cluster_regions(iter_ldn, overlap = TRUE),
               region_info, by = "region_id")

JOIN_X <- "NAME"     # column in london_boroughs_map
JOIN_Y <- "borough"  # column in cluster results


## ---- 6. Map 1: Most likely cluster (leaflet) ----
ldn1 <- merge(london_boroughs_map, cr1,
              by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)
mlc <- result_ldn$most_likely_cluster

ldn1$label <- ifelse(
  is.na(ldn1$cluster),
  paste0("<b>", ldn1$NAME, "</b><br>Not in cluster"),
  paste0("<b>", ldn1$NAME, "</b><br>",
         "<span style='color:#C44E52'><b>", ldn1$node_id, "</b></span><br>",
         "LR: ", round(ldn1$llr, 1),
         " | p = ", round(ldn1$pvalue, 3))
)

map1 <- leaflet(ldn1) %>%
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(
    fillColor   = ~ifelse(is.na(cluster), "#E8E8E8", "#C44E52"),
    fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
    color       = "gray40", weight = 1.5,
    label       = ~lapply(label, HTML)
  ) %>%
  setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
  addLegend(
    position = "bottomright",
    colors   = c("#C44E52", "#E8E8E8"),
    labels   = c(paste0(mlc$node_id, " (",
                        length(mlc$region_ids), " boroughs)"),
                 "Not in cluster"),
    title    = paste0("LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_ldn$pvalue, digits = 3)),
    opacity  = 0.8
  )

saveWidget(map1, "london_cluster_mlc.html", selfcontained = TRUE)
cat("Saved: london_cluster_mlc.html\n")


## ---- 7. Map 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
# A borough may appear in multiple cluster panels in overlap mode; for the
# leaflet map we keep the lowest-ranked cluster (cluster 1 wins over 2).
cr2_first <- cr2[order(cr2$cluster), ]
cr2_first <- cr2_first[!duplicated(cr2_first$borough), ]

ldn2 <- merge(london_boroughs_map, cr2_first,
              by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)

palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")
top2 <- filter_clusters(result_ldn)
top2 <- head(top2, 2)

ldn2$fillc <- ifelse(is.na(ldn2$cluster), "#E8E8E8",
                     palette2[as.character(ldn2$cluster)])
ldn2$label <- ifelse(
  is.na(ldn2$cluster),
  paste0("<b>", ldn2$NAME, "</b><br>Not in any top-2 cluster"),
  paste0("<b>", ldn2$NAME, "</b><br>",
         "Cluster ", ldn2$cluster, ": ",
         "<span style='color:", palette2[as.character(ldn2$cluster)],
         "'><b>", ldn2$node_id, "</b></span><br>",
         "LR: ", round(ldn2$llr, 1),
         " | p = ", round(ldn2$pvalue, 3))
)

map2 <- leaflet(ldn2) %>%
  addProviderTiles(providers$CartoDB.Positron) %>%
  addPolygons(
    fillColor   = ~fillc,
    fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
    color       = "gray40", weight = 1.5,
    label       = ~lapply(label, HTML)
  ) %>%
  setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
  addLegend(
    position = "bottomright",
    colors   = c(palette2, "#E8E8E8"),
    labels   = c(paste0("Cluster ", seq_along(top2$node_id), ": ",
                        top2$node_id),
                 "Other"),
    title    = "Top 2 distinct (paper Sec. 5.1.1)",
    opacity  = 0.8
  )

saveWidget(map2, "london_clusters_top2.html", selfcontained = TRUE)
cat("Saved: london_clusters_top2.html\n")


## ---- 8. Map 3 (optional): Iterative scan clusters ----
n_iter <- iter_ldn$n_iter
if (n_iter > 0) {
  cr_it_first <- cr_it[order(cr_it$cluster), ]
  cr_it_first <- cr_it_first[!duplicated(cr_it_first$borough), ]

  ldn_it <- merge(london_boroughs_map, cr_it_first,
                  by.x = JOIN_X, by.y = JOIN_Y, all.x = TRUE)

  palette_it <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_it) <- as.character(seq_len(n_iter))

  n_sig <- sum(iter_ldn$clusters$significant, na.rm = TRUE)
  ldn_it$fillc <- ifelse(is.na(ldn_it$cluster), "#E8E8E8",
                          palette_it[as.character(ldn_it$cluster)])
  ldn_it$label <- ifelse(
    is.na(ldn_it$cluster),
    paste0("<b>", ldn_it$NAME, "</b><br>Not detected"),
    paste0("<b>", ldn_it$NAME, "</b><br>",
           "Iteration ", ldn_it$cluster, ": ",
           "<span style='color:", palette_it[as.character(ldn_it$cluster)],
           "'><b>", ldn_it$node_id, "</b></span><br>",
           "LR: ", round(ldn_it$llr, 1),
           " | adj p = ", round(ldn_it$pvalue_adjusted, 3))
  )

  map_it <- leaflet(ldn_it) %>%
    addProviderTiles(providers$CartoDB.Positron) %>%
    addPolygons(
      fillColor   = ~fillc,
      fillOpacity = ~ifelse(is.na(cluster), 0.3, 0.65),
      color       = "gray40", weight = 1.5,
      label       = ~lapply(label, HTML)
    ) %>%
    setView(lng = -0.12, lat = 51.5, zoom = 10) %>%
    addLegend(
      position = "bottomright",
      colors   = c(palette_it, "#E8E8E8"),
      labels   = c(paste0("Iteration ", seq_len(n_iter)), "Other"),
      title    = paste0("Iterative scan (extension)<br>",
                        n_sig, " of ", n_iter,
                        " significant (Holm-Bonferroni)"),
      opacity  = 0.8
    )

  saveWidget(map_it, "london_clusters_iterative.html", selfcontained = TRUE)
  cat("Saved: london_clusters_iterative.html\n")
}


## ---- 9. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "\n")
cat("  Boroughs:", length(mlc$region_ids), "\n")
cat("  Collisions:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_ldn$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Iterative scan: ", n_iter, " iterations, ",
    sum(iter_ldn$clusters$significant, na.rm = TRUE),
    " significant after Holm-Bonferroni\n", sep = "")
cat("\nDone!\n")

#' treeSS: Tree-Spatial Scan Statistic for Cluster Detection
#'
#' Implements the tree-spatial scan statistic for detecting clusters that
#' combine both spatial and hierarchical structures, as proposed by Cançado
#' et al. (2025). The method extends Kulldorff's (1997) circular spatial scan
#' statistic and the tree-based scan statistic (Kulldorff et al. 2003) by
#' searching for anomalies in both geographic regions and branches of
#' hierarchical trees simultaneously.
#'
#' @section Main functions:
#' \describe{
#'   \item{\code{\link{treespatial_scan}}}{Performs the tree-spatial scan
#'     statistic, detecting clusters defined by pairs of spatial zones and
#'     tree branches.}
#'   \item{\code{\link{circular_scan}}}{Performs Kulldorff's circular spatial
#'     scan statistic.}
#'   \item{\code{\link{tree_scan}}}{Performs the tree-based scan statistic.}
#' }
#'
#' @section Helper functions:
#' \describe{
#'   \item{\code{\link{build_zones}}}{Constructs candidate spatial zones using
#'     circular windows centered on region centroids.}
#'   \item{\code{\link{aggregate_tree}}}{Aggregates case counts from leaves to
#'     internal nodes of the hierarchical tree.}
#'   \item{\code{\link{filter_clusters}}}{Removes overlapping secondary
#'     clusters from the tree-spatial scan output.}
#' }
#'
#' @references
#' Cançado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
#' (2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
#' Statistics}, 32, 953–978. \doi{10.1007/s10651-025-00670-w}
#'
#' Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
#' Statistics - Theory and Methods}, 26(6), 1481–1496.
#'
#' Kulldorff, M., Fang, Z., & Walsh, S. J. (2003). A tree-based scan
#' statistic for database disease surveillance. \emph{Biometrics}, 59(2),
#' 323–331.
#'
#' @docType package
#' @name treeSS-package
#' @useDynLib treeSS, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom utils head
"_PACKAGE"

## .cat_wrapped(): print "  Label: v1, v2, v3, ..." with automatic
## line-wrapping at 65 characters (so the output fits in the R Journal
## two-column PDF format) and optional truncation when there are too
## many values to print in full.
##
## When `length(values) > max_show`, only the first `max_show` are
## shown and a tail of "... and N more" is appended. Pass `max_show
## = -1L` (or any value larger than `length(values)`) to print the
## full list. This mirrors the way `tibble` prints data frames:
## sensible default, with an explicit opt-in for the full output.
##
## Typical use:
##   .cat_wrapped("  Leaf IDs:", mlc$leaf_ids, max_show = 10L)
.cat_wrapped <- function(label, values, sep = ", ", max_show = 10L) {
  n <- length(values)
  if (n == 0L) {
    cat(label, "\n", sep = "")
    return(invisible())
  }
  truncated <- max_show > 0L && n > max_show
  if (truncated) {
    shown <- values[seq_len(max_show)]
    text  <- paste0(paste(shown, collapse = sep),
                    sep, "... and ", n - max_show, " more")
  } else {
    text  <- paste(values, collapse = sep)
  }
  width <- min(getOption("width", 80L), 65L)
  # Indent continuation lines with the same number of spaces as `label`
  # plus one for the space that follows it.
  indent <- paste(rep(" ", nchar(label) + 1L), collapse = "")
  lines <- strwrap(text, width = width,
                   initial = paste0(label, " "),
                   prefix  = indent)
  cat(lines, sep = "\n")
  cat("\n")
  invisible()
}

#' Print Method for circular_scan Objects
#'
#' @param x An object of class \code{"circular_scan"}.
#' @param max_show Integer. Maximum number of region IDs to display
#'   in full before truncating with "... and N more". The default of
#'   \code{10L} keeps console output compact for large clusters; set
#'   \code{max_show = -1L} (or any value larger than the cluster
#'   size) to print every region. Mirrors the convention used by
#'   \pkg{tibble} for printing wide tables.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns \code{x} (the input object of class
#'   \code{"circular_scan"}), unchanged. Called for the side effect
#'   of printing a human-readable summary of the scan result to the
#'   console: the total cases and population, the number of regions
#'   scanned, the number of Monte Carlo simulations, and the
#'   contents of the most likely cluster (region IDs, cases,
#'   expected count, population, relative risk, log-likelihood
#'   ratio, and p-value).
#' @export
print.circular_scan <- function(x, max_show = 10L, ...) {
  cat("Circular Spatial Scan Statistic\n")
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat("Total cases:", x$total_cases, "\n")
  cat("Total population:", x$total_population, "\n")
  cat("Number of regions:", nrow(x$regions), "\n")
  cat("Monte Carlo simulations:", x$nsim, "\n\n")

  mlc <- x$most_likely_cluster
  cat("Most likely cluster:\n")
  .cat_wrapped("  Regions:", mlc$region_ids, max_show = max_show)
  cat("  Number of regions:", length(mlc$region_ids), "\n")
  cat("  Cases:", mlc$cases, "\n")
  cat("  Expected:", round(mlc$expected, 2), "\n")
  cat("  Population:", mlc$population, "\n")
  cat("  Relative risk:", round(mlc$rr, 4), "\n")
  cat("  Log-LR:", round(mlc$llr, 4), "\n")
  cat("  P-value:", format.pval(x$pvalue, digits = 4), "\n")

  if (!is.null(x$secondary_clusters) && nrow(x$secondary_clusters) > 0) {
    cat("  Secondary candidates:", nrow(x$secondary_clusters),
        "(use filter_clusters() to extract distinct clusters)\n")
  }

  invisible(x)
}

#' Summary Method for circular_scan Objects
#'
#' Prints the same fields as \code{print.circular_scan()}, followed
#' by a numeric summary of the simulated log-likelihood-ratio
#' distribution under the null. Useful for checking that the Monte
#' Carlo replicates produced a reasonable spread relative to the
#' observed test statistic.
#'
#' @param object An object of class \code{"circular_scan"}.
#' @param ... Further arguments passed to \code{print.circular_scan()}.
#'   In particular, \code{max_show} controls how many region IDs of
#'   the most-likely cluster are printed in full before truncating
#'   with \code{"... and N more"}; see
#'   \code{\link{print.circular_scan}} for details.
#'
#' @return Invisibly returns \code{object} (the input object of
#'   class \code{"circular_scan"}), unchanged. Called for the side
#'   effect of printing the same fields as
#'   \code{print.circular_scan()}, followed by a numeric summary
#'   (min, quartiles, median, mean, max) of the simulated
#'   log-likelihood-ratio distribution under the null.
#' @export
summary.circular_scan <- function(object, ...) {
  print.circular_scan(object, ...)
  cat("\nSimulated LLR distribution:\n")
  print(summary(object$simulated_llr))
  invisible(object)
}

#' Print Method for tree_scan Objects
#'
#' @param x An object of class \code{"tree_scan"}.
#' @param max_show Integer. Maximum number of leaf IDs to display
#'   in full before truncating with "... and N more". The default of
#'   \code{10L} keeps console output compact when a node spans
#'   hundreds or thousands of leaves (such as the root of a deep
#'   tree); set \code{max_show = -1L} to print every leaf.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns \code{x} (the input object of class
#'   \code{"tree_scan"}), unchanged. Called for the side effect of
#'   printing a human-readable summary of the scan result to the
#'   console: the total cases and population, the number of tree
#'   nodes, the number of Monte Carlo simulations, the contents of
#'   the most likely cluster (node ID, leaf IDs, cases, expected
#'   count, log-likelihood ratio, and p-value), and the top
#'   significant cuts at the chosen significance threshold.
#' @export
print.tree_scan <- function(x, max_show = 10L, ...) {
  cat("Tree-Based Scan Statistic\n")
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat("Total cases:", x$total_cases, "\n")
  cat("Total population:", x$total_population, "\n")
  cat("Number of nodes:", nrow(x$tree), "\n")
  cat("Monte Carlo simulations:", x$nsim, "\n\n")

  mlc <- x$most_likely_cluster
  cat("Most likely cluster:\n")
  cat("  Node ID:", mlc$node_id, "\n")
  .cat_wrapped("  Leaf IDs:", mlc$leaf_ids, max_show = max_show)
  cat("  Cases:", mlc$cases, "\n")
  cat("  Expected:", round(mlc$expected, 2), "\n")
  cat("  Log-LR:", round(mlc$llr, 4), "\n")
  cat("  P-value:", format.pval(x$pvalue, digits = 4), "\n")

  nsig <- nrow(x$significant_cuts)
  cat("\nSignificant cuts (alpha =", x$alpha, "):", nsig, "\n")

  if (nsig > 0) {
    top <- head(x$significant_cuts, 10)
    cat("\nTop significant cuts:\n")
    print(top, row.names = FALSE)
  }

  invisible(x)
}

#' Summary Method for tree_scan Objects
#'
#' Prints the same fields as \code{print.tree_scan()}, followed by
#' the full table of all candidate cuts (not just significant ones)
#' ordered by decreasing log-likelihood ratio.
#'
#' @param object An object of class \code{"tree_scan"}.
#' @param ... Further arguments passed to \code{print.tree_scan()}.
#'   In particular, \code{max_show} controls how many leaf IDs of
#'   the most-likely cluster are printed in full before truncating
#'   with \code{"... and N more"}; see \code{\link{print.tree_scan}}
#'   for details.
#'
#' @return Invisibly returns \code{object} (the input object of
#'   class \code{"tree_scan"}), unchanged. Called for the side
#'   effect of printing the same fields as
#'   \code{print.tree_scan()}, followed by the full table of all
#'   candidate cuts (not just significant ones) ordered by
#'   decreasing log-likelihood ratio.
#' @export
summary.tree_scan <- function(object, ...) {
  print.tree_scan(object, ...)
  cat("\nAll cuts:\n")
  print(object$all_cuts, row.names = FALSE)
  invisible(object)
}

#' Print Method for treespatial_scan Objects
#'
#' @param x An object of class \code{"treespatial_scan"}.
#' @param max_show Integer. Maximum number of leaf IDs and region
#'   IDs to display in full before truncating with "... and N more".
#'   The default of \code{10L} keeps console output compact when the
#'   most likely cluster spans many leaves (for example, when the
#'   root node maximises the likelihood ratio for an aggregated
#'   denominator) or many regions; set \code{max_show = -1L} to
#'   print every value. Mirrors the convention used by \pkg{tibble}.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns \code{x} (the input object of class
#'   \code{"treespatial_scan"}), unchanged. Called for the side
#'   effect of printing a human-readable summary of the scan result
#'   to the console: the total population, the number of regions
#'   and tree nodes, the number of Monte Carlo simulations, and the
#'   contents of the most likely cluster (tree node, leaf IDs,
#'   region IDs, cases, expected count, population, relative risk,
#'   log-likelihood ratio, and p-value).
#' @export
print.treespatial_scan <- function(x, max_show = 10L, ...) {
  cat("Tree-Spatial Scan Statistic\n")
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat("Total population:", x$total_population, "\n")
  cat("Number of regions:", nrow(x$regions), "\n")
  cat("Number of tree nodes:", nrow(x$tree), "\n")
  cat("Monte Carlo simulations:", x$nsim, "\n\n")

  mlc <- x$most_likely_cluster
  cat("Most likely cluster:\n")
  cat("  Tree node:", mlc$node_id, "\n")
  .cat_wrapped("  Leaf IDs:", mlc$leaf_ids, max_show = max_show)
  .cat_wrapped("  Regions:", mlc$region_ids, max_show = max_show)
  cat("  Number of regions:", length(mlc$region_ids), "\n")
  cat("  Cases in (zone, branch):", mlc$cases, "\n")
  cat("  Expected:", round(mlc$expected, 2), "\n")
  cat("  Population:", mlc$population, "\n")
  cat("  Relative risk:", round(mlc$rr, 4), "\n")
  cat("  Log-LR:", round(mlc$llr, 4), "\n")
  cat("  P-value:", format.pval(x$pvalue, digits = 4), "\n")

  invisible(x)
}

#' Summary Method for treespatial_scan Objects
#'
#' Prints the same fields as \code{print.treespatial_scan()}, followed
#' by the top-10 branches by total case count and a numeric summary
#' of the simulated log-likelihood-ratio distribution under the null.
#' Useful for diagnosing how heavily one branch of the tree dominates
#' the dataset and whether the observed maximum LR is a clear outlier
#' against the simulated null.
#'
#' @param object An object of class \code{"treespatial_scan"}.
#' @param ... Further arguments passed to
#'   \code{print.treespatial_scan()}. In particular, \code{max_show}
#'   controls how many leaf IDs and region IDs of the most-likely
#'   cluster are printed in full before truncating with \code{"...
#'   and N more"}; see \code{\link{print.treespatial_scan}} for
#'   details.
#'
#' @return Invisibly returns \code{object} (the input object of
#'   class \code{"treespatial_scan"}), unchanged. Called for the
#'   side effect of printing the same fields as
#'   \code{print.treespatial_scan()}, followed by the top-10
#'   branches by total case count and a numeric summary (min,
#'   quartiles, median, mean, max) of the simulated
#'   log-likelihood-ratio distribution under the null.
#' @export
summary.treespatial_scan <- function(object, ...) {
  print.treespatial_scan(object, ...)

  cat("\nTotal cases by branch (top 10):\n")
  cg <- sort(object$total_cases_by_branch, decreasing = TRUE)
  print(head(cg, 10))

  cat("\nSimulated LLR distribution:\n")
  print(summary(object$simulated_llr))

  invisible(object)
}

#' Print Method for sequential_scan Objects
#'
#' @param x An object of class \code{"sequential_scan"}.
#' @param max_show Integer. Maximum number of region IDs (or leaf IDs,
#'   for tree-only scans) to display in full per iteration before
#'   truncating with \code{"... and N more"}. Default \code{10L}; set
#'   \code{max_show = -1L} to print every value.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns \code{x} (the input object of class
#'   \code{"sequential_scan"}), unchanged. Called for the side effect
#'   of printing a human-readable summary of the sequential scan
#'   result to the console: the scan type, the buffer size, the
#'   number of iterations performed, the significance threshold, the
#'   number of significant clusters, and a per-iteration table of
#'   detected clusters.
#' @export
print.sequential_scan <- function(x, max_show = 10L, ...) {
  cat("Sequential Scan (Zhang, Assuncao & Kulldorff, 2010)\n")
  cat(paste(rep("-", 55), collapse = ""), "\n")
  cat("Scan type           :", x$scan_type, "\n")
  cat("Buffer size         :", x$buffer_size,
      "neighbouring region(s) per iteration\n")
  cat("Iterations performed:", x$n_iter, "\n")
  cat("Alpha               :", x$alpha, "\n")
  cat("MC replications/iter:", x$nsim, "\n\n")

  if (nrow(x$clusters) > 0L) {
    n_sig <- sum(x$clusters$significant, na.rm = TRUE)
    cat("Significant clusters: ", n_sig, " of ",
        nrow(x$clusters), "\n\n", sep = "")
    display <- x$clusters
    # Hide list-columns from the tabular print
    if (!isTRUE(max_show < 0L)) {
      display$region_ids <- NULL
      display$leaf_ids   <- NULL
    }
    print(display, row.names = FALSE)
  } else {
    cat("No clusters detected.\n")
  }
  invisible(x)
}


#' Summary Method for sequential_scan Objects
#'
#' @param object An object of class \code{"sequential_scan"}.
#' @param ... Passed to \code{print.sequential_scan()}.
#'
#' @return Invisibly returns \code{object}. Called for the side effect
#'   of printing the same fields as \code{print.sequential_scan()}
#'   followed by, for each iteration, the cluster composition
#'   (region IDs or leaf IDs).
#' @export
summary.sequential_scan <- function(object, ...) {
  print.sequential_scan(object, ...)
  if (nrow(object$clusters) > 0L) {
    cat("\nCluster composition per iteration:\n")
    cl <- object$clusters
    for (k in seq_len(nrow(cl))) {
      cat(sprintf("\n  Iteration %d:\n", cl$iteration[k]))
      if ("region_ids" %in% names(cl)) {
        .cat_wrapped("    Regions :", cl$region_ids[[k]])
      }
      if ("leaf_ids" %in% names(cl)) {
        .cat_wrapped("    Leaves  :", cl$leaf_ids[[k]])
      }
    }
  }
  invisible(object)
}

#' Iterative Scan
#'
#' Runs the scan statistic iteratively, removing the cases of each detected
#' cluster before the next iteration. Supports tree-spatial, circular, and
#' tree-only scans, dispatched by which arguments are supplied.
#'
#' \strong{Note on methodology.} This iterative procedure is \emph{not} part
#' of the original tree-spatial scan statistic of Cancado et al. (2025). It is
#' an extension inspired by the conditional scan approach of Zhang et al.
#' (2010), which sequentially removes cases attributed to detected clusters
#' before re-running the scan to find additional, distinct anomalies. To
#' reproduce the secondary-cluster procedure described in Section 5.1.1 of
#' Cancado et al. (2025), use \code{\link{filter_clusters}} or
#' \code{\link{get_cluster_regions}} with \code{n_clusters > 1} on a single
#' scan result instead.
#'
#' \strong{Multiple-testing correction.} Because each iteration is a separate
#' hypothesis test on data that has been modified by the previous iteration,
#' the raw p-values overstate significance. This function collects raw p-values
#' from all iterations performed and applies the Holm-Bonferroni correction
#' (\code{\link[stats]{p.adjust}} with \code{method = "holm"}) at the end. The
#' returned \code{clusters} data.frame includes both the raw p-value
#' (\code{pvalue}) and the adjusted p-value (\code{pvalue_adjusted}), plus a
#' logical \code{significant} column indicating whether the cluster is
#' significant after correction at level \code{alpha}.
#'
#' The loop stops early only when the residual signal is exhausted (the most
#' likely cluster has \code{LR = 0} or zero cases), not based on raw p-values,
#' since the multiple-testing correction depends on the full set of tests.
#'
#' @param cases Numeric vector. For tree-spatial scan: one entry per
#'   (region, leaf) row. For circular: one entry per region. For tree-only:
#'   one entry per leaf.
#' @param population Numeric vector parallel to \code{cases}.
#' @param region_id,x,y Vectors parallel to \code{cases} (omit for
#'   tree-only scan).
#' @param node_id Vector parallel to \code{cases} (omit for circular and
#'   tree-only scans).
#' @param tree Tree as a 2-column data.frame (\code{node_id, parent_id}),
#'   or use \code{tree_node_id}/\code{tree_parent_id}. Omit for circular
#'   scan.
#' @param tree_node_id,tree_parent_id Optional. Parallel vectors as an
#'   alternative to \code{tree}.
#' @param max_iter Integer. Maximum number of iterations.
#' @param alpha Numeric. Significance threshold applied to Holm-Bonferroni
#'   adjusted p-values.
#' @param nsim Integer. MC simulations per iteration.
#' @param max_pop_pct Numeric. Passed to inner scans.
#' @param model Character. \code{"poisson"} or \code{"binomial"}.
#' @param seed Integer or \code{NULL}.
#' @param verbose Logical.
#' @param n_cores Integer. OpenMP threads.
#'
#' @return An object of class \code{"iterative_scan"} with components:
#'   \describe{
#'     \item{clusters}{A data.frame with one row per iteration, containing
#'       the raw p-value (\code{pvalue}), the Holm-Bonferroni adjusted
#'       p-value (\code{pvalue_adjusted}), a logical \code{significant}
#'       column, plus the cluster's node, regions, cases, expected, RR, and
#'       LLR.}
#'     \item{iterations}{A list with the full scan result of each iteration.}
#'     \item{regions, tree, alpha, n_iter, scan_type}{Bookkeeping fields.}
#'   }
#'
#' @references
#' Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L. H.
#' (2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
#' Statistics}, 32, 953-978.
#'
#' Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
#' Statistics - Theory and Methods}, 26(6), 1481-1496.
#'
#' Zhang, Z., Assuncao, R., & Kulldorff, M. (2010). Spatial scan statistics
#' adjusted for multiple clusters. \emph{Journal of Probability and
#' Statistics}, 2010, 642379.
#'
#' Holm, S. (1979). A simple sequentially rejective multiple test procedure.
#' \emph{Scandinavian Journal of Statistics}, 6(2), 65-70.
#'
#' @seealso \code{\link{treespatial_scan}}, \code{\link{circular_scan}},
#'   \code{\link{tree_scan}}, \code{\link{filter_clusters}},
#'   \code{\link{get_cluster_regions}}
#'
#' @export
#' @examples
#' \donttest{
#' data(london_collisions); data(london_tree)
#' result <- iterative_scan(
#'   cases       = london_collisions$cases,
#'   population  = london_collisions$population,
#'   region_id   = london_collisions$region_id,
#'   x           = london_collisions$x,
#'   y           = london_collisions$y,
#'   node_id     = london_collisions$node_id,
#'   tree        = london_tree,
#'   max_iter = 3, nsim = 99, seed = 42
#' )
#' print(result)
#' }
iterative_scan <- function(cases = NULL, population = NULL,
                           region_id = NULL, x = NULL, y = NULL,
                           node_id = NULL,
                           tree = NULL,
                           tree_node_id = NULL, tree_parent_id = NULL,
                           max_iter = 5L, alpha = 0.05,
                           nsim = 999L, max_pop_pct = 0.5,
                           model = c("poisson", "binomial"),
                           seed = NULL, verbose = TRUE,
                           n_cores = 1L) {

  model <- match.arg(model)

  has_tree <- !is.null(tree) || (!is.null(tree_node_id) &&
                                  !is.null(tree_parent_id))
  has_node <- !is.null(node_id)
  has_geo  <- !is.null(region_id) && !is.null(x) && !is.null(y)

  if (has_tree && has_node && has_geo) {
    scan_type <- "treespatial"
    tree <- .normalize_tree(tree, tree_node_id, tree_parent_id)
  } else if (!has_tree && has_geo) {
    scan_type <- "circular"
  } else if (has_tree && !has_node && !has_geo) {
    scan_type <- "tree"
    tree <- .normalize_tree(tree, tree_node_id, tree_parent_id)
  } else {
    stop("Inputs are not consistent. Provide:\n",
         "  tree-spatial: cases, population, region_id, x, y, node_id, tree\n",
         "  circular:     cases, population, region_id, x, y\n",
         "  tree-only:    cases, population, tree (no region_id/x/y/node_id)",
         call. = FALSE)
  }

  # State carried across iterations
  if (scan_type == "treespatial") {
    prepared <- .build_inputs(cases, population, region_id, x, y,
                              node_id, tree)
    regions       <- prepared$regions
    cases_current <- prepared$cases_matrix    # matrix
  } else if (scan_type == "circular") {
    regions       <- .build_regions_circular(cases, population,
                                              region_id, x, y)
    cases_current <- as.numeric(cases)        # vector
  } else {
    # tree-only
    if (is.null(population)) {
      stop("Tree-only scan requires 'population' (vector of leaf-level ",
           "denominators).", call. = FALSE)
    }
    cases_current <- as.numeric(cases)
    regions <- NULL
  }

  clusters_rows <- list()
  iterations <- list()

  for (k in seq_len(max_iter)) {
    iter_seed <- if (is.null(seed)) NULL else seed + k - 1L

    if (verbose) message(sprintf("Iteration %d/%d ...", k, max_iter))

    if (scan_type == "treespatial") {
      # Convert cases matrix back to vectors for treespatial_scan
      vec <- .matrix_to_vectors(cases_current, regions, tree)
      res <- treespatial_scan(
        cases       = vec$cases,
        population  = vec$population,
        region_id   = vec$region_id,
        x           = vec$x,
        y           = vec$y,
        node_id     = vec$node_id,
        tree        = tree,
        max_pop_pct = max_pop_pct,
        nsim        = nsim, alpha = alpha,
        model       = model,
        seed        = iter_seed, n_cores = n_cores
      )
    } else if (scan_type == "circular") {
      res <- circular_scan(
        cases       = cases_current,
        population  = regions$population,
        region_id   = regions$region_id,
        x           = regions$x,
        y           = regions$y,
        max_pop_pct = max_pop_pct,
        nsim        = nsim, alpha = alpha,
        model       = model,
        seed        = iter_seed, n_cores = n_cores
      )
    } else {
      res <- tree_scan(tree, cases_current, population = population,
                       nsim = nsim, alpha = alpha,
                       model = model,
                       seed = iter_seed, n_cores = n_cores)
    }

    mlc <- res$most_likely_cluster
    pvalue <- res$pvalue

    # Stop only if the residual signal is gone (no cluster found, LLR=0).
    # Otherwise, continue collecting iterations - significance is decided
    # AT THE END via Holm-Bonferroni multiple-testing correction.
    if (is.null(mlc$llr) || mlc$llr <= 0 ||
        is.null(mlc$cases) || mlc$cases == 0) {
      if (verbose) {
        message(sprintf("  Iteration %d: no residual signal (LR=0). Stopping.",
                        k))
      }
      break
    }

    # --- Record this cluster ---
    if (scan_type == "tree") {
      row <- data.frame(
        iteration  = k,
        node_id    = as.character(mlc$node_id),
        cases      = mlc$cases,
        expected   = mlc$expected,
        population = mlc$population,
        llr        = mlc$llr,
        pvalue     = pvalue,
        stringsAsFactors = FALSE
      )
      row$leaf_ids <- I(list(mlc$leaf_ids))
    } else {
      row <- data.frame(
        iteration  = k,
        node_id    = if (has_tree) as.character(mlc$node_id) else NA_character_,
        n_regions  = length(mlc$region_ids),
        cases      = mlc$cases,
        expected   = mlc$expected,
        population = mlc$population,
        rr         = mlc$rr,
        llr        = mlc$llr,
        pvalue     = pvalue,
        stringsAsFactors = FALSE
      )
      row$region_ids <- I(list(mlc$region_ids))
    }

    clusters_rows[[k]] <- row
    iterations[[k]] <- res

    if (verbose) {
      if (scan_type == "tree") {
        message(sprintf("  Cluster %d: node = %s, LR = %.2f, raw p = %.3f",
                        k, mlc$node_id, mlc$llr, pvalue))
      } else if (scan_type == "treespatial") {
        message(sprintf("  Cluster %d: node = %s, %d regions, LR = %.2f, raw p = %.3f",
                        k, mlc$node_id, length(mlc$region_ids), mlc$llr, pvalue))
      } else {
        message(sprintf("  Cluster %d: %d regions, LR = %.2f, raw p = %.3f",
                        k, length(mlc$region_ids), mlc$llr, pvalue))
      }
    }

    # --- Remove cases for next iteration ---
    cases_current <- .zero_cluster_cases(cases_current, mlc, tree, regions,
                                         scan_type)
  }

  # --- Apply Holm-Bonferroni correction across the m collected iterations ---
  if (length(clusters_rows) > 0) {
    raw_p   <- vapply(clusters_rows, function(r) r$pvalue, numeric(1))
    adj_p   <- stats::p.adjust(raw_p, method = "holm")
    sig_vec <- adj_p < alpha
    for (k in seq_along(clusters_rows)) {
      clusters_rows[[k]]$pvalue_adjusted <- adj_p[k]
      clusters_rows[[k]]$significant     <- sig_vec[k]
    }
    if (verbose) {
      message(sprintf("\nHolm-Bonferroni adjustment over %d iteration(s):",
                      length(clusters_rows)))
      for (k in seq_along(clusters_rows)) {
        message(sprintf("  Iter %d: raw p = %.3f, adjusted p = %.3f -> %s",
                        k, raw_p[k], adj_p[k],
                        if (sig_vec[k]) "SIGNIFICANT" else "not significant"))
      }
    }
  }

  # For tree-only mode, regions wasn't created in this scope
  out_regions <- if (scan_type == "tree") NULL else regions

  out <- list(
    clusters   = if (length(clusters_rows) > 0) do.call(rbind, clusters_rows)
                 else data.frame(),
    iterations = iterations,
    regions    = out_regions,
    tree       = tree,
    alpha      = alpha,
    n_iter     = length(clusters_rows),
    scan_type  = scan_type
  )
  class(out) <- "iterative_scan"
  out
}


# ---- Helper: zero out cluster cases for next iteration ----
#' @keywords internal
.zero_cluster_cases <- function(cases, mlc, tree, regions, scan_type) {

  if (scan_type == "circular") {
    region_idx <- match(mlc$region_ids, regions$region_id)
    region_idx <- region_idx[!is.na(region_idx)]
    cases[region_idx] <- 0
    return(cases)
  }

  if (scan_type == "tree") {
    # Tree-only: zero leaves under the detected node
    leaves <- .get_leaves(tree)
    desc <- .get_descendants(tree, as.character(mlc$node_id))
    desc_leaves <- intersect(desc, leaves)
    leaf_idx <- match(desc_leaves, leaves)
    leaf_idx <- leaf_idx[!is.na(leaf_idx)]
    cases[leaf_idx] <- 0
    return(cases)
  }

  # Tree-spatial: zero leaf descendants x regions
  node_id <- as.character(mlc$node_id)
  descendants <- .get_descendants(tree, node_id)

  parents_set <- unique(tree$parent_id[!is.na(tree$parent_id)])
  all_leaves <- setdiff(tree$node_id, parents_set)
  desc_leaves <- intersect(descendants, all_leaves)

  leaf_idx <- match(desc_leaves, rownames(cases))
  leaf_idx <- leaf_idx[!is.na(leaf_idx)]

  region_idx <- match(mlc$region_ids, regions$region_id)
  region_idx <- region_idx[!is.na(region_idx)]

  if (length(leaf_idx) > 0 && length(region_idx) > 0) {
    cases[leaf_idx, region_idx] <- 0L
  }
  cases
}


#' Print Method for iterative_scan Objects
#'
#' @param x An object of class \code{"iterative_scan"}.
#' @param max_show Integer. Accepted for API consistency with
#'   \code{print.treespatial_scan()}, \code{print.tree_scan()}, and
#'   \code{print.circular_scan()}. Currently the only field affected
#'   is the per-iteration \code{leaf_ids} column of \code{x$clusters},
#'   which is omitted from the printed table by default to keep the
#'   summary compact; pass \code{max_show = -1L} to include both
#'   \code{leaf_ids} and \code{region_ids} in the printed table.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return Invisibly returns \code{x} (the input object of class
#'   \code{"iterative_scan"}), unchanged. Called for the side
#'   effect of printing a human-readable summary of the iterative
#'   scan result to the console: the scan type, the number of
#'   iterations performed, the significance threshold (applied to
#'   Holm-Bonferroni adjusted p-values), the number of significant
#'   clusters after correction, and a per-iteration table of
#'   detected clusters.
#' @export
print.iterative_scan <- function(x, max_show = 10L, ...) {
  cat("Iterative Scan\n")
  cat(paste(rep("-", 50), collapse = ""), "\n")
  cat("Scan type:", x$scan_type, "\n")
  cat("Iterations performed:", x$n_iter, "\n")
  cat("Alpha:", x$alpha,
      "  (applied to Holm-Bonferroni adjusted p-values)\n\n")

  if (nrow(x$clusters) > 0) {
    n_sig <- sum(x$clusters$significant, na.rm = TRUE)
    cat("Significant clusters after Holm-Bonferroni: ", n_sig,
        " of ", nrow(x$clusters), "\n\n", sep = "")

    cat("All iterations:\n")
    display <- x$clusters
    if (!isTRUE(max_show < 0L)) {
      display$region_ids <- NULL
      display$leaf_ids <- NULL
    }
    print(display, row.names = FALSE)
  } else {
    cat("No significant clusters detected.\n")
  }

  invisible(x)
}

#' Generate Example Data for Tree-Spatial Scan
#'
#' Creates a synthetic dataset for demonstrating and testing the tree-spatial
#' scan statistic. Returns parallel vectors (\code{cases}, \code{population},
#' \code{region_id}, \code{x}, \code{y}, \code{node_id}) and a \code{tree},
#' matching the input format expected by \code{\link{treespatial_scan}}.
#'
#' @param n_regions Integer. Default 50.
#' @param pop_per_region Numeric. Default 1000.
#' @param cluster_regions Integer vector. Default \code{1:7}.
#' @param cluster_leaves Integer vector. Default \code{c(3, 4)}.
#' @param rr Numeric. Relative risk. Default 2.0.
#' @param Cg Integer. Cases per branch. Default 200.
#' @param seed Integer or \code{NULL}. Random seed. When non-\code{NULL},
#'   the user's pre-existing RNG state is saved on entry and restored on
#'   exit, so the \code{seed} argument affects only the result of the
#'   call. Default \code{NULL} (the user's session-level RNG state is
#'   used as-is and is not modified by the function).
#'
#' @return A list with vector components ready to feed into
#'   \code{\link{treespatial_scan}}: \code{cases}, \code{population},
#'   \code{region_id}, \code{x}, \code{y}, \code{node_id}, plus the
#'   \code{tree} (data.frame) and a \code{true_cluster} list describing the
#'   injected cluster.
#'
#' @export
#' @examples
#' ex <- generate_example_data(seed = 42)
#' dat <- data.frame(
#'   cases = ex$cases, population = ex$population, region_id = ex$region_id,
#'   x = ex$x, y = ex$y, node_id = ex$node_id
#' )
#' result <- treespatial_scan(
#'   dat,
#'   cases       = cases,
#'   population  = population,
#'   region_id   = region_id,
#'   x           = x,
#'   y           = y,
#'   node_id     = node_id,
#'   tree        = ex$tree,
#'   nsim        = 99
#' )
#' print(result)
generate_example_data <- function(n_regions = 50L, pop_per_region = 1000,
                                  cluster_regions = 1:7,
                                  cluster_leaves = c(3, 4),
                                  rr = 2.0, Cg = 200L, seed = NULL) {
  if (!is.null(seed)) {
    .snap__ <- .seed_save_and_set(seed)
    on.exit(.seed_restore(.snap__), add = TRUE)
  }

  sq <- ceiling(sqrt(n_regions))
  grid_x <- rep(seq_len(sq), each = sq)[seq_len(n_regions)]
  grid_y <- rep(seq_len(sq), times = sq)[seq_len(n_regions)]
  grid_x <- grid_x + 0.5 * (grid_y %% 2)

  region_ids <- seq_len(n_regions)
  pop_vec    <- rep(pop_per_region, n_regions)

  tree <- data.frame(
    node_id   = 1:15,
    parent_id = c(NA, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7)
  )
  n_leaves <- 8
  leaf_node_ids <- 8:15

  N <- n_regions * pop_per_region
  prob_base <- pop_vec / N

  cases_mat <- matrix(0L, nrow = n_leaves, ncol = n_regions)

  for (leaf in seq_len(n_leaves)) {
    if (leaf %in% cluster_leaves) {
      prob_cluster <- prob_base
      prob_cluster[cluster_regions] <- prob_cluster[cluster_regions] * rr
      prob_cluster <- prob_cluster / sum(prob_cluster)
      cases_mat[leaf, ] <- as.integer(stats::rmultinom(1, Cg, prob_cluster))
    } else {
      cases_mat[leaf, ] <- as.integer(stats::rmultinom(1, Cg, prob_base))
    }
  }

  # Convert to long parallel vectors
  pos <- which(cases_mat > 0, arr.ind = TRUE)
  cases_vec   <- cases_mat[pos]
  region_vec  <- region_ids[pos[, "col"]]
  node_vec    <- leaf_node_ids[pos[, "row"]]
  pop_long    <- pop_vec[pos[, "col"]]
  x_long      <- as.numeric(grid_x)[pos[, "col"]]
  y_long      <- as.numeric(grid_y)[pos[, "col"]]

  true_leaf_nodes <- leaf_node_ids[cluster_leaves]
  true_cluster <- list(
    region_ids    = cluster_regions,
    leaf_indices  = cluster_leaves,
    leaf_node_ids = true_leaf_nodes,
    rr            = rr
  )

  list(
    cases        = as.integer(cases_vec),
    population   = pop_long,
    region_id    = as.integer(region_vec),
    x            = x_long,
    y            = y_long,
    node_id      = as.integer(node_vec),
    tree         = tree,
    true_cluster = true_cluster
  )
}

test_that("sequential_scan returns valid object structure (tree-spatial)", {
  set.seed(1L)
  ex <- generate_example_data(n_regions = 20L, seed = 11L)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res <- sequential_scan(dat, cases, population, region_id, x, y, node_id,
                         tree = ex$tree, max_iter = 3L, nsim = 29L, seed = 7L,
                         verbose = FALSE)

  expect_s3_class(res, "sequential_scan")
  expect_true(all(c("clusters", "iterations", "regions", "tree",
                    "alpha", "nsim", "buffer_size", "n_iter",
                    "scan_type") %in% names(res)))
  expect_equal(res$scan_type, "treespatial")
  expect_true(is.data.frame(res$clusters))
  expect_true(all(c("iteration", "node_id", "n_regions", "cases",
                    "expected", "rr", "llr", "pvalue", "significant")
                  %in% names(res$clusters)))
  expect_gte(res$n_iter, 1L)
  expect_lte(res$n_iter, 3L)
  expect_equal(nrow(res$clusters), res$n_iter)
})

test_that("sequential_scan respects max_iter", {
  set.seed(2L)
  ex <- generate_example_data(n_regions = 18L, seed = 13L)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res <- sequential_scan(dat, cases, population, region_id, x, y, node_id,
                         tree = ex$tree, max_iter = 2L, nsim = 29L, seed = 1L,
                         verbose = FALSE)
  expect_lte(nrow(res$clusters), 2L)
})

test_that("sequential_scan stops at first non-significant MLC", {
  set.seed(3L)
  # No injected clusters: scan should detect at most one (likely non-sig)
  n <- 15
  dat <- data.frame(cases = rpois(n, lambda = 5), population = rep(1000, n),
                    region_id = seq_len(n),
                    x = runif(n, 0, 10), y = runif(n, 0, 10))
  res <- sequential_scan(dat, cases, population, region_id, x, y,
                         max_iter = 5L, nsim = 99L, alpha = 0.05,
                         seed = 7L, verbose = FALSE)
  expect_s3_class(res, "sequential_scan")
  expect_equal(res$scan_type, "circular")
  if (nrow(res$clusters) > 1L) {
    expect_true(all(res$clusters$significant[-nrow(res$clusters)]))
  }
})

test_that("sequential_scan with buffer_size > 0 works", {
  set.seed(4L)
  ex <- generate_example_data(n_regions = 25L, seed = 21L)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res0 <- sequential_scan(dat, cases, population, region_id, x, y, node_id,
                          tree = ex$tree, max_iter = 2L, nsim = 29L,
                          buffer_size = 0L, seed = 11L, verbose = FALSE)
  res2 <- sequential_scan(dat, cases, population, region_id, x, y, node_id,
                          tree = ex$tree, max_iter = 2L, nsim = 29L,
                          buffer_size = 2L, seed = 11L, verbose = FALSE)
  expect_equal(res0$buffer_size, 0L)
  expect_equal(res2$buffer_size, 2L)
  expect_s3_class(res0, "sequential_scan")
  expect_s3_class(res2, "sequential_scan")
})

test_that("sequential_scan circular mode works", {
  set.seed(5L)
  n <- 20
  cases <- rpois(n, lambda = 8)
  cases[1:4] <- rpois(4, lambda = 40)
  dat <- data.frame(cases = cases, population = rep(500, n),
                    region_id = seq_len(n),
                    x = runif(n, 0, 10), y = runif(n, 0, 10))
  res <- sequential_scan(dat, cases, population, region_id, x, y,
                         max_iter = 3L, nsim = 49L, seed = 2L, verbose = FALSE)
  expect_s3_class(res, "sequential_scan")
  expect_equal(res$scan_type, "circular")
})

test_that("sequential_scan tree-only mode works", {
  tree <- data.frame(
    node_id   = c(1, 2, 3, 4, 5, 6, 7),
    parent_id = c(NA, 1, 1, 2, 2, 3, 3)
  )
  # leaves are 4, 5, 6, 7
  dat <- data.frame(node_id = c(4, 5, 6, 7), cases = c(60, 5, 4, 3),
                    population = c(100, 100, 100, 100))
  res <- sequential_scan(dat, cases, population, node_id = node_id,
                         tree = tree, max_iter = 2L, nsim = 49L, seed = 9L,
                         verbose = FALSE)
  expect_s3_class(res, "sequential_scan")
  expect_equal(res$scan_type, "tree")
  expect_null(res$regions)
})

test_that("sequential_scan print runs without error", {
  set.seed(6L)
  ex <- generate_example_data(n_regions = 14L, seed = 31L)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res <- sequential_scan(dat, cases, population, region_id, x, y, node_id,
                         tree = ex$tree, max_iter = 2L, nsim = 19L, seed = 1L,
                         verbose = FALSE)
  expect_output(print(res), "Sequential Scan")
})

test_that("treespatial_scan requires data and validates columns", {
  tree <- data.frame(node_id = c(1, 2, 3), parent_id = c(NA, 1, 1))
  dat <- data.frame(cases = 1:3, population = c(100, 100, 100),
                    region_id = 1:3, x = 1:3, y = 1:3, node_id = c(2, 2, 3))
  # `data` is required
  expect_error(treespatial_scan(), "is required")
  # unknown column name is reported
  expect_error(
    treespatial_scan(dat, cases = nope, population = population,
                     region_id = region_id, x = x, y = y, node_id = node_id,
                     tree = tree),
    "nope"
  )
})

test_that("treespatial_scan returns expected class and structure", {
  ex <- generate_example_data(n_regions = 16, seed = 1)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res <- treespatial_scan(dat, cases, population, region_id, x, y, node_id,
                          tree = ex$tree, nsim = 29, seed = 42)
  expect_s3_class(res, "treespatial_scan")
  expect_true(all(c("most_likely_cluster", "pvalue", "nsim") %in% names(res)))
  expect_gte(res$most_likely_cluster$llr, 0)
  expect_gte(res$pvalue, 0); expect_lte(res$pvalue, 1)
})

test_that("treespatial_scan detects injected cluster", {
  ex <- generate_example_data(n_regions = 25, cluster_regions = 1:5,
                              cluster_leaves = c(3, 4),
                              rr = 4.0, Cg = 300L, seed = 77)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res <- treespatial_scan(dat, cases, population, region_id, x, y, node_id,
                          tree = ex$tree, nsim = 199, seed = 77)
  detected <- res$most_likely_cluster$region_ids
  injected <- ex$true_cluster$region_ids
  overlap <- length(intersect(detected, injected)) / length(injected)
  expect_gte(overlap, 0.5)
})

test_that("treespatial_scan accepts a column name held in a variable", {
  ex <- generate_example_data(n_regions = 16, seed = 1)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  pop_col <- "population"          # name held in a variable (programmatic use)
  res <- treespatial_scan(dat, cases, population = pop_col,
                          region_id = region_id, x = x, y = y,
                          node_id = node_id, tree = ex$tree,
                          nsim = 19, seed = 1)
  res2 <- treespatial_scan(dat, cases, population = population,
                           region_id = region_id, x = x, y = y,
                           node_id = node_id, tree = ex$tree,
                           nsim = 19, seed = 1)
  expect_s3_class(res, "treespatial_scan")
  expect_equal(res$most_likely_cluster$llr, res2$most_likely_cluster$llr)
})

test_that("treespatial_scan accepts tree_node_id/tree_parent_id form", {
  ex <- generate_example_data(n_regions = 16, seed = 1)
  dat <- data.frame(cases = ex$cases, population = ex$population,
                    region_id = ex$region_id, x = ex$x, y = ex$y,
                    node_id = ex$node_id)
  res1 <- treespatial_scan(dat, cases, population, region_id, x, y, node_id,
                           tree = ex$tree, nsim = 19, seed = 1)
  res2 <- treespatial_scan(dat, cases, population, region_id, x, y, node_id,
                           tree_node_id = ex$tree$node_id,
                           tree_parent_id = ex$tree$parent_id,
                           nsim = 19, seed = 1)
  expect_equal(res1$most_likely_cluster$llr,
               res2$most_likely_cluster$llr)
})

#' Kulldorff's Circular Spatial Scan Statistic
#'
#' Performs Kulldorff's circular spatial scan statistic for detecting
#' spatial clusters. Inputs are supplied through a single \code{data}
#' data.frame with one row per region (cases must already be aggregated to
#' the region level); the relevant columns are referenced by name.
#'
#' @param data A \code{data.frame} with one row per region. The
#'   \code{cases}, \code{population}, \code{region_id}, \code{x} and \code{y}
#'   arguments name columns of this data.frame.
#' @param cases Column of \code{data} giving the total cases in each region.
#'   Given as an unquoted column name (a string or expression also works).
#' @param population Column of \code{data} giving the population (or
#'   denominator) of each region.
#' @param region_id Column of \code{data} giving the region identifier.
#' @param x,y Columns of \code{data} giving the region centroid coordinates.
#' @param max_pop_pct Numeric. Default \code{0.5}.
#' @param nsim Integer. Number of MC simulations. Default \code{999}.
#' @param alpha Numeric. Significance level. Default \code{0.05}.
#' @param n_secondary Integer. Default \code{1000}.
#' @param model Character. \code{"poisson"} or \code{"binomial"}.
#' @param seed Integer or \code{NULL}. Random seed for the Monte
#'   Carlo loop. When non-\code{NULL}, the user's pre-existing RNG
#'   state is saved on entry and restored on exit, so the seed
#'   argument affects only this call and does not leak into
#'   subsequent draws in the user's session.
#' @param n_cores Integer. OpenMP threads.
#'
#' @return An object of class \code{"circular_scan"}.
#'
#' @references
#' Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
#' Statistics - Theory and Methods}, 26(6), 1481-1496.
#'
#' @seealso \code{\link{filter_clusters}}, \code{\link{tree_scan}},
#'   \code{\link{treespatial_scan}}, \code{\link{get_cluster_regions}},
#'   \code{\link{sequential_scan}}
#'
#' @export
#' @examples
#' set.seed(42)
#' n <- 20
#' regions <- data.frame(
#'   region_id  = 1:n,
#'   cases      = rpois(n, lambda = 10),
#'   population = rep(1000, n),
#'   x          = runif(n, 0, 10),
#'   y          = runif(n, 0, 10)
#' )
#' regions$cases[1:5] <- rpois(5, lambda = 30)
#'
#' result <- circular_scan(
#'   regions,
#'   cases       = cases,
#'   population  = population,
#'   region_id   = region_id,
#'   x           = x,
#'   y           = y,
#'   nsim        = 99
#' )
#' print(result)
circular_scan <- function(data, cases, population, region_id, x, y,
                          max_pop_pct = 0.5, nsim = 999L,
                          alpha = 0.05, n_secondary = 1000L,
                          model = c("poisson", "binomial"),
                          seed = NULL, n_cores = 1L) {

  ## --- resolve data/column inputs (substitutive interface, treeSS >= 0.2.0) ---
  if (missing(data)) {
    stop("`data` is required: a data.frame with one row per region.",
         call. = FALSE)
  }
  data <- as.data.frame(data)
  .env <- parent.frame()
  cases      <- .resolve_col(substitute(cases),      data, .env, "cases")
  population <- .resolve_col(substitute(population),  data, .env, "population")
  region_id  <- .resolve_col(substitute(region_id),   data, .env, "region_id")
  x          <- .resolve_col(substitute(x),           data, .env, "x")
  y          <- .resolve_col(substitute(y),           data, .env, "y")

  model <- match.arg(model)
  model_int <- if (model == "binomial") 1L else 0L

  regions <- .build_regions_circular(cases, population, region_id, x, y)

  .validate_regions(regions)

  if (!is.null(seed)) {
    .snap__ <- .seed_save_and_set(seed)
    on.exit(.seed_restore(.snap__), add = TRUE)
  }

  n <- nrow(regions)
  N <- sum(regions$population)
  C <- sum(cases)
  max_pop <- max_pop_pct * N

  zones <- build_zones(regions, max_pop = max_pop)

  if (length(zones) == 0) {
    stop("No valid candidate zones found.", call. = FALSE)
  }

  # C++ backend for Monte Carlo
  prob <- regions$population / N
  zone_csr <- .zones_to_csr(zones)

  cpp_result <- mc_spatial_cpp(
    as.numeric(cases),
    as.numeric(N),
    as.numeric(C),
    zone_csr$zone_region_idx,
    zone_csr$zone_ptr,
    zone_csr$zone_pop,
    as.numeric(prob),
    as.integer(nsim),
    model_int,
    as.integer(max(1L, n_cores))
  )

  obs_llr <- cpp_result$obs_max_llr
  best_z <- cpp_result$obs_best_z  # 1-based
  sim_llr <- cpp_result$sim_llr

  pvalue <- (sum(sim_llr >= obs_llr) + 1) / (nsim + 1)

  best_idx <- zones[[best_z]]$region_idx
  best_pop <- zones[[best_z]]$population
  best_cases <- sum(cases[best_idx])
  expected <- C * best_pop / N

  # --- Compute LLR for all zones (for secondary clusters) ---
  n_zones <- length(zones)
  zone_cases <- numeric(n_zones)
  zone_pop_vec <- numeric(n_zones)
  zone_centers <- integer(n_zones)
  zone_sizes <- integer(n_zones)

  for (z in seq_len(n_zones)) {
    idx <- zones[[z]]$region_idx
    zone_cases[z] <- sum(cases[idx])
    zone_pop_vec[z] <- zones[[z]]$population
    zone_centers[z] <- zones[[z]]$center
    zone_sizes[z] <- length(idx)
  }

  zone_expected <- C * zone_pop_vec / N
  zone_cases_bar <- C - zone_cases
  zone_expected_bar <- C - zone_expected
  zone_pop_bar <- N - zone_pop_vec

  if (model == "binomial") {
    rate_in  <- zone_cases / pmax(zone_pop_vec, 1e-300)
    rate_out <- zone_cases_bar / pmax(zone_pop_bar, 1e-300)
    mask <- rate_in > rate_out & zone_cases > 0
    p0 <- C / N

    t1  <- ifelse(mask & zone_cases > 0,
                  zone_cases * log(pmax(rate_in, 1e-300)), 0)
    t1b <- ifelse(mask & (zone_pop_vec - zone_cases) > 0,
                  (zone_pop_vec - zone_cases) * log(pmax(1 - rate_in, 1e-300)), 0)
    t2  <- ifelse(mask & zone_cases_bar > 0,
                  zone_cases_bar * log(pmax(rate_out, 1e-300)), 0)
    t2b <- ifelse(mask & (zone_pop_bar - zone_cases_bar) > 0,
                  (zone_pop_bar - zone_cases_bar) * log(pmax(1 - rate_out, 1e-300)), 0)
    t0  <- C * log(pmax(p0, 1e-300)) + (N - C) * log(pmax(1 - p0, 1e-300))
    zone_llr <- ifelse(mask, t1 + t1b + t2 + t2b - t0, 0)
    zone_llr[zone_llr < 0] <- 0
  } else {
    mask <- zone_cases > zone_expected & zone_cases > 0 & zone_expected > 0
    t1 <- ifelse(mask, zone_cases * log(pmax(zone_cases, 1e-300) /
                                         pmax(zone_expected, 1e-300)), 0)
    t2 <- ifelse(mask & zone_cases_bar > 0,
                 zone_cases_bar * log(pmax(zone_cases_bar, 1e-300) /
                                       pmax(zone_expected_bar, 1e-300)), 0)
    zone_llr <- ifelse(mask, t1 + t2, 0)
  }

  # Keep top n_secondary zones
  top_idx <- order(-zone_llr)
  top_idx <- top_idx[seq_len(min(n_secondary, sum(zone_llr > 0)))]

  all_pairs <- data.frame(
    center     = zone_centers[top_idx],
    n_regions  = zone_sizes[top_idx],
    cases      = zone_cases[top_idx],
    expected   = zone_expected[top_idx],
    population = zone_pop_vec[top_idx],
    llr        = zone_llr[top_idx],
    stringsAsFactors = FALSE
  )

  result <- list(
    most_likely_cluster = list(
      zone_idx = best_idx,
      region_ids = regions$region_id[best_idx],
      center = zones[[best_z]]$center,
      cases = best_cases,
      expected = expected,
      population = best_pop,
      rr = (best_cases / best_pop) / (C / N),
      llr = obs_llr
    ),
    pvalue = pvalue,
    alpha = alpha,
    nsim = nsim,
    total_cases = C,
    total_population = N,
    regions = regions,
    simulated_llr = sim_llr,
    secondary_clusters = all_pairs,
    model = model
  )

  class(result) <- "circular_scan"
  result
}

test_that("circular_scan returns correct class and structure", {
  set.seed(1)
  n <- 20
  dat <- data.frame(
    cases      = rpois(n, lambda = 10),
    population = rep(1000, n),
    region_id  = 1:n,
    x          = runif(n, 0, 10),
    y          = runif(n, 0, 10)
  )
  res <- circular_scan(dat, cases, population, region_id, x, y,
                       nsim = 49, seed = 42)
  expect_s3_class(res, "circular_scan")
  expect_true(all(c("most_likely_cluster", "pvalue") %in% names(res)))
})

test_that("circular_scan detects obvious cluster", {
  set.seed(99)
  n <- 25
  dat <- data.frame(
    cases      = c(rpois(5, 50), rpois(n - 5, 5)),
    population = rep(1000, n),
    region_id  = 1:n,
    x          = c(rep(0, 5), runif(n - 5, 5, 15)),
    y          = c(rep(0, 5), runif(n - 5, 5, 15))
  )
  res <- circular_scan(dat, cases, population, region_id, x, y,
                       nsim = 199, seed = 99)
  expect_lt(res$pvalue, 0.10)
  expect_true(all(1:3 %in% res$most_likely_cluster$region_ids))
})

test_that("circular_scan errors when cases is not aggregated", {
  # 4 rows, 2 unique regions -> not aggregated
  dat <- data.frame(
    cases      = c(5, 3, 4, 2),
    population = c(1000, 1000, 2000, 2000),
    region_id  = c(1, 1, 2, 2),
    x          = c(1, 1, 2, 2),
    y          = c(1, 1, 2, 2)
  )
  expect_error(
    circular_scan(dat, cases, population, region_id, x, y),
    "must already be aggregated"
  )
})

#' Extract Cluster Membership for Each Region
#'
#' Returns a data.frame indicating which cluster (if any) each region
#' belongs to. This is the primary output for visualization — use it with
#' any mapping package (ggplot2, leaflet, tmap, etc.).
#'
#' This is a generic with methods for objects returned by
#' \code{\link{treespatial_scan}}, \code{\link{circular_scan}}, and
#' \code{\link{sequential_scan}}.
#'
#' @param result A \code{"treespatial_scan"} or \code{"circular_scan"}
#'   object (single-pass scan), or a \code{"sequential_scan"} object
#'   (Zhang, Assuncao and Kulldorff, 2010).
#' @param n_clusters Integer. (Single-pass methods only.) Number of
#'   clusters to extract. \code{1} returns only the most likely cluster.
#'   Values greater than 1 use \code{\link{filter_clusters}} internally
#'   to identify distinct secondary clusters. Default is \code{1}.
#'   Ignored for sequential scan inputs (all detected iterations are
#'   returned).
#' @param overlap Logical. If \code{TRUE} (default), returns a facet-ready
#'   data.frame: all regions are replicated for each cluster panel, with a
#'   \code{panel} column for faceting and \code{cluster} marked only for
#'   member regions. If \code{FALSE}, returns one row per region assigned
#'   to its highest-ranked cluster (for single-panel maps).
#' @param ... Further arguments passed to methods.
#'
#' @return A \code{data.frame} with columns from \code{result$regions} plus:
#'   \describe{
#'     \item{cluster}{Integer cluster number (1 = most likely / first
#'       iteration, 2 = first secondary / second iteration, etc.), or
#'       \code{NA} if the region is not in the cluster.}
#'     \item{node_id}{The tree node of the cluster, or \code{NA}.}
#'     \item{llr}{The log-likelihood ratio of the cluster, or \code{NA}.}
#'     \item{pvalue}{The p-value of the cluster, or \code{NA}.}
#'     \item{panel}{(Only when \code{overlap = TRUE}) A two-line label
#'       for \code{facet_wrap}, with the cluster identifier on the first
#'       line and the test statistic on the second. For single-pass
#'       scans the label looks like \code{"#1 P209\\n(LR=39.6)"}; for
#'       sequential scans it looks like
#'       \code{"Iter 1: P209\\n(LR=39.6, p=0.005)"}. The newline keeps
#'       long node identifiers from overflowing the strip in multi-panel
#'       layouts.}
#'   }
#'
#' @seealso \code{\link{treespatial_scan}}, \code{\link{circular_scan}},
#'   \code{\link{sequential_scan}}, \code{\link{filter_clusters}}
#'
#' @examples
#' data(london_collisions); data(london_tree)
#' result <- treespatial_scan(
#'   london_collisions,
#'   cases       = cases,
#'   population  = population,
#'   region_id   = region_id,
#'   x           = x,
#'   y           = y,
#'   node_id     = node_id,
#'   tree        = london_tree,
#'   nsim        = 99, seed = 42
#' )
#'
#' # Long format suitable for merging with a polygon layer.
#' cr <- get_cluster_regions(result, n_clusters = 2L, overlap = TRUE)
#' head(cr)
#'
#' @export
get_cluster_regions <- function(result, n_clusters = 1L, overlap = TRUE,
                                 ...) {
  UseMethod("get_cluster_regions")
}


#' @rdname get_cluster_regions
#' @export
get_cluster_regions.default <- function(result, n_clusters = 1L,
                                          overlap = TRUE, ...) {

  if (!inherits(result, "treespatial_scan") &&
      !inherits(result, "circular_scan")) {
    stop("'result' must be a treespatial_scan, circular_scan, ",
         "or sequential_scan object.", call. = FALSE)
  }
  .get_cluster_regions_singlepass(result, n_clusters, overlap)
}


#' @rdname get_cluster_regions
#' @export
get_cluster_regions.sequential_scan <- function(result, n_clusters = 1L,
                                                  overlap = TRUE, ...) {

  seq_obj <- result
  if (seq_obj$n_iter == 0) {
    return(data.frame())
  }
  if (is.null(seq_obj$regions)) {
    stop("Sequential scan has no $regions table (tree-only scan). ",
         "Mapping by region is not applicable.", call. = FALSE)
  }

  full_regions <- seq_obj$regions
  parts <- vector("list", seq_obj$n_iter)

  for (k in seq_len(seq_obj$n_iter)) {
    cl_regs <- seq_obj$clusters$region_ids[[k]]
    in_cl   <- full_regions$region_id %in% cl_regs

    cr_k <- cbind(
      full_regions,
      data.frame(
        cluster = ifelse(in_cl, k, NA_integer_),
        node_id = ifelse(in_cl,
                          as.character(seq_obj$clusters$node_id[k]),
                          NA_character_),
        llr     = ifelse(in_cl, seq_obj$clusters$llr[k],     NA_real_),
        pvalue  = ifelse(in_cl, seq_obj$clusters$pvalue[k],  NA_real_),
        stringsAsFactors = FALSE
      )
    )

    if (overlap) {
      sig_marker <- if (isTRUE(seq_obj$clusters$significant[k])) ""
                    else " (n.s.)"
      cr_k$panel <- paste0("Iter ", k, ": ",
                           seq_obj$clusters$node_id[k], "\n",
                           "(LR=", round(seq_obj$clusters$llr[k], 1),
                           ", p=",
                           format.pval(seq_obj$clusters$pvalue[k],
                                        digits = 2),
                           sig_marker, ")")
    }

    parts[[k]] <- cr_k
  }

  if (overlap) {
    do.call(rbind, parts)
  } else {
    # Non-overlap: one row per region, lowest iteration wins.
    full <- do.call(rbind, parts)
    full <- full[order(full$cluster, na.last = TRUE), ]
    full[!duplicated(full$region_id), ]
  }
}


# Internal: original single-pass logic moved out of the generic
.get_cluster_regions_singlepass <- function(result, n_clusters, overlap) {

  regions <- result$regions
  n <- nrow(regions)

  # --- Collect all cluster region assignments ---
  cluster_list <- list()

  # Cluster 1: most likely
  mlc <- result$most_likely_cluster
  mlc_idx <- match(mlc$region_ids, regions$region_id)
  mlc_idx <- mlc_idx[!is.na(mlc_idx)]
  mlc_node <- if (!is.null(mlc$node_id)) as.character(mlc$node_id) else "Zone"
  cluster_list[[1]] <- list(
    idx     = mlc_idx,
    cluster = 1L,
    node_id = mlc_node,
    llr     = mlc$llr,
    pvalue  = result$pvalue
  )

  # Secondary clusters
  if (n_clusters > 1) {
    dc <- filter_clusters(result)
    if (!is.null(dc) && nrow(dc) > 1) {
      zones <- build_zones(regions, max_pop = result$total_population * 0.5)
      sec_rows <- dc[-1, , drop = FALSE]
      n_sec <- min(nrow(sec_rows), n_clusters - 1)
      has_node <- "node_id" %in% names(sec_rows)

      for (k in seq_len(n_sec)) {
        zi <- .find_zone(zones, sec_rows$center[k], sec_rows$n_regions[k])
        if (!is.null(zi)) {
          sec_node <- if (has_node) as.character(sec_rows$node_id[k]) else "Zone"
          cluster_list[[k + 1]] <- list(
            idx     = zones[[zi]]$region_idx,
            cluster = as.integer(k + 1),
            node_id = sec_node,
            llr     = sec_rows$llr[k],
            pvalue  = sec_rows$pvalue[k]
          )
        }
      }
    }
  }

  # --- Build output ---
  if (overlap) {
    # Facet-ready format: ALL regions replicated for each cluster panel.
    # In each panel, cluster members are marked, non-members have NA.
    # Ready for: merge(map, cr) then ggplot + facet_wrap(~ panel)
    rows <- list()

    for (cl in cluster_list) {
      # Skip empty slots that arise when .find_zone() could not locate a
      # secondary cluster's zone in the rebuilt zone family. Iterating over
      # a NULL slot would silently produce a chunk with character(0) panel.
      if (is.null(cl)) next

      chunk <- regions
      chunk$cluster <- NA_integer_
      chunk$node_id <- NA_character_
      chunk$llr     <- NA_real_
      chunk$pvalue  <- NA_real_
      chunk$panel   <- paste0("#", cl$cluster, " ", cl$node_id, "\n",
                              "(LR=", round(cl$llr, 1), ")")

      # Mark cluster members
      chunk$cluster[cl$idx] <- cl$cluster
      chunk$node_id[cl$idx] <- cl$node_id
      chunk$llr[cl$idx]     <- cl$llr
      chunk$pvalue[cl$idx]  <- cl$pvalue

      rows <- c(rows, list(chunk))
    }

    out <- do.call(rbind, rows)
    rownames(out) <- NULL

  } else {
    # Single-map format: one row per region, highest-ranked cluster wins.
    out <- regions
    out$cluster <- NA_integer_
    out$node_id <- NA_character_
    out$llr     <- NA_real_
    out$pvalue  <- NA_real_

    for (cl in cluster_list) {
      if (is.null(cl)) next
      free <- is.na(out$cluster[cl$idx])
      out$cluster[cl$idx[free]] <- cl$cluster
      out$node_id[cl$idx[free]] <- cl$node_id
      out$llr[cl$idx[free]]     <- cl$llr
      out$pvalue[cl$idx[free]]  <- cl$pvalue
    }
  }

  out
}

#' @keywords internal
.find_zone <- function(zones, center, n_regions) {
  for (i in seq_along(zones)) {
    if (zones[[i]]$center == center &&
        length(zones[[i]]$region_idx) == n_regions) {
      return(i)
    }
  }
  NULL
}

test_that("get_cluster_regions dispatches on sequential_scan", {
  data(london_collisions)
  data(london_tree)

  seq_res <- sequential_scan(
    london_collisions,
    cases = cases, population = population, region_id = region_id,
    x = x, y = y, node_id = node_id, tree = london_tree,
    max_iter = 3, nsim = 19, seed = 42,
    max_pop_pct = 0.25, n_cores = 1L, verbose = FALSE
  )

  expect_s3_class(seq_res, "sequential_scan")
  expect_gt(seq_res$n_iter, 0)

  cr <- get_cluster_regions(seq_res, overlap = TRUE)
  expect_s3_class(cr, "data.frame")
  expect_true(all(c("cluster", "node_id", "llr", "pvalue", "panel")
                   %in% names(cr)))
  per_panel <- table(cr$panel)
  expect_true(all(per_panel == nrow(seq_res$regions)))
  expect_equal(nrow(cr), seq_res$n_iter * nrow(seq_res$regions))

  cr_no <- get_cluster_regions(seq_res, overlap = FALSE)
  expect_equal(nrow(cr_no), nrow(seq_res$regions))
  expect_false("panel" %in% names(cr_no))
})


test_that("get_cluster_regions still works on single-pass scans", {
  data(london_collisions)
  data(london_tree)

  res <- treespatial_scan(
    london_collisions,
    cases = cases, population = population, region_id = region_id,
    x = x, y = y, node_id = node_id, tree = london_tree,
    max_pop_pct = 0.25, nsim = 19, seed = 42, n_cores = 1L
  )

  cr1 <- get_cluster_regions(res, n_clusters = 1, overlap = FALSE)
  expect_equal(nrow(cr1), nrow(res$regions))
  expect_true("cluster" %in% names(cr1))
})


test_that("get_cluster_regions on sequential_scan rejects tree-only", {
  data(london_collisions)
  data(london_tree)

  agg <- stats::aggregate(cases ~ node_id, data = london_collisions,
                          FUN = sum)
  agg$population <- rep(sum(london_collisions$population) / nrow(agg),
                        nrow(agg))
  seq_t <- sequential_scan(
    agg, cases = cases, population = population, node_id = node_id,
    tree = london_tree, max_iter = 2, nsim = 19, seed = 42, verbose = FALSE
  )

  expect_error(get_cluster_regions(seq_t),
                regexp = "no \\$regions table")
})

#' Tree-Spatial Scan Statistic
#'
#' Performs the tree-spatial scan statistic, combining Kulldorff's circular
#' scan (spatial clusters) with the tree-based scan (hierarchical data
#' mining). Searches all combinations of spatial zones and tree branches
#' to identify pairs \eqn{(z, g)} with significantly more cases than
#' expected.
#'
#' Inputs are supplied through a single \code{data} data.frame in long
#' format (one row per (region, tree-leaf) observation); the case, population,
#' region, coordinate and leaf columns are referenced by name. The user is
#' responsible for choosing which column to use as \code{population} (e.g.,
#' total population, live births, person-years), making the choice of
#' denominator explicit.
#'
#' \strong{Secondary clusters.} The returned object contains the most likely
#' cluster as well as the full set of evaluated (zone, branch) pairs in
#' \code{secondary_clusters}. To obtain the distinct secondary clusters as
#' described in Section 5.1.1 of Cancado et al. (2025) (filtering out pairs
#' that overlap in regions or branches with already-retained clusters), use
#' \code{\link{filter_clusters}} or \code{\link{get_cluster_regions}} with
#' \code{n_clusters > 1}.
#'
#' @param data A \code{data.frame} in long format with one row per
#'   (region, tree-leaf) observation. The \code{cases}, \code{population},
#'   \code{region_id}, \code{x}, \code{y} and \code{node_id} arguments name
#'   columns of this data.frame.
#' @param cases Column of \code{data} giving the number of cases observed
#'   for each (region, leaf) row. Given as an unquoted column name (a string
#'   or an expression on columns also works).
#' @param population Column of \code{data} giving the population (or
#'   denominator) of the region for each row. The same value should be
#'   repeated across all rows of a given region; if it varies, the first
#'   occurrence per region is used and a warning is issued.
#' @param region_id Column of \code{data} giving the region identifier.
#' @param x,y Columns of \code{data} giving the region centroid coordinates.
#'   Like \code{population}, these should be constant within region.
#' @param node_id Column of \code{data} giving the tree leaf identifier for
#'   each row. Each value must match a leaf of the tree.
#' @param tree A \code{data.frame} with columns \code{node_id} and
#'   \code{parent_id}. The root node(s) must have \code{parent_id = NA}.
#'   As an alternative, pass \code{tree_node_id} and \code{tree_parent_id}
#'   as parallel vectors instead of this argument.
#' @param tree_node_id,tree_parent_id Optional. Parallel vectors describing
#'   the tree edges, used as an alternative to \code{tree}. If both
#'   \code{tree} and these vectors are supplied, an error is raised.
#' @param max_pop_pct Numeric. Maximum proportion of total population
#'   allowed inside a zone. Default \code{0.5}.
#' @param nsim Integer. Number of Monte Carlo simulations. Default \code{999}.
#' @param alpha Numeric. Significance level. Default \code{0.05}.
#' @param model Character. \code{"poisson"} (default) or \code{"binomial"}.
#' @param seed Integer or \code{NULL}. Random seed for the Monte
#'   Carlo loop. When non-\code{NULL}, the user's pre-existing RNG
#'   state is saved on entry and restored on exit, so the seed
#'   argument affects only this call and does not leak into
#'   subsequent draws in the user's session.
#' @param n_cores Integer. OpenMP threads for the Monte Carlo loop.
#'   Default \code{1L} (serial).
#'
#' @return An object of class \code{"treespatial_scan"}.
#'
#' @references
#' Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
#' (2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
#' Statistics}, 32, 953--978. \doi{10.1007/s10651-025-00670-w}
#'
#' @seealso \code{\link{circular_scan}}, \code{\link{tree_scan}},
#'   \code{\link{aggregate_tree}}, \code{\link{filter_clusters}},
#'   \code{\link{get_cluster_regions}}, \code{\link{sequential_scan}}
#'
#' @export
#' @examples
#' set.seed(123)
#' n_regions <- 10
#' tree <- data.frame(
#'   node_id   = c(1, 2, 3, 4, 5, 6, 7),
#'   parent_id = c(NA, 1, 1, 2, 2, 3, 3)
#' )
#' # Long-format data.frame: one row per (region, leaf) combination
#' crimes <- expand.grid(region_id = 1:n_regions, node_id = c(4, 5, 6, 7))
#' xy <- data.frame(region_id = 1:n_regions,
#'                  x = runif(n_regions, 0, 10),
#'                  y = runif(n_regions, 0, 10))
#' crimes <- merge(crimes, xy, by = "region_id")
#' crimes$population <- 1000
#' crimes$cases <- rpois(nrow(crimes), lambda = 5)
#' crimes$cases[crimes$node_id == 4 & crimes$region_id %in% 1:3] <- rpois(3, 30)
#'
#' result <- treespatial_scan(
#'   crimes,
#'   cases       = cases,
#'   population  = population,
#'   region_id   = region_id,
#'   x           = x,
#'   y           = y,
#'   node_id     = node_id,
#'   tree        = tree,
#'   nsim        = 99
#' )
#' print(result)
treespatial_scan <- function(data, cases, population, region_id, x, y, node_id,
                             tree = NULL,
                             tree_node_id = NULL, tree_parent_id = NULL,
                             max_pop_pct = 0.5,
                             nsim = 999L, alpha = 0.05,
                             model = c("poisson", "binomial"),
                             seed = NULL, n_cores = 1L) {

  ## --- resolve data/column inputs (substitutive interface, treeSS >= 0.2.0) ---
  if (missing(data)) {
    stop("`data` is required: a data.frame with one row per (region, leaf).",
         call. = FALSE)
  }
  data <- as.data.frame(data)
  .env <- parent.frame()
  cases      <- .resolve_col(substitute(cases),      data, .env, "cases")
  population <- .resolve_col(substitute(population),  data, .env, "population")
  region_id  <- .resolve_col(substitute(region_id),   data, .env, "region_id")
  x          <- .resolve_col(substitute(x),           data, .env, "x")
  y          <- .resolve_col(substitute(y),           data, .env, "y")
  node_id    <- .resolve_col(substitute(node_id),     data, .env, "node_id")

  model <- match.arg(model)
  model_int <- if (model == "binomial") 1L else 0L

  tree <- .normalize_tree(tree, tree_node_id, tree_parent_id)

  # Build internal regions table + cases matrix from parallel vectors
  prepared <- .build_inputs(cases, population, region_id, x, y, node_id, tree)
  regions      <- prepared$regions
  cases_matrix <- prepared$cases_matrix

  .validate_regions(regions)

  if (!is.null(seed)) {
    .snap__ <- .seed_save_and_set(seed)
    on.exit(.seed_restore(.snap__), add = TRUE)
  }

  n <- nrow(regions)
  N <- sum(regions$population)
  max_pop <- max_pop_pct * N
  leaves <- .get_leaves(tree)
  all_nodes <- tree$node_id
  n_nodes <- length(all_nodes)

  # Aggregate cases from leaves to all nodes (uses the cases_matrix built
  # above, not the user's cases vector)
  full_cases <- .aggregate_leaves_to_all(cases_matrix, tree)

  # Total cases per branch (across all regions)
  Cg <- rowSums(full_cases)
  names(Cg) <- as.character(all_nodes)

  # Build candidate spatial zones
  zones <- build_zones(regions, max_pop = max_pop)
  n_zones <- length(zones)

  if (n_zones == 0) {
    stop("No valid candidate zones found.", call. = FALSE)
  }

  # --- C++ backend: observed scan + Monte Carlo ---
  prob <- regions$population / N

  # Prepare CSR structures
  zone_csr <- .zones_to_csr(zones)
  tree_csr <- .tree_to_csr_children(tree)
  leaf_idx_0 <- as.integer(match(leaves, all_nodes) - 1L)  # 0-based
  depths <- .compute_depths(tree)
  proc_order_0 <- as.integer(order(depths, decreasing = TRUE) - 1L)  # 0-based

  cpp_result <- mc_treespatial_cpp(
    as.matrix(full_cases),
    as.numeric(Cg),
    as.numeric(N),
    zone_csr$zone_region_idx,
    zone_csr$zone_ptr,
    zone_csr$zone_pop,
    leaf_idx_0,
    tree_csr$children_idx,
    tree_csr$children_ptr,
    proc_order_0,
    as.numeric(prob),
    as.integer(nsim),
    model_int,
    as.integer(max(1L, n_cores))
  )

  obs_max_llr <- cpp_result$obs_max_llr
  best_g <- cpp_result$obs_best_g  # 1-based
  best_z <- cpp_result$obs_best_z  # 1-based
  sim_llr <- cpp_result$sim_llr

  # P-value
  pvalue <- (sum(sim_llr >= obs_max_llr) + 1) / (nsim + 1)

  # --- Secondary clusters (R-side, single pass, using matrix ops) ---
  zone_mat <- matrix(0L, nrow = n_zones, ncol = n)
  zone_pop_vec <- numeric(n_zones)
  zone_centers <- integer(n_zones)
  zone_sizes <- integer(n_zones)
  zone_region_list <- vector("list", n_zones)

  for (zi in seq_len(n_zones)) {
    idx <- zones[[zi]]$region_idx
    zone_mat[zi, idx] <- 1L
    zone_pop_vec[zi] <- zones[[zi]]$population
    zone_centers[zi] <- zones[[zi]]$center
    zone_sizes[zi] <- length(idx)
    zone_region_list[[zi]] <- idx
  }

  all_pairs <- .build_pairs_df_r(full_cases, Cg, N, zone_mat, zone_pop_vec,
                                  zone_centers, zone_sizes, all_nodes,
                                  model = model)

  # --- Build result ---
  branches <- .get_branches(tree)
  best_leaves <- branches[[as.character(all_nodes[best_g])]]$leaves
  best_zone_idx <- zone_region_list[[best_z]]
  cases_in_zone <- sum(full_cases[best_g, best_zone_idx])
  expected_in_zone <- Cg[best_g] * zone_pop_vec[best_z] / N

  result <- list(
    most_likely_cluster = list(
      node_id = all_nodes[best_g],
      leaf_ids = best_leaves,
      zone_idx = best_zone_idx,
      region_ids = regions$region_id[best_zone_idx],
      cases = cases_in_zone,
      expected = expected_in_zone,
      population = zone_pop_vec[best_z],
      rr = (cases_in_zone / zone_pop_vec[best_z]) / (Cg[best_g] / N),
      llr = obs_max_llr
    ),
    secondary_clusters = all_pairs,
    pvalue = pvalue,
    alpha = alpha,
    nsim = nsim,
    total_cases_by_branch = Cg,
    total_population = N,
    regions = regions,
    tree = tree,
    full_cases = full_cases,
    simulated_llr = sim_llr,
    model = model
  )

  class(result) <- "treespatial_scan"
  result
}

#' @keywords internal
.build_pairs_df_r <- function(full_cases, Cg, N, zone_mat, zone_pop,
                               zone_centers, zone_sizes, all_nodes,
                               model = "poisson") {
  # Compute all (node, zone) cases via single matrix multiply
  cz_mat <- full_cases %*% t(zone_mat)
  n_nodes <- nrow(cz_mat)
  n_zones <- ncol(cz_mat)

  nz_mat <- matrix(rep(zone_pop, each = n_nodes), nrow = n_nodes)
  nz_bar_mat <- N - nz_mat
  cz_bar <- Cg - cz_mat

  # Always compute "expected under H0" - same in Poisson and Binomial as a
  # descriptive statistic (Cg * zone_pop / N). It enters the LLR formula
  # only in the Poisson case, but we report it on the data.frame either way.
  expected <- outer(Cg, zone_pop) / N

  if (model == "binomial") {
    # Binomial LLR
    rate_in  <- cz_mat / pmax(nz_mat, 1e-300)
    rate_out <- cz_bar / pmax(nz_bar_mat, 1e-300)
    mask <- rate_in > rate_out & cz_mat > 0

    # L(Ha): inside
    t1 <- ifelse(mask & cz_mat > 0,
                 cz_mat * log(pmax(cz_mat / nz_mat, 1e-300)), 0)
    t1b <- ifelse(mask & (nz_mat - cz_mat) > 0,
                  (nz_mat - cz_mat) * log(pmax(1 - cz_mat / nz_mat, 1e-300)), 0)
    # L(Ha): outside
    t2 <- ifelse(mask & cz_bar > 0,
                 cz_bar * log(pmax(cz_bar / nz_bar_mat, 1e-300)), 0)
    t2b <- ifelse(mask & (nz_bar_mat - cz_bar) > 0,
                  (nz_bar_mat - cz_bar) * log(pmax(1 - cz_bar / nz_bar_mat, 1e-300)), 0)
    # L(H0)
    p0 <- Cg / N
    t0a <- ifelse(Cg > 0, Cg * log(pmax(p0, 1e-300)), 0)
    t0b <- ifelse((N - Cg) > 0, (N - Cg) * log(pmax(1 - p0, 1e-300)), 0)
    t0 <- t0a + t0b

    llr_mat <- ifelse(mask, t1 + t1b + t2 + t2b - t0, 0)
    llr_mat[llr_mat < 0] <- 0
  } else {
    # Poisson LLR
    expected_bar <- outer(Cg, zone_pop, function(c, p) c - c * p / N)

    mask <- cz_mat > expected & cz_mat > 0 & expected > 0
    t1 <- ifelse(mask & cz_mat > 0,
                 cz_mat * log(pmax(cz_mat, 1e-300) / pmax(expected, 1e-300)), 0)
    t2 <- ifelse(mask & cz_bar > 0,
                 cz_bar * log(pmax(cz_bar, 1e-300) / pmax(expected_bar, 1e-300)), 0)
    llr_mat <- ifelse(mask, t1 + t2, 0)
  }
  llr_mat[Cg == 0, ] <- 0

  # Extract positive pairs
  pos <- which(llr_mat > 0)
  if (length(pos) == 0) {
    return(data.frame(node_id = character(0), center = integer(0),
                      n_regions = integer(0), cases = numeric(0),
                      expected = numeric(0), population = numeric(0),
                      llr = numeric(0), stringsAsFactors = FALSE))
  }

  g_idx <- ((pos - 1) %% n_nodes) + 1
  z_idx <- ((pos - 1) %/% n_nodes) + 1

  df <- data.frame(
    node_id = all_nodes[g_idx],
    center = zone_centers[z_idx],
    n_regions = zone_sizes[z_idx],
    cases = cz_mat[pos],
    expected = expected[pos],
    population = zone_pop[z_idx],
    llr = llr_mat[pos],
    stringsAsFactors = FALSE
  )
  df <- df[order(-df$llr), ]
  # Keep all evaluated pairs (not capped) so that filter_clusters can
  # recover distinct secondary clusters per Cancado et al. (2025) Sec 5.1.1.
  df
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mc_treespatial_cpp <- function(full_cases, Cg, N, zone_region_idx, zone_ptr, zone_pop, leaf_tree_idx, children_idx, children_ptr, proc_order, prob, nsim, model = 0L, n_cores = 1L) {
    .Call(`_treeSS_mc_treespatial_cpp`, full_cases, Cg, N, zone_region_idx, zone_ptr, zone_pop, leaf_tree_idx, children_idx, children_ptr, proc_order, prob, nsim, model, n_cores)
}

mc_spatial_cpp <- function(cases, N, C, zone_region_idx, zone_ptr, zone_pop, prob, nsim, model = 0L, n_cores = 1L) {
    .Call(`_treeSS_mc_spatial_cpp`, cases, N, C, zone_region_idx, zone_ptr, zone_pop, prob, nsim, model, n_cores)
}

mc_treescan_cpp <- function(node_cases, node_pop, C, N, leaf_tree_idx, leaf_pop, children_idx, children_ptr, proc_order, nsim, model = 0L, n_cores = 1L) {
    .Call(`_treeSS_mc_treescan_cpp`, node_cases, node_pop, C, N, leaf_tree_idx, leaf_pop, children_idx, children_ptr, proc_order, nsim, model, n_cores)
}

##=============================================================================
## Example 4: General Mortality in Florida, USA (2016)
##
## Two analyses:
##   1. PAPER-FAITHFUL multi-cluster procedure (Cancado et al. 2025, Sec. 5.1.1):
##      single scan + filter_clusters / get_cluster_regions(n_clusters = N).
##   2. (Optional) iterative_scan with Holm-Bonferroni - NOT in paper.
##
## Pedagogical workflow:
##   - Load raw long-format data (fl_deaths)
##   - Build ICD-10 tree directly from the codes that actually appear
##   - Download county polygons + centroids from tigris
##   - Build parallel vectors (cases, population, region_id, x, y, node_id)
##   - Run scan + iterative scan with Holm-Bonferroni
##
## Data: CDC WONDER Compressed Mortality File 1999-2016
##=============================================================================

options(bitmapType = "cairo")

library(treeSS)
library(ggplot2)
library(sf)


## ---- 1. Load raw data ----
data(fl_deaths)

cat("=== Florida Mortality 2016 ===\n")
cat("Rows (raw):  ", nrow(fl_deaths), "\n")
cat("Counties:    ", length(unique(fl_deaths$county_fips)), "\n")
cat("ICD-10 codes:", length(unique(fl_deaths$icd10_code)), "\n")
cat("Total deaths:", sum(fl_deaths$deaths), "\n")

icd_lookup <- unique(fl_deaths[, c("icd10_code", "icd10_desc")])
icd_desc <- function(code) {
  d <- icd_lookup$icd10_desc[icd_lookup$icd10_code == code]
  if (length(d) == 0) return(code)
  d[1]
}


## ---- 2. Build ICD-10 tree directly from fl_deaths ----
##
## fl_deaths uses two kinds of codes:
##   * 3-char (e.g. "C56", "R54") - leaf at the chapter level
##   * 5-char with a dot (e.g. "A04.7") - leaf under a 3-char internal parent

cat("\nBuilding ICD-10 tree from fl_deaths codes...\n")
icd_codes <- sort(unique(fl_deaths$icd10_code))

parsed <- data.frame(
  code = icd_codes,
  three = ifelse(grepl("\\.", icd_codes),
                  sub("\\..*", "", icd_codes),
                  icd_codes),
  is_three_leaf = !grepl("\\.", icd_codes) & nchar(icd_codes) == 3,
  stringsAsFactors = FALSE
)

icd_chapter_of <- function(three) {
  L <- substr(three, 1, 1)
  num <- suppressWarnings(as.integer(substr(three, 2, 3)))
  switch(
    L,
    A = "I_AB", B = "I_AB",
    C = "II_C_D",
    D = if (!is.na(num) && num <= 48) "II_C_D" else "III_D",
    E = "IV_E",
    F = "V_F",
    G = "VI_G",
    H = if (!is.na(num) && num <= 59) "VII_H" else "VIII_H",
    I = "IX_I",
    J = "X_J",
    K = "XI_K",
    L = "XII_L",
    M = "XIII_M",
    N = "XIV_N",
    O = "XV_O",
    P = "XVI_P",
    Q = "XVII_Q",
    R = "XVIII_R",
    S = "XIX_S_T", T = "XIX_S_T",
    V = "XX_V_Y", W = "XX_V_Y", X = "XX_V_Y", Y = "XX_V_Y",
    "Other"
  )
}
parsed$chapter <- sapply(parsed$three, icd_chapter_of)

fl_tree <- unique(rbind(
  data.frame(node_id = "Root", parent_id = NA_character_,
             stringsAsFactors = FALSE),
  data.frame(node_id = unique(parsed$chapter), parent_id = "Root",
             stringsAsFactors = FALSE),
  unique(data.frame(node_id = parsed$three, parent_id = parsed$chapter,
                    stringsAsFactors = FALSE)),
  data.frame(
    node_id   = parsed$code[!parsed$is_three_leaf],
    parent_id = parsed$three[!parsed$is_three_leaf],
    stringsAsFactors = FALSE
  )
))

parents <- unique(fl_tree$parent_id[!is.na(fl_tree$parent_id)])
leaves  <- setdiff(fl_tree$node_id, parents)
stopifnot(length(setdiff(icd_codes, leaves)) == 0)
cat("  Total nodes:", nrow(fl_tree),
    "| Chapters:", length(unique(parsed$chapter)),
    "| Leaves:", length(leaves), "\n")


## ---- 3. Download county polygons + centroids ----
if (!requireNamespace("tigris", quietly = TRUE)) {
  stop("Install 'tigris': install.packages('tigris')")
}
library(tigris)
cat("\nDownloading FL county boundaries...\n")
fl_map <- counties(state = "FL", cb = TRUE, year = 2016)
fl_map <- st_transform(fl_map, 4326)
centroids <- st_centroid(st_geometry(fl_map))
coords <- st_coordinates(centroids)
fl_map$x <- coords[, "X"]
fl_map$y <- coords[, "Y"]


## ---- 4. Build parallel vectors ----
fl_pop <- aggregate(population ~ county_fips, data = fl_deaths, FUN = max)
xy <- data.frame(county_fips = fl_map$GEOID, x = fl_map$x, y = fl_map$y,
                  stringsAsFactors = FALSE)
dat <- merge(fl_deaths[, c("county_fips", "icd10_code", "deaths")],
             fl_pop, by = "county_fips")
dat <- merge(dat, xy, by = "county_fips")
dat$region_id <- as.integer(factor(dat$county_fips,
                                    levels = sort(unique(dat$county_fips))))
dat <- dat[dat$deaths > 0, ]

region_info <- unique(dat[, c("region_id", "county_fips")])
cat("Long rows:", nrow(dat), "\n")


## ---- 5. Run the tree-spatial scan ----
cat("\nRunning tree-spatial scan (nsim=999, n_cores=4, max_pop_pct=0.05)...\n")
system.time({
  result_fl <- treespatial_scan(
    cases       = dat$deaths,
    population  = dat$population,
    region_id   = dat$region_id,
    x           = dat$x,
    y           = dat$y,
    node_id     = dat$icd10_code,
    tree        = fl_tree,
    max_pop_pct = 0.05,
    nsim        = 999, seed = 2016,
    n_cores     = 4L
  )
})
print(result_fl)


## ---- 6. Paper-faithful: distinct top clusters via filter_clusters ----
cat("\n=== Distinct top clusters (paper Sec. 5.1.1) ===\n")
fc <- filter_clusters(result_fl)
print(head(fc[, c("node_id", "n_regions", "cases", "expected", "llr", "pvalue")],
            5))


## ---- 7. (Optional) Iterative scan with Holm-Bonferroni ----
cat("\n=== Iterative scan (extension, with Holm-Bonferroni) ===\n")
iter_fl <- iterative_scan(
  cases       = dat$deaths,
  population  = dat$population,
  region_id   = dat$region_id,
  x           = dat$x,
  y           = dat$y,
  node_id     = dat$icd10_code,
  tree        = fl_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2016,
  max_pop_pct = 0.05, n_cores = 4L
)
print(iter_fl)


## ---- 8. Cluster membership ----
cr1   <- merge(get_cluster_regions(result_fl, n_clusters = 1, overlap = FALSE),
               region_info, by = "region_id")
cr2   <- merge(get_cluster_regions(result_fl, n_clusters = 2, overlap = TRUE),
               region_info, by = "region_id")
cr_it <- merge(get_cluster_regions(iter_fl, overlap = TRUE),
               region_info, by = "region_id")


## ---- 9. Plot 1: Most likely cluster ----
fl1 <- merge(fl_map, cr1, by.x = "GEOID", by.y = "county_fips", all.x = TRUE)
mlc <- result_fl$most_likely_cluster

p1 <- ggplot(fl1) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.15, alpha = 0.75) +
  scale_fill_manual(
    values   = c("1" = "#C44E52"),
    na.value = "gray95",
    labels   = c("1" = paste0(mlc$node_id, " (",
                              length(mlc$region_ids), " counties)")),
    name     = "Cluster"
  ) +
  labs(
    title    = paste0("Tree-Spatial Scan: most likely cluster (", mlc$node_id, ")"),
    subtitle = paste0(icd_desc(mlc$node_id),
                      " | LR=", round(mlc$llr, 1),
                      ", p=", format.pval(result_fl$pvalue, digits = 3),
                      " | ", mlc$cases, " deaths (expected ",
                      round(mlc$expected, 1), ")")
  ) +
  theme_minimal(base_size = 12) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 8))

ggsave("fl_cluster_mlc.png", p1, width = 9, height = 8, dpi = 300)
cat("Saved: fl_cluster_mlc.png\n")


## ---- 10. Plot 2: Top-2 distinct clusters (paper Sec. 5.1.1) ----
fl2 <- merge(fl_map, cr2, by.x = "GEOID", by.y = "county_fips")
palette2 <- c("1" = "#C44E52", "2" = "#4C72B0")

p2 <- ggplot(fl2) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(values = palette2, na.value = "gray95",
                    name = "Cluster", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  labs(title    = "Tree-Spatial Scan: Top 2 Distinct Clusters",
       subtitle = "Cancado et al. (2025), Sec. 5.1.1 - Florida mortality, 2016") +
  theme_minimal(base_size = 11) +
  theme(plot.title    = element_text(face = "bold"),
        plot.subtitle = element_text(color = "gray40"),
        strip.text    = element_text(face = "bold", size = 10),
        axis.title    = element_blank(),
        axis.text     = element_text(color = "gray50", size = 7),
        legend.position = "none")

ggsave("fl_clusters_top2.png", p2, width = 14, height = 6, dpi = 300)
cat("Saved: fl_clusters_top2.png\n")


## ---- 11. Plot 3 (optional): Iterative scan clusters ----
n_iter <- iter_fl$n_iter
if (n_iter > 0) {
  fl_it <- merge(fl_map, cr_it, by.x = "GEOID", by.y = "county_fips", all.x = TRUE)
  palette_it <- c("#C44E52", "#4C72B0", "#55A868", "#8172B2", "#CCB974")[
    seq_len(n_iter)
  ]
  names(palette_it) <- as.character(seq_len(n_iter))

  n_sig <- sum(iter_fl$clusters$significant, na.rm = TRUE)
  p_it <- ggplot(fl_it) +
    geom_sf(aes(fill = factor(cluster)), color = "gray40",
            linewidth = 0.12, alpha = 0.75) +
    scale_fill_manual(values = palette_it, na.value = "gray95",
                      name = "Iteration", na.translate = FALSE) +
    facet_wrap(~ panel, nrow = 1) +
    labs(title    = paste0("Iterative Scan (extension): ", n_iter,
                            " iterations, ", n_sig,
                            " significant after Holm-Bonferroni"),
         subtitle = "Each panel = one iteration (cases removed before next). NOT in paper.") +
    theme_minimal(base_size = 11) +
    theme(plot.title    = element_text(face = "bold"),
          plot.subtitle = element_text(color = "gray40"),
          strip.text    = element_text(face = "bold", size = 9),
          axis.title    = element_blank(),
          axis.text     = element_text(color = "gray50", size = 7),
          legend.position = "none")

  ggsave("fl_clusters_iterative.png", p_it,
         width = max(8, 4 * n_iter), height = 6, dpi = 300)
  cat("Saved: fl_clusters_iterative.png (", n_iter, " iterations)\n")
}


## ---- 12. Summary ----
cat("\n=== SUMMARY ===\n")
cat("Most likely cluster:\n")
cat("  Node:", mlc$node_id, "(", icd_desc(mlc$node_id), ")\n")
cat("  Counties:", length(mlc$region_ids), "\n")
cat("  Deaths:", mlc$cases, "(expected", round(mlc$expected, 1), ")\n")
cat("  RR:", round(mlc$rr, 2),
    "  LR:", round(mlc$llr, 2),
    "  p-value:", result_fl$pvalue, "\n")
cat("\nDistinct clusters via filter_clusters:", nrow(fc), "\n")
cat("Iterative scan: ", n_iter, " iterations, ",
    sum(iter_fl$clusters$significant, na.rm = TRUE),
    " significant after Holm-Bonferroni\n", sep = "")
cat("\nDone!\n")

## =============================================================================
## Documentation for datasets shipped with treeSS (>= 0.1.4).
##
## Each topical example ships as a pair:
##     <prefix>_<topic>   - combined long-format data.frame (input to scans)
##     <prefix>_tree      - hierarchical tree (node_id, parent_id)
##
## Plus the standalone datasets:
##     fl_deaths             - raw long-format death counts (Florida; user
##                             builds the tree)
##     chicago_map           - sf polygons for Chicago community areas
##     london_boroughs_map   - sf polygons for London boroughs
## =============================================================================


# ==== Rio de Janeiro =========================================================

#' Rio de Janeiro Infant Mortality (2016)
#'
#' Combined long-format dataset of infant mortality in the 92 municipalities
#' of Rio de Janeiro state, Brazil, 2016. Each row represents a (municipality,
#' ICD-10 leaf) combination with at least one death; missing combinations
#' are implicitly zero. Use together with \code{\link{rj_tree}}.
#'
#' @format A data.frame in long format with columns:
#' \describe{
#'   \item{region_id}{Integer identifier of the municipality.}
#'   \item{ibge_code}{6-digit IBGE municipality code.}
#'   \item{name}{Municipality name.}
#'   \item{population}{IBGE municipal population estimate for 2016.}
#'   \item{live_births}{Number of live births in 2016 (SINASC). Used as the
#'     population denominator in Section 5.2 of Cancado et al. (2025).}
#'   \item{x}{Longitude of the municipality centroid.}
#'   \item{y}{Latitude of the municipality centroid.}
#'   \item{node_id}{Character ICD-10 leaf code (4 characters), matching a
#'     leaf of \code{\link{rj_tree}}.}
#'   \item{cases}{Integer count of infant deaths.}
#' }
#'
#' @details
#' To reproduce Section 5.2 of Cancado et al. (2025), use \code{live_births}
#' as the population denominator:
#' \preformatted{
#'   data <- rj_mortality
#'   data$population <- data$live_births
#'   treespatial_scan(data, cases, population, region_id, x, y, node_id,
#'                    tree = rj_tree, ...)
#' }
#'
#' Municipality polygons are provided as the \code{rj_map} dataset.
#'
#' @source
#' Population: IBGE municipal estimates. Live births: DATASUS/SINASC via
#' TabNet. Deaths: DATASUS/SIM microdata via OpenDATASUS
#' (\url{https://opendatasus.saude.gov.br}). Centroids: \code{geobr}.
#'
#' @seealso \code{\link{rj_tree}}
#'
#' @references
#' Cancado, A.L.F., Oliveira, G.S., Quadros, A.V.C., Duczmal, L. (2025).
#' A tree-spatial scan statistic. \emph{Environmental and Ecological
#' Statistics}, 32, 953--978. \doi{10.1007/s10651-025-00670-w}
#'
#' @examples
#' data(rj_mortality)
#' head(rj_mortality)
#' cat("Total deaths:", sum(rj_mortality$cases), "\n")
"rj_mortality"


#' Rio de Janeiro Infant Mortality - ICD-10 Tree
#'
#' ICD-10 hierarchy used by the \code{\link{rj_mortality}} dataset.
#' Three levels: Chapter -> 3-character category -> 4-character subcategory.
#'
#' @format A data.frame with 622 rows and 2 columns:
#' \describe{
#'   \item{node_id}{Character identifier of each ICD-10 node.}
#'   \item{parent_id}{Character identifier of the parent node. \code{NA}
#'     for the root.}
#' }
#'
#' @details
#' The tree has 410 leaves (4-character ICD-10 codes) and 622 total nodes.
#'
#' @source
#' Constructed from the leaf-level codes in \code{\link{rj_mortality}}, which
#' in turn come from the Brazilian Mortality Information System (SIM)
#' maintained by DATASUS. See \code{data-raw/} in the GitHub repository for
#' the build script.
#'
#' @seealso \code{\link{rj_mortality}}
#'
#' @examples
#' data(rj_tree)
#' head(rj_tree)
"rj_tree"


# ==== London =================================================================

#' London Road Collisions (2022)
#'
#' Combined long-format dataset of road traffic collisions in the 33 London
#' boroughs, 2022. Each row represents a (borough, collision-category)
#' combination with at least one collision. Use together with
#' \code{\link{london_tree}}.
#'
#' @format A data.frame in long format with columns:
#' \describe{
#'   \item{region_id}{Integer identifier of the borough.}
#'   \item{borough}{Borough name.}
#'   \item{population}{Total collisions in the borough (used as population
#'     denominator).}
#'   \item{x}{Longitude of the borough centroid.}
#'   \item{y}{Latitude of the borough centroid.}
#'   \item{node_id}{Character leaf identifier from \code{\link{london_tree}}.}
#'   \item{cases}{Integer count of collisions.}
#' }
#'
#' @seealso \code{\link{london_tree}}, \code{\link{london_boroughs_map}}
#'
#' @source
#' UK Department for Transport, STATS19 data via the \code{stats19} R package.
#'
#' @examples
#' data(london_collisions)
#' head(london_collisions)
#' cat("Total collisions:", sum(london_collisions$cases), "\n")
"london_collisions"


#' London Road Collisions - Hierarchy
#'
#' Hierarchical classification of road collisions by light conditions,
#' road type, and junction detail.
#'
#' @format A data.frame with 81 rows and 2 columns:
#' \describe{
#'   \item{node_id}{Character identifier of each node.}
#'   \item{parent_id}{Character identifier of the parent node. \code{NA}
#'     for the root.}
#' }
#'
#' @details
#' 3-level tree: Root -> Light conditions (Daylight/Darkness) -> Road type
#' (5 types) -> Junction detail (7 types). 68 leaf categories.
#'
#' @source
#' Constructed from the leaf-level codes in \code{\link{london_collisions}}.
#' See \code{data-raw/} in the GitHub repository for the build script.
#'
#' @seealso \code{\link{london_collisions}}
#'
#' @examples
#' data(london_tree)
#' head(london_tree)
"london_tree"


#' London Borough Boundaries
#'
#' Simplified polygon boundaries for the 33 London boroughs. Use with
#' \code{\link{london_collisions}} for map-based visualization of cluster
#' results.
#'
#' @format An \code{sf} data.frame with 33 rows and 3 columns:
#' \describe{
#'   \item{NAME}{Borough name, e.g. \code{"Westminster"}.}
#'   \item{GSS_CODE}{ONS geography code, e.g. \code{"E09000033"}.}
#'   \item{geometry}{MULTIPOLYGON geometry in WGS84 (EPSG:4326).}
#' }
#'
#' @details
#' Geometries are simplified (\code{st_simplify(dTolerance = 0.001)}) to
#' reduce file size. For full-resolution boundaries, download from the
#' source URL.
#'
#' To merge with cluster results from \code{\link{get_cluster_regions}},
#' use: \code{merge(london_boroughs_map, cr, by.x = "NAME", by.y = "borough")}.
#'
#' @source
#' London Datastore, Statistical GIS Boundary Files for London
#' (see \url{https://data.london.gov.uk/dataset/statistical-gis-boundary-files-london/}).
#' Contains National Statistics data (c) Crown copyright and database
#' right 2014.
#'
#' @seealso \code{\link{london_collisions}}
#'
#' @examples
#' data(london_boroughs_map)
#' cat("Boroughs:", nrow(london_boroughs_map), "\n")
"london_boroughs_map"


# ==== Chicago ================================================================

#' Chicago Crime (2023)
#'
#' Combined long-format dataset of crime incidents in the 77 community areas
#' of Chicago, 2023. Use together with \code{\link{chicago_tree}}.
#'
#' @format A data.frame in long format with columns:
#' \describe{
#'   \item{region_id}{Integer identifier of the community area.}
#'   \item{area_number}{Official community area number (1--77).}
#'   \item{name}{Community area name.}
#'   \item{population}{Total incidents in the area. This is a compositional
#'     denominator (sums to the total incident count of the city) and is
#'     useful for analyses that ask "which crime types over-occur in which
#'     areas relative to the citywide profile". Not a population at risk.}
#'   \item{pop_residential}{Residential population of the community area.
#'     Use this as the denominator for incidence-rate analyses (incidents
#'     per resident). Source: U.S. Census Bureau, ACS 2020 5-year estimates,
#'     aggregated to community areas by CMAP Community Data Snapshots.
#'     The 77 values sum to approximately 2.71M, matching the published
#'     Chicago population.}
#'   \item{x}{Longitude of the centroid.}
#'   \item{y}{Latitude of the centroid.}
#'   \item{node_id}{Character leaf identifier from \code{\link{chicago_tree}}.}
#'   \item{cases}{Integer count of crime incidents.}
#' }
#'
#' @seealso \code{\link{chicago_tree}}, \code{\link{chicago_map}}
#'
#' @source
#' Crime incidents: City of Chicago Data Portal, Crimes -- 2001 to Present.
#'
#' Residential population: U.S. Census Bureau ACS 2020 5-year estimates,
#' aggregated to community areas by the Chicago Metropolitan Agency for
#' Planning (CMAP), Community Data Snapshots (2023 release).
#' \url{https://cmap.illinois.gov/data/community-data-snapshots/}
#'
#' @examples
#' data(chicago_crimes)
#' head(chicago_crimes)
#' cat("Total incidents:", sum(chicago_crimes$cases), "\n")
#' cat("Total residents:",
#'     sum(unique(chicago_crimes[, c("area_number", "pop_residential")])$pop_residential),
#'     "\n")
"chicago_crimes"


#' Chicago Crime - Hierarchy
#'
#' Hierarchical classification of crime incidents by type, description and
#' location group.
#'
#' @format A data.frame with 2,841 rows and 2 columns:
#' \describe{
#'   \item{node_id}{Character identifier of each node.}
#'   \item{parent_id}{Character identifier of the parent node. \code{NA}
#'     for the root.}
#' }
#'
#' @details
#' 4-level tree: Root -> Crime Type -> Description -> Location Group.
#' 2,486 leaf nodes. Leaf node IDs follow the pattern
#' \code{"<TYPE> | <DESCRIPTION> | <LOCATION>"}.
#'
#' @source
#' Constructed from the leaf-level codes in \code{\link{chicago_crimes}}.
#' See \code{data-raw/} in the GitHub repository for the build script.
#'
#' @seealso \code{\link{chicago_crimes}}
#'
#' @examples
#' data(chicago_tree)
#' head(chicago_tree)
"chicago_tree"


#' Chicago Community Area Boundaries
#'
#' Simplified polygon boundaries for the 77 Chicago community areas.
#'
#' @format An \code{sf} data.frame with 77 rows and 3 columns:
#' \describe{
#'   \item{NAME}{Community area name.}
#'   \item{AREA_NUM}{Community area number (integer, 1--77).}
#'   \item{geometry}{MULTIPOLYGON geometry in WGS84 (EPSG:4326).}
#' }
#'
#' @source
#' City of Chicago Data Portal, Community Area Boundaries.
#'
#' @seealso \code{\link{chicago_crimes}}
#'
#' @examples
#' data(chicago_map)
#' cat("Community areas:", nrow(chicago_map), "\n")
"chicago_map"


# ==== Florida (raw) ==========================================================

#' Florida General Mortality Data (2016)
#'
#' Death counts by county and ICD-10 cause of death for the state of Florida,
#' USA, 2016. This is \strong{raw data} -- the user builds the ICD-10 tree
#' and the cases matrix in the analysis script, making it a pedagogical
#' example of the full workflow.
#'
#' @format A data.frame with 3,066 rows and 6 columns:
#' \describe{
#'   \item{county_fips}{5-digit FIPS code (character), e.g. \code{"12086"}.}
#'   \item{county_name}{County name, e.g. \code{"Miami-Dade County"}.}
#'   \item{icd10_code}{ICD-10 cause of death code, e.g. \code{"I25.1"}.}
#'   \item{icd10_desc}{Description of the ICD-10 code, e.g.
#'     \code{"Atherosclerotic heart disease"}.}
#'   \item{deaths}{Number of deaths (integer).}
#'   \item{population}{County population estimate (integer).}
#' }
#'
#' @details
#' Covers 65 of Florida's 67 counties (2 small counties excluded by CDC
#' suppression rules), 253 ICD-10 codes, and 157,000 total deaths. All
#' ages, all causes.
#'
#' To use with \code{\link{treespatial_scan}}, rename and merge with
#' centroids:
#' \preformatted{
#'   # Centroids from tigris
#'   data <- transform(fl_deaths,
#'                     region_id = county_fips,
#'                     node_id   = icd10_code,
#'                     cases     = deaths)
#'   # Add x, y from a centroids table
#'   data <- merge(data, centroids, by = "county_fips")
#'   treespatial_scan(data, cases, population, region_id, x, y, node_id,
#'                    tree = fl_tree, ...)
#' }
#'
#' County polygons and centroids can be obtained via
#' \code{tigris::counties(state = "FL", cb = TRUE)}.
#'
#' The ICD-10 descriptions in \code{icd10_desc} can be used to build a
#' lookup table: \code{unique(fl_deaths[, c("icd10_code", "icd10_desc")])}.
#'
#' @source
#' CDC WONDER Compressed Mortality File 1999--2016
#' (\url{https://wonder.cdc.gov/cmf-icd10.html}).
#'
#' @examples
#' data(fl_deaths)
#' head(fl_deaths)
#' cat("Counties:", length(unique(fl_deaths$county_fips)), "\n")
#' cat("ICD-10 codes:", length(unique(fl_deaths$icd10_code)), "\n")
#' cat("Total deaths:", sum(fl_deaths$deaths), "\n")
"fl_deaths"


#' Rio de Janeiro Municipal Boundaries
#'
#' Polygon boundaries for the 92 municipalities of Rio de Janeiro state,
#' used to map clusters from the \code{rj_mortality} analysis.
#'
#' @format An \code{sf} data.frame with 92 rows and 3 columns:
#' \describe{
#'   \item{ibge_code}{6-digit IBGE municipality code; join key to
#'     \code{rj_mortality$ibge_code}.}
#'   \item{NAME}{Municipality name.}
#'   \item{geometry}{MULTIPOLYGON geometry in WGS84 (EPSG:4326).}
#' }
#'
#' @source
#' Municipal boundaries (Malhas Municipais) from the Brazilian Institute of
#' Geography and Statistics (IBGE).
#'
#' @seealso \code{\link{rj_mortality}}
#'
#' @examples
#' data(rj_map)
#' cat("Municipalities:", nrow(rj_map), "\n")
"rj_map"

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>",
  eval = FALSE
)

## ----fl-load------------------------------------------------------------------
#  library(treeSS)
#  data(fl_deaths)
#  
#  str(fl_deaths)
#  #> 'data.frame': N obs. of 5 variables:
#  #>  $ county_fips: chr  ...
#  #>  $ icd10_code : chr  "C56" "C61" "I10" ...
#  #>  $ icd10_desc : chr  ...
#  #>  $ deaths     : int  ...
#  #>  $ population : int  ...
#  
#  c(rows         = nrow(fl_deaths),
#    counties     = length(unique(fl_deaths$county_fips)),
#    icd10_codes  = length(unique(fl_deaths$icd10_code)),
#    total_deaths = sum(fl_deaths$deaths))

## ----fl-tree------------------------------------------------------------------
#  icd_codes <- sort(unique(fl_deaths$icd10_code))
#  
#  parsed <- data.frame(
#    code  = icd_codes,
#    three = ifelse(grepl("\\.", icd_codes),
#                    sub("\\..*", "", icd_codes),
#                    icd_codes),
#    is_three_leaf = !grepl("\\.", icd_codes) & nchar(icd_codes) == 3,
#    stringsAsFactors = FALSE
#  )
#  
#  # Map 3-character codes to ICD-10 chapters
#  icd_chapter_of <- function(three) {
#    L   <- substr(three, 1, 1)
#    num <- suppressWarnings(as.integer(substr(three, 2, 3)))
#    switch(L,
#      A = "I_AB", B = "I_AB",
#      C = "II_C_D",
#      D = if (!is.na(num) && num <= 48) "II_C_D" else "III_D",
#      E = "IV_E",  F = "V_F",  G = "VI_G",
#      H = if (!is.na(num) && num <= 59) "VII_H" else "VIII_H",
#      I = "IX_I",  J = "X_J",  K = "XI_K", L = "XII_L",
#      M = "XIII_M", N = "XIV_N", O = "XV_O", P = "XVI_P",
#      Q = "XVII_Q", R = "XVIII_R",
#      S = "XIX_S_T", T = "XIX_S_T",
#      V = "XX_V_Y", W = "XX_V_Y", X = "XX_V_Y", Y = "XX_V_Y",
#      "Other"
#    )
#  }
#  parsed$chapter <- sapply(parsed$three, icd_chapter_of)
#  
#  fl_tree <- unique(rbind(
#    data.frame(node_id = "Root", parent_id = NA_character_,
#               stringsAsFactors = FALSE),
#    data.frame(node_id = unique(parsed$chapter), parent_id = "Root",
#               stringsAsFactors = FALSE),
#    unique(data.frame(node_id = parsed$three, parent_id = parsed$chapter,
#                      stringsAsFactors = FALSE)),
#    data.frame(
#      node_id   = parsed$code[!parsed$is_three_leaf],
#      parent_id = parsed$three[!parsed$is_three_leaf],
#      stringsAsFactors = FALSE
#    )
#  ))
#  
#  # Sanity check: every code in fl_deaths is a leaf of fl_tree
#  parents <- unique(fl_tree$parent_id[!is.na(fl_tree$parent_id)])
#  leaves  <- setdiff(fl_tree$node_id, parents)
#  stopifnot(length(setdiff(icd_codes, leaves)) == 0)
#  
#  c(total_nodes = nrow(fl_tree),
#    chapters    = length(unique(parsed$chapter)),
#    leaves      = length(leaves))

## ----fl-polygons--------------------------------------------------------------
#  library(sf)
#  library(tigris)
#  
#  fl_map <- counties(state = "FL", cb = TRUE, year = 2016)
#  fl_map <- st_transform(fl_map, 4326)
#  
#  centroids   <- st_centroid(st_geometry(fl_map))
#  coords      <- st_coordinates(centroids)
#  fl_map$x    <- coords[, "X"]
#  fl_map$y    <- coords[, "Y"]

## ----fl-vectors---------------------------------------------------------------
#  # Per-county population: same value across all leaves, so take any one
#  fl_pop <- aggregate(population ~ county_fips, data = fl_deaths,
#                       FUN = max)
#  
#  # Per-county centroids from the polygon layer
#  xy <- data.frame(county_fips = fl_map$GEOID,
#                    x = fl_map$x, y = fl_map$y,
#                    stringsAsFactors = FALSE)
#  
#  dat <- merge(fl_deaths[, c("county_fips", "icd10_code", "deaths")],
#               fl_pop, by = "county_fips")
#  dat <- merge(dat, xy, by = "county_fips")
#  
#  # Integer region_id is convenient downstream; here we use the rank
#  # of county_fips in alphabetical order
#  dat$region_id <- as.integer(factor(
#    dat$county_fips, levels = sort(unique(dat$county_fips))
#  ))
#  
#  # Keep only rows with at least one death (sparse-by-default scan)
#  dat <- dat[dat$deaths > 0, ]
#  
#  c(long_rows = nrow(dat),
#    counties  = length(unique(dat$region_id)))

## ----fl-scan------------------------------------------------------------------
#  result_fl <- treespatial_scan(
#    dat,
#    cases       = deaths,
#    population  = population,
#    region_id   = region_id,
#    x           = x,
#    y           = y,
#    node_id     = icd10_code,
#    tree        = fl_tree,
#    max_pop_pct = 0.05,
#    nsim        = 999, seed = 2016,
#    n_cores     = 4L
#  )
#  print(result_fl)

## ----fl-filter----------------------------------------------------------------
#  filter_clusters(result_fl)[1:5,
#                              c("node_id", "n_regions",
#                                "cases", "expected", "llr",
#                                "pvalue")]

## ----fl-seq-------------------------------------------------------------------
#  seq_fl <- sequential_scan(
#    dat,
#    cases       = deaths,
#    population  = population,
#    region_id   = region_id,
#    x           = x,
#    y           = y,
#    node_id     = icd10_code,
#    tree        = fl_tree,
#    max_iter    = 5L, alpha = 0.05,
#    nsim        = 999, seed = 2016,
#    max_pop_pct = 0.05, n_cores = 4L
#  )
#  print(seq_fl)

## ----fl-plot------------------------------------------------------------------
#  library(ggplot2)
#  library(dplyr)
#  
#  region_info <- unique(dat[, c("region_id", "county_fips")])
#  
#  cr_seq <- merge(
#    get_cluster_regions(seq_fl, overlap = TRUE),
#    region_info, by = "region_id"
#  )
#  
#  # Cross-join the polygon layer with every iteration panel so all
#  # counties appear in every panel (those outside the analysis
#  # dataset show as NA-coloured rather than as an extra empty panel)
#  panels       <- unique(cr_seq$panel)
#  cr_seq_keys  <- unique(cr_seq[, c("county_fips", "cluster",
#                                     "node_id", "llr", "pvalue",
#                                     "panel")])
#  fl_seq_plot <- merge(
#    do.call(rbind,
#            lapply(panels, function(p) cbind(fl_map, panel = p))),
#    cr_seq_keys,
#    by.x = c("GEOID", "panel"),
#    by.y = c("county_fips", "panel"),
#    all.x = TRUE
#  )
#  
#  ggplot(fl_seq_plot) +
#    geom_sf(aes(fill = factor(cluster)), color = "gray40",
#            linewidth = 0.12, alpha = 0.75) +
#    scale_fill_manual(
#      values = c("#C44E52", "#4C72B0", "#55A868",
#                 "#8172B2", "#CCB974"),
#      na.value = "gray95", na.translate = FALSE
#    ) +
#    facet_wrap(~ panel, nrow = 1) +
#    theme_minimal() +
#    theme(legend.position = "none")

#' Sequential Scan for Secondary Clusters
#'
#' Implements the sequential adjustment of Zhang, Assuncao and Kulldorff
#' (2010) for detecting secondary clusters. After the most likely cluster
#' (MLC) is detected and found significant, the regions composing it
#' (optionally together with a buffer of nearest-neighbour regions) are
#' \emph{removed} from the dataset -- treated as a \dQuote{lake} with no
#' population and no cases -- and the scan is re-run on the reduced data
#' with a fresh Monte Carlo simulation. The procedure is iterated until
#' the MLC of the current reduced data is no longer significant, or
#' \code{max_iter} is reached.
#'
#' This differs from \code{\link{filter_clusters}}, which scans the
#' single-pass candidate pool for non-overlapping clusters (the original
#' Cancado et al. 2025 criterion). The sequential approach computes a
#' fresh, unconservative p-value for each secondary cluster, conditional
#' on the prior detections.
#'
#' Inputs are supplied through a single \code{data} data.frame; the
#' relevant columns are referenced by name. Which columns are supplied
#' selects one of three scan types:
#' \itemize{
#'   \item tree-spatial: all of \code{cases}, \code{population},
#'         \code{region_id}, \code{x}, \code{y}, \code{node_id},
#'         \code{tree} are required (\code{data} in long format, one row
#'         per (region, leaf));
#'   \item circular: \code{cases}, \code{population}, \code{region_id},
#'         \code{x}, \code{y} (no \code{node_id} / \code{tree};
#'         one row per region);
#'   \item tree-only: \code{cases}, \code{population}, \code{node_id},
#'         \code{tree} (no spatial inputs). \code{buffer_size} is ignored
#'         for the tree-only case.
#' }
#'
#' @param data A \code{data.frame} whose columns are named by the
#'   \code{cases}, \code{population} and (depending on scan type)
#'   \code{region_id}, \code{x}, \code{y}, \code{node_id} arguments.
#' @param cases Column of \code{data} giving case counts. Given as an
#'   unquoted column name (a string or expression also works).
#' @param population Column of \code{data} giving the denominator.
#' @param region_id,x,y Columns of \code{data} (omit for tree-only scan).
#' @param node_id Column of \code{data} (omit for circular scan).
#' @param tree Tree as a 2-column data.frame
#'   (\code{node_id, parent_id}), or use
#'   \code{tree_node_id}/\code{tree_parent_id}. Omit for circular scan.
#' @param tree_node_id,tree_parent_id Optional. Parallel vectors as an
#'   alternative to \code{tree}.
#' @param max_iter Integer. Maximum number of iterations (including the
#'   primary MLC). Default 5.
#' @param alpha Numeric. Significance level. Iteration stops when the
#'   MLC p-value of the current reduced data exceeds \code{alpha}.
#'   Default 0.05.
#' @param nsim Integer. Monte Carlo replications per iteration.
#'   Default 999.
#' @param max_pop_pct Numeric. Maximum zone-size constraint, passed to
#'   the inner scans. Default 0.5.
#' @param buffer_size Integer. Number of nearest-neighbour regions to
#'   remove together with each detected cluster, computed by Euclidean
#'   distance from each cluster region to the remaining ones. Default
#'   0. Zhang et al. (2010) report that the type I error and power are
#'   essentially insensitive to the buffer size and that
#'   \code{buffer_size = 0} is generally adequate. Ignored for
#'   tree-only scans.
#' @param model Character. \code{"poisson"} (default) or
#'   \code{"binomial"}.
#' @param seed Integer or \code{NULL}. Seed for the first iteration;
#'   subsequent iterations advance the RNG state to avoid using the
#'   same seed across iterations.
#' @param verbose Logical. Print progress at each iteration.
#' @param n_cores Integer. OpenMP threads passed to inner scans.
#'
#' @return An object of class \code{"sequential_scan"} with components:
#'   \describe{
#'     \item{clusters}{A data.frame with one row per iteration performed,
#'       with columns \code{iteration}, \code{node_id} (only for
#'       tree-spatial / tree-only scans), \code{n_regions} (only for
#'       scans with a spatial component), \code{cases}, \code{expected},
#'       \code{population}, \code{rr} (only for scans with a spatial
#'       component), \code{llr}, \code{pvalue}, and a logical
#'       \code{significant} flag (\code{pvalue <= alpha}). The list-
#'       columns \code{region_ids} and/or \code{leaf_ids} carry the
#'       cluster composition.}
#'     \item{iterations}{A list with the full scan result of each
#'       iteration.}
#'     \item{regions, tree, alpha, nsim, buffer_size, n_iter,
#'           scan_type}{Bookkeeping fields. \code{n_iter} is the number
#'       of iterations actually performed.}
#'   }
#'
#' @references
#' Zhang, Z., Assuncao, R., & Kulldorff, M. (2010). Spatial scan
#' statistics adjusted for multiple clusters. \emph{Journal of
#' Probability and Statistics}, 2010, 642379.
#'
#' Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal,
#' L. H. (2025). A tree-spatial scan statistic. \emph{Environmental
#' and Ecological Statistics}, 32, 953-978.
#' \doi{10.1007/s10651-025-00670-w}
#'
#' Kulldorff, M. (1997). A spatial scan statistic.
#' \emph{Communications in Statistics - Theory and Methods}, 26(6),
#' 1481-1496.
#'
#' @seealso \code{\link{treespatial_scan}}, \code{\link{circular_scan}},
#'   \code{\link{tree_scan}}, \code{\link{filter_clusters}},
#'   \code{\link{get_cluster_regions}}
#'
#' @export
#' @examples
#' \donttest{
#' data(london_collisions); data(london_tree)
#' result <- sequential_scan(
#'   london_collisions,
#'   cases       = cases,
#'   population  = population,
#'   region_id   = region_id,
#'   x           = x,
#'   y           = y,
#'   node_id     = node_id,
#'   tree        = london_tree,
#'   max_iter = 3, nsim = 99, seed = 42
#' )
#' print(result)
#' }
sequential_scan <- function(data, cases, population,
                            region_id = NULL, x = NULL, y = NULL,
                            node_id = NULL,
                            tree = NULL,
                            tree_node_id = NULL, tree_parent_id = NULL,
                            max_iter = 5L, alpha = 0.05,
                            nsim = 999L, max_pop_pct = 0.5,
                            buffer_size = 0L,
                            model = c("poisson", "binomial"),
                            seed = NULL, verbose = TRUE,
                            n_cores = 1L) {

  ## --- resolve data/column inputs (substitutive interface, treeSS >= 0.2.0) ---
  if (missing(data)) {
    stop("`data` is required: a data.frame whose columns are named by the ",
         "`cases`, `population` (and scan-type-dependent) arguments.",
         call. = FALSE)
  }
  data <- as.data.frame(data)
  .env <- parent.frame()
  q_region <- if (missing(region_id)) NULL else substitute(region_id)
  q_x      <- if (missing(x))         NULL else substitute(x)
  q_y      <- if (missing(y))         NULL else substitute(y)
  q_node   <- if (missing(node_id))   NULL else substitute(node_id)
  cases      <- .resolve_col(substitute(cases),      data, .env, "cases")
  population <- .resolve_col(substitute(population),  data, .env, "population")
  region_id  <- .resolve_col(q_region, data, .env, "region_id")
  x          <- .resolve_col(q_x,      data, .env, "x")
  y          <- .resolve_col(q_y,      data, .env, "y")
  node_id    <- .resolve_col(q_node,   data, .env, "node_id")

  model <- match.arg(model)

  has_tree <- !is.null(tree) || (!is.null(tree_node_id) &&
                                  !is.null(tree_parent_id))
  has_node <- !is.null(node_id)
  has_geo  <- !is.null(region_id) && !is.null(x) && !is.null(y)

  if (has_tree && has_node && has_geo) {
    scan_type <- "treespatial"
    tree <- .normalize_tree(tree, tree_node_id, tree_parent_id)
  } else if (!has_tree && has_geo) {
    scan_type <- "circular"
  } else if (has_tree && has_node && !has_geo) {
    scan_type <- "tree"
    tree <- .normalize_tree(tree, tree_node_id, tree_parent_id)
  } else {
    stop("Inputs are not consistent. Provide columns of `data` for:\n",
         "  tree-spatial: cases, population, region_id, x, y, node_id (+ tree)\n",
         "  circular:     cases, population, region_id, x, y\n",
         "  tree-only:    cases, population, node_id (+ tree; no region_id/x/y)",
         call. = FALSE)
  }

  stopifnot(is.numeric(max_iter),    length(max_iter)    == 1L,
            max_iter >= 1L)
  stopifnot(is.numeric(alpha),       length(alpha)       == 1L,
            alpha > 0,    alpha < 1)
  stopifnot(is.numeric(nsim),        length(nsim)        == 1L,
            nsim >= 1L)
  stopifnot(is.numeric(max_pop_pct), length(max_pop_pct) == 1L,
            max_pop_pct > 0, max_pop_pct <= 1)
  stopifnot(is.numeric(buffer_size), length(buffer_size) == 1L,
            buffer_size >= 0L)

  max_iter    <- as.integer(max_iter)
  buffer_size <- as.integer(buffer_size)

  # Working copies of the per-row vectors that we will shrink at each
  # iteration by dropping rows that belong to the union of detected
  # cluster regions (plus buffer). For the tree-only scan we drop leaves
  # instead.
  if (scan_type == "tree") {
    # Re-key the leaf rows of `data` onto the canonical leaf order of the
    # tree (cases summed within leaf; denominator from first occurrence).
    leaves0 <- as.character(.get_leaves(tree))
    nid0    <- as.character(node_id)
    unknown0 <- setdiff(unique(nid0), leaves0)
    if (length(unknown0) > 0) {
      stop("'node_id' contains values that are not leaves of the tree: ",
           paste(utils::head(unknown0, 5), collapse = ", "),
           if (length(unknown0) > 5) ", ..." else "", ".", call. = FALSE)
    }
    fac0 <- factor(nid0, levels = leaves0)
    cur_cases <- as.numeric(tapply(as.numeric(cases), fac0, sum))
    cur_cases[is.na(cur_cases)] <- 0
    cur_population <- as.numeric(tapply(as.numeric(population), fac0,
                                        function(z) z[1L]))
    cur_population[is.na(cur_population)] <- 0
  } else {
    cur_cases      <- as.numeric(cases)
    cur_population <- as.numeric(population)
    cur_region_id <- region_id
    cur_x         <- as.numeric(x)
    cur_y         <- as.numeric(y)
    if (scan_type == "treespatial") {
      cur_node_id <- node_id
    }
  }

  removed_region_ids <- character(0)
  removed_leaf_ids   <- character(0)
  clusters_rows <- list()
  iterations    <- list()

  for (k in seq_len(max_iter)) {
    iter_seed <- if (is.null(seed)) NULL else seed + k - 1L

    if (verbose) message(sprintf("Iteration %d/%d ...", k, max_iter))

    # Early stop: no data left
    if (length(cur_cases) == 0L || sum(cur_cases) <= 0 ||
        sum(cur_population) <= 0) {
      if (verbose) message(sprintf(
        "  Iteration %d: reduced dataset exhausted. Stopping.", k))
      break
    }

    # Run the appropriate scan on the current reduced data. The public
    # scans now take a `data.frame`; we assemble one from the current
    # working vectors and name its columns (string column names are
    # accepted by the data/column interface).
    if (scan_type == "treespatial") {
      .df <- data.frame(
        cases      = cur_cases,
        population = cur_population,
        region_id  = cur_region_id,
        x          = cur_x,
        y          = cur_y,
        node_id    = cur_node_id,
        stringsAsFactors = FALSE
      )
      res <- treespatial_scan(
        .df,
        cases = "cases", population = "population",
        region_id = "region_id", x = "x", y = "y", node_id = "node_id",
        tree        = tree,
        max_pop_pct = max_pop_pct,
        nsim        = nsim, alpha = alpha,
        model       = model,
        seed        = iter_seed, n_cores = n_cores
      )
    } else if (scan_type == "circular") {
      .df <- data.frame(
        cases      = cur_cases,
        population = cur_population,
        region_id  = cur_region_id,
        x          = cur_x,
        y          = cur_y,
        stringsAsFactors = FALSE
      )
      res <- circular_scan(
        .df,
        cases = "cases", population = "population",
        region_id = "region_id", x = "x", y = "y",
        max_pop_pct = max_pop_pct,
        nsim        = nsim, alpha = alpha,
        model       = model,
        seed        = iter_seed, n_cores = n_cores
      )
    } else {
      # tree-only: cur_cases / cur_population are aligned to the current
      # tree's leaf order, so reconstruct node_id from the leaves.
      .df <- data.frame(
        cases      = cur_cases,
        population = cur_population,
        node_id    = as.character(.get_leaves(tree)),
        stringsAsFactors = FALSE
      )
      res <- tree_scan(
        .df,
        cases = "cases", node_id = "node_id",
        tree = tree, population = "population",
        nsim = nsim, alpha = alpha,
        model = model,
        seed = iter_seed, n_cores = n_cores
      )
    }

    mlc <- res$most_likely_cluster
    pvalue <- res$pvalue
    is_sig <- !is.na(pvalue) && pvalue <= alpha

    # Stop on dead signal (LLR=0 or no cases): nothing left to find
    if (is.null(mlc$llr) || mlc$llr <= 0 ||
        is.null(mlc$cases) || mlc$cases == 0) {
      if (verbose) message(sprintf(
        "  Iteration %d: no residual signal (LR = 0). Stopping.", k))
      break
    }

    # Record this cluster
    if (scan_type == "tree") {
      row <- data.frame(
        iteration  = k,
        node_id    = as.character(mlc$node_id),
        cases      = mlc$cases,
        expected   = mlc$expected,
        population = mlc$population,
        llr        = mlc$llr,
        pvalue     = pvalue,
        significant = is_sig,
        stringsAsFactors = FALSE
      )
      row$leaf_ids <- I(list(mlc$leaf_ids))
    } else {
      row <- data.frame(
        iteration  = k,
        node_id    = if (scan_type == "treespatial")
                       as.character(mlc$node_id) else NA_character_,
        n_regions  = length(mlc$region_ids),
        cases      = mlc$cases,
        expected   = mlc$expected,
        population = mlc$population,
        rr         = mlc$rr,
        llr        = mlc$llr,
        pvalue     = pvalue,
        significant = is_sig,
        stringsAsFactors = FALSE
      )
      row$region_ids <- I(list(mlc$region_ids))
    }

    clusters_rows[[k]] <- row
    iterations[[k]]    <- res

    if (verbose) {
      if (scan_type == "tree") {
        message(sprintf(
          "  Cluster %d: node = %s, LR = %.2f, p = %.3f%s",
          k, mlc$node_id, mlc$llr, pvalue,
          if (is_sig) " *" else ""))
      } else if (scan_type == "treespatial") {
        message(sprintf(
          "  Cluster %d: node = %s, %d region(s), LR = %.2f, p = %.3f%s",
          k, mlc$node_id, length(mlc$region_ids), mlc$llr, pvalue,
          if (is_sig) " *" else ""))
      } else {
        message(sprintf(
          "  Cluster %d: %d region(s), LR = %.2f, p = %.3f%s",
          k, length(mlc$region_ids), mlc$llr, pvalue,
          if (is_sig) " *" else ""))
      }
    }

    # Stop after recording if not significant -- the sequential procedure
    # of Zhang et al. (2010) tests in order, stopping at the first
    # non-significant MLC.
    if (!is_sig) {
      if (verbose) message(sprintf(
        "  Iteration %d: MLC is not significant at alpha = %g. Stopping.",
        k, alpha))
      break
    }

    # Stop if budget reached
    if (k == max_iter) break

    # --- Remove the MLC regions (+ optional buffer) from working data ---
    if (scan_type == "tree") {
      # Remove the leaves descending from the detected node from the
      # current data.
      leaves_all <- .get_leaves(tree)
      desc <- .get_descendants(tree, as.character(mlc$node_id))
      desc_leaves <- intersect(desc, leaves_all)
      # cur_cases / cur_population are vectors aligned to leaves_all
      keep <- !(leaves_all %in% desc_leaves)
      cur_cases      <- cur_cases[keep]
      cur_population <- cur_population[keep]
      # Now the next tree_scan needs a tree consistent with the kept
      # leaves: prune the tree to the kept leaves and their ancestors.
      removed_leaf_ids <- union(removed_leaf_ids, desc_leaves)
      tree <- .prune_tree_to_leaves(tree, leaves_all[keep])
    } else {
      regs_to_remove <- mlc$region_ids
      if (buffer_size > 0L) {
        regs_to_remove <- .add_buffer_regions(
          cluster_region_ids = regs_to_remove,
          all_region_id      = cur_region_id,
          all_x              = cur_x,
          all_y              = cur_y,
          buffer_size        = buffer_size
        )
      }
      removed_region_ids <- union(removed_region_ids, regs_to_remove)
      keep <- !(cur_region_id %in% regs_to_remove)
      cur_cases      <- cur_cases[keep]
      cur_population <- cur_population[keep]
      cur_region_id  <- cur_region_id[keep]
      cur_x          <- cur_x[keep]
      cur_y          <- cur_y[keep]
      if (scan_type == "treespatial") {
        cur_node_id <- cur_node_id[keep]
      }
    }
  }

  # Build the regions table that we attach to the result, for plotting
  # downstream. We use the ORIGINAL (uncut) inputs so that
  # get_cluster_regions() can colour every region.
  if (scan_type == "tree") {
    out_regions <- NULL
  } else if (scan_type == "circular") {
    # Circular inputs are already one row per region.
    out_regions <- .build_regions_circular(
      cases = cases, population = population,
      region_id = region_id, x = x, y = y
    )
  } else {
    # Tree-spatial: inputs are long (region x leaf). Build the unique
    # regions table directly, taking population/x/y from the first
    # occurrence of each region_id (matches .build_inputs).
    reg_ids <- unique(region_id)
    first_idx <- match(reg_ids, region_id)
    out_regions <- data.frame(
      region_id  = reg_ids,
      population = population[first_idx],
      x          = x[first_idx],
      y          = y[first_idx],
      stringsAsFactors = FALSE
    )
  }

  out <- list(
    clusters    = if (length(clusters_rows) > 0L)
                    do.call(rbind, clusters_rows) else data.frame(),
    iterations  = iterations,
    regions     = out_regions,
    tree        = tree,
    alpha       = alpha,
    nsim        = nsim,
    buffer_size = buffer_size,
    n_iter      = length(clusters_rows),
    scan_type   = scan_type
  )
  class(out) <- "sequential_scan"
  out
}


# =============================================================================
# Internal helpers
# =============================================================================

#' @keywords internal
#' Compute the union of cluster regions and their nearest-neighbour buffer.
#' Distance is Euclidean on (x, y); regions are picked by smallest minimum
#' distance to any cluster region.
.add_buffer_regions <- function(cluster_region_ids, all_region_id,
                                 all_x, all_y, buffer_size) {

  cl_mask  <- all_region_id %in% cluster_region_ids
  if (!any(cl_mask) || all(cl_mask) || buffer_size == 0L) {
    return(unique(cluster_region_ids))
  }

  out_idx <- which(!cl_mask)
  cl_x <- all_x[cl_mask]; cl_y <- all_y[cl_mask]

  # Min distance from each outside region to any cluster region.
  min_dist <- vapply(out_idx, function(i) {
    d <- sqrt((cl_x - all_x[i])^2 + (cl_y - all_y[i])^2)
    min(d)
  }, numeric(1L))

  k <- min(buffer_size, length(out_idx))
  picked <- out_idx[order(min_dist)[seq_len(k)]]
  unique(c(cluster_region_ids, all_region_id[picked]))
}


#' @keywords internal
#' Prune a tree to a subset of leaves: keep those leaves and all their
#' ancestors; drop any internal node that has no kept descendant.
.prune_tree_to_leaves <- function(tree, leaves_kept) {
  if (length(leaves_kept) == 0L) {
    return(tree[integer(0), , drop = FALSE])
  }
  # Ancestors of each kept leaf
  needed <- character(0)
  for (lf in leaves_kept) {
    nd <- as.character(lf)
    while (length(nd) == 1L && !is.na(nd) && nd != "") {
      needed <- c(needed, nd)
      parent <- tree$parent_id[match(nd, tree$node_id)]
      if (is.na(parent)) break
      nd <- as.character(parent)
      if (nd %in% needed) break
    }
  }
  needed <- unique(needed)
  tree[tree$node_id %in% needed, , drop = FALSE]
}

## =============================================================================
## vector_inputs.R
##
## Helpers introduced in treeSS 0.1.5 for the vector-based API.
##
## The user supplies parallel vectors (cases, population, region_id, x, y,
## node_id) and a tree (data.frame or two vectors). These helpers:
##   - normalize the tree input to a 2-column data.frame
##   - validate that the parallel vectors are coherent
##   - build the regions table and the leaf x region cases matrix that
##     the C++ core consumes
## =============================================================================


#' @keywords internal
#' Coerce a tree input (either a 2-column data.frame, or two parallel
#' vectors) into a canonical data.frame with columns node_id, parent_id.
.normalize_tree <- function(tree = NULL,
                             tree_node_id = NULL, tree_parent_id = NULL) {
  has_df  <- !is.null(tree)
  has_vec <- !is.null(tree_node_id) && !is.null(tree_parent_id)

  if (has_df && has_vec) {
    stop("Provide either 'tree' (data.frame) or both 'tree_node_id' and ",
         "'tree_parent_id' (vectors), not both.", call. = FALSE)
  }
  if (!has_df && !has_vec) {
    stop("Tree input is missing. Provide 'tree' as a data.frame with ",
         "columns 'node_id' and 'parent_id', or pass 'tree_node_id' and ",
         "'tree_parent_id' as parallel vectors.", call. = FALSE)
  }

  if (has_vec) {
    if (length(tree_node_id) != length(tree_parent_id)) {
      stop("'tree_node_id' and 'tree_parent_id' must have the same length.",
           call. = FALSE)
    }
    tree <- data.frame(node_id   = tree_node_id,
                       parent_id = tree_parent_id,
                       stringsAsFactors = FALSE)
  } else {
    if (!is.data.frame(tree)) {
      stop("'tree' must be a data.frame with columns 'node_id' and ",
           "'parent_id'.", call. = FALSE)
    }
    if (!all(c("node_id", "parent_id") %in% names(tree))) {
      stop("'tree' must contain columns 'node_id' and 'parent_id'.",
           call. = FALSE)
    }
  }
  tree
}


#' @keywords internal
#' Validate that the parallel vectors (cases, population, region_id, x, y,
#' node_id) are coherent: same length, no NAs in keys, etc.
.validate_vectors <- function(cases, population, region_id, x, y,
                              node_id = NULL) {
  n <- length(cases)
  if (length(population) != n) {
    stop("'population' must have the same length as 'cases'.", call. = FALSE)
  }
  if (length(region_id) != n) {
    stop("'region_id' must have the same length as 'cases'.", call. = FALSE)
  }
  if (length(x) != n) {
    stop("'x' must have the same length as 'cases'.", call. = FALSE)
  }
  if (length(y) != n) {
    stop("'y' must have the same length as 'cases'.", call. = FALSE)
  }
  if (!is.null(node_id) && length(node_id) != n) {
    stop("'node_id' must have the same length as 'cases'.", call. = FALSE)
  }
  if (any(is.na(cases))) stop("'cases' contains NA values.", call. = FALSE)
  if (any(cases < 0))    stop("'cases' must be non-negative.", call. = FALSE)
  if (any(is.na(region_id))) stop("'region_id' contains NA values.",
                                  call. = FALSE)
  if (!is.null(node_id) && any(is.na(node_id))) {
    stop("'node_id' contains NA values.", call. = FALSE)
  }
  invisible(TRUE)
}


#' @keywords internal
#' From parallel vectors, build:
#'   - regions: a data.frame with one row per unique region_id
#'              (with population, x, y; region_id is preserved)
#'   - cases_matrix: leaf x region matrix (rows in tree leaf order,
#'                   columns in regions$region_id order)
#'
#' Population/x/y are taken as the FIRST occurrence per region. If they
#' vary across rows of the same region, a warning is issued.
.build_inputs <- function(cases, population, region_id, x, y,
                           node_id, tree) {

  .validate_vectors(cases, population, region_id, x, y, node_id)
  .validate_tree(tree)

  # Order by region_id (first occurrence) to make regions table deterministic
  reg_ids <- unique(region_id)

  # Take population/x/y from first occurrence; warn if inconsistent
  first_idx <- match(reg_ids, region_id)
  reg_pop <- population[first_idx]
  reg_x   <- x[first_idx]
  reg_y   <- y[first_idx]

  # Sanity check: warn if population/x/y vary within region
  for (r in reg_ids) {
    rows <- which(region_id == r)
    if (length(rows) > 1) {
      if (length(unique(population[rows])) > 1L) {
        warning("'population' varies within region_id == ", r,
                "; using the first value.", call. = FALSE)
      }
      if (length(unique(x[rows])) > 1L || length(unique(y[rows])) > 1L) {
        warning("'x' or 'y' varies within region_id == ", r,
                "; using the first values.", call. = FALSE)
      }
    }
  }

  regions <- data.frame(
    region_id  = reg_ids,
    population = reg_pop,
    x          = reg_x,
    y          = reg_y,
    stringsAsFactors = FALSE
  )

  # Build cases matrix
  leaves <- .get_leaves(tree)
  n_leaves  <- length(leaves)
  n_regions <- nrow(regions)

  tree_leaves <- leaves
  if (is.character(tree$node_id) || is.character(node_id)) {
    node_id     <- as.character(node_id)
    tree_leaves <- as.character(tree_leaves)
  }
  if (is.character(regions$region_id) || is.character(region_id)) {
    region_id  <- as.character(region_id)
    regions$region_id <- as.character(regions$region_id)
  }

  unknown_nodes <- setdiff(unique(node_id), tree_leaves)
  if (length(unknown_nodes) > 0) {
    stop("'node_id' contains values that are not leaves of the tree: ",
         paste(utils::head(unknown_nodes, 5), collapse = ", "),
         if (length(unknown_nodes) > 5) ", ..." else "",
         ".", call. = FALSE)
  }

  row_idx <- match(node_id, tree_leaves)
  col_idx <- match(region_id, regions$region_id)

  cases_mat <- matrix(0, nrow = n_leaves, ncol = n_regions)
  rownames(cases_mat) <- as.character(tree_leaves)
  colnames(cases_mat) <- as.character(regions$region_id)

  # Drop rows where cases == 0 to match earlier behaviour
  keep <- cases > 0
  if (any(keep)) {
    key <- paste(row_idx[keep], col_idx[keep], sep = "_")
    if (anyDuplicated(key)) {
      agg <- stats::aggregate(cases[keep],
                              by = list(r = row_idx[keep],
                                        c = col_idx[keep]),
                              FUN = sum)
      cases_mat[cbind(agg$r, agg$c)] <- agg$x
    } else {
      cases_mat[cbind(row_idx[keep], col_idx[keep])] <- cases[keep]
    }
  }

  list(regions = regions, cases_matrix = cases_mat)
}


#' @keywords internal
#' Build a regions table from circular-scan vector inputs (no node_id).
.build_regions_circular <- function(cases, population, region_id, x, y) {
  .validate_vectors(cases, population, region_id, x, y, node_id = NULL)
  n <- length(cases)
  reg_ids <- unique(region_id)
  if (length(reg_ids) != n) {
    stop("For circular_scan, 'cases' must already be aggregated to one ",
         "value per region. Got ", n, " observations but ",
         length(reg_ids), " unique region_id values.\n",
         "If your data is in long format, aggregate first, e.g.:\n",
         "  agg <- aggregate(cases ~ region_id + x + y + population, ",
         "data = ..., FUN = sum)\n",
         "and then pass agg$cases, agg$population, agg$region_id, agg$x, ",
         "agg$y to circular_scan().", call. = FALSE)
  }
  data.frame(
    region_id  = region_id,
    population = population,
    x          = x,
    y          = y,
    stringsAsFactors = FALSE
  )
}


## =============================================================================
## Data/column interface helper (treeSS >= 0.2.0)
##
## The public scan functions take a `data` data.frame as their first argument
## and refer to its columns by (unquoted) name. `.resolve_col()` turns a
## captured argument expression into the corresponding vector, accepting:
##   * a bare column name:        treespatial_scan(d, cases, ...)
##   * a character column name:   treespatial_scan(d, "cases", ...)
##   * an expression on columns:  treespatial_scan(d, raw_count * w, ...)
## Expressions are evaluated in `data` first, then in the caller environment
## `env`, so locally defined variables remain visible.
## =============================================================================

#' @keywords internal
.resolve_col <- function(quo, data, env, arg_name) {
  if (is.null(quo)) return(NULL)
  # Literal string column name: treespatial_scan(d, "cases", ...)
  if (is.character(quo) && length(quo) == 1L) {
    if (!quo %in% names(data)) {
      stop(sprintf("Column \"%s\" (argument `%s`) not found in `data`.",
                   quo, arg_name), call. = FALSE)
    }
    return(data[[quo]])
  }
  val <- tryCatch(
    eval(quo, data, env),
    error = function(e)
      stop(sprintf("Cannot resolve argument `%s` = `%s` against `data`: %s",
                   arg_name, deparse(quo), conditionMessage(e)),
           call. = FALSE)
  )
  # Indirection: a variable or expression that yields a single column name,
  # e.g. `pop <- "live_births"; treespatial_scan(d, population = pop, ...)`.
  # A scalar string that names a column of `data` is taken as that column.
  # (The scan inputs are length-`nrow(data)` vectors, so a length-1 string
  # is never itself a valid value here.)
  if (is.character(val) && length(val) == 1L && val %in% names(data)) {
    return(data[[val]])
  }
  val
}

#' Build Candidate Spatial Zones Using Circular Windows
#'
#' Constructs the set of candidate spatial zones for the circular scan
#' statistic. Each zone is defined by a circular window centered on a region's
#' centroid, containing all regions whose centroids fall within the circle.
#'
#' @param regions A \code{data.frame} with columns \code{region_id},
#'   \code{population}, \code{x}, and \code{y}. Coordinates \code{x} and
#'   \code{y} represent centroids of each region.
#' @param max_pop A numeric value specifying the maximum population allowed
#'   inside a zone. If \code{NULL} (default), it is set to 50\% of the total
#'   population.
#'
#' @return A \code{list} of zones. Each zone is a list with elements:
#'   \describe{
#'     \item{center}{Integer index of the center region.}
#'     \item{region_idx}{Integer vector of region indices inside the zone.}
#'     \item{population}{Total population inside the zone.}
#'   }
#'
#' @details
#' For each region \eqn{j}, regions are sorted by distance from \eqn{j}. Regions
#' are added incrementally to the zone as long as the total population does
#' not exceed \code{max_pop}. This generates a nested set of circular windows
#' centered on each region.
#'
#' @references
#' Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
#' Statistics - Theory and Methods}, 26(6), 1481–1496.
#'
#' @export
#' @examples
#' regions <- data.frame(
#'   region_id = 1:5,
#'   population = c(1000, 2000, 1500, 800, 1200),
#'   x = c(0, 1, 2, 0.5, 1.5),
#'   y = c(0, 0, 0, 1, 1)
#' )
#' zones <- build_zones(regions, max_pop = 3000)
#' length(zones) # number of candidate zones
build_zones <- function(regions, max_pop = NULL) {

  .validate_regions(regions)

  n <- nrow(regions)
  N <- sum(regions$population)

  if (is.null(max_pop)) {
    max_pop <- 0.5 * N
  }

  # Compute distance matrix
  coords <- as.matrix(regions[, c("x", "y")])
  dist_mat <- as.matrix(stats::dist(coords, method = "euclidean"))

  pop <- regions$population

  ## Pass 1 -- for each center, walk the regions in increasing distance and
  ## record the subsequence that fits under the population cap. A region that
  ## would exceed the cap is skipped and the walk continues (identical
  ## semantics to previous versions). Only scalar bookkeeping happens inside
  ## the loop; no vectors are grown incrementally.
  included <- vector("list", n)   # per center: region indices, inclusion order
  cumpops  <- vector("list", n)   # per center: cumulative population at each k
  n_inc    <- integer(n)          # per center: number of zones (= prefixes)
  buf_idx  <- integer(n)
  buf_pop  <- numeric(n)

  for (center in seq_len(n)) {
    ordered_idx <- order(dist_mat[center, ])
    k <- 0L
    cum <- 0

    for (i in ordered_idx) {
      new_pop <- cum + pop[i]
      if (new_pop <= max_pop) {
        k <- k + 1L
        buf_idx[k] <- i
        buf_pop[k] <- new_pop
        cum <- new_pop
      }
    }

    n_inc[center]    <- k
    included[[center]] <- buf_idx[seq_len(k)]
    cumpops[[center]]  <- buf_pop[seq_len(k)]
  }

  ## Pass 2 -- each zone is a prefix of its center's inclusion sequence. The
  ## output list is preallocated at its final length and each zone's members
  ## are extracted with a single C-level subset, so construction cost is
  ## proportional to the total number of (zone, region) memberships.
  zones <- vector("list", sum(n_inc))
  zone_id <- 0L

  for (center in seq_len(n)) {
    inc <- included[[center]]
    cp  <- cumpops[[center]]
    for (k in seq_len(n_inc[center])) {
      zone_id <- zone_id + 1L
      zones[[zone_id]] <- list(
        center = center,
        region_idx = inc[seq_len(k)],
        population = cp[k]
      )
    }
  }

  zones
}

---
title: "Building treeSS inputs from raw data: Florida mortality 2016"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Building treeSS inputs from raw data: Florida mortality 2016}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>",
  eval = FALSE
)
```

## What this vignette adds

The companion vignette
(`vignette("introduction", package = "treeSS")`) walks through the
Rio de Janeiro dataset, which is **already pre-assembled** in
exactly the long format that `treespatial_scan()` expects: one row
per (region, leaf), with parallel columns for cases, population,
region centroids, and tree node identifiers.

In practice, most users will start one step earlier: with a raw
case-level table, a separately maintained polygon layer (counties,
boroughs, census tracts...), and a classification that needs to be
turned into a tree. This vignette goes through exactly that workflow
on the **`fl_deaths`** dataset shipped with the package, mirroring
the annotated example file
`inst/examples/example_florida.R`.

What you will see:

  1. How to **build the ICD-10 tree** directly from the codes that
     appear in the raw data.
  2. How to **assemble parallel vectors** of `(cases, population,
     region_id, x, y, node_id)` by joining the case table to a
     polygon layer fetched on demand.
  3. How to run `treespatial_scan()`, `filter_clusters()`, and
     `sequential_scan()` on the assembled inputs and visualise the
     results.

The dataset is the CDC WONDER Compressed Mortality File 1999–2016
restricted to Florida deaths in 2016. It covers 65 of Florida's 67
counties, 253 ICD-10 codes, and ≈157,000 deaths.

---

## 1. Load the raw data

`fl_deaths` ships in **raw long form** — one row per (county_fips,
icd10_code) combination — with population and an ICD-10 description
column attached:

```{r fl-load}
library(treeSS)
data(fl_deaths)

str(fl_deaths)
#> 'data.frame': N obs. of 5 variables:
#>  $ county_fips: chr  ...
#>  $ icd10_code : chr  "C56" "C61" "I10" ...
#>  $ icd10_desc : chr  ...
#>  $ deaths     : int  ...
#>  $ population : int  ...

c(rows         = nrow(fl_deaths),
  counties     = length(unique(fl_deaths$county_fips)),
  icd10_codes  = length(unique(fl_deaths$icd10_code)),
  total_deaths = sum(fl_deaths$deaths))
```

Note what `fl_deaths` does **not** carry:

  * No tree object — we have ICD-10 codes but no parent–child structure.
  * No spatial coordinates — we have county FIPS codes but no centroids.

We need to supply both before calling the scan.

---

## 2. Build the ICD-10 tree from the codes in the data

The codes in `fl_deaths` use two granularities:

  * **3-character codes** (e.g. `"C56"`, `"R54"`) treated as leaves at
    the 3-character level — the death certificate was coded to that
    precision and no further.
  * **5-character codes** with a dot (e.g. `"A04.7"`) — leaves under a
    3-character internal parent.

We build a three-level tree (Root → ICD-10 chapter → 3-character
node → 5-character leaf) directly from the codes that *actually
appear* in `fl_deaths`, so the tree contains no unused nodes:

```{r fl-tree}
icd_codes <- sort(unique(fl_deaths$icd10_code))

parsed <- data.frame(
  code  = icd_codes,
  three = ifelse(grepl("\\.", icd_codes),
                  sub("\\..*", "", icd_codes),
                  icd_codes),
  is_three_leaf = !grepl("\\.", icd_codes) & nchar(icd_codes) == 3,
  stringsAsFactors = FALSE
)

# Map 3-character codes to ICD-10 chapters
icd_chapter_of <- function(three) {
  L   <- substr(three, 1, 1)
  num <- suppressWarnings(as.integer(substr(three, 2, 3)))
  switch(L,
    A = "I_AB", B = "I_AB",
    C = "II_C_D",
    D = if (!is.na(num) && num <= 48) "II_C_D" else "III_D",
    E = "IV_E",  F = "V_F",  G = "VI_G",
    H = if (!is.na(num) && num <= 59) "VII_H" else "VIII_H",
    I = "IX_I",  J = "X_J",  K = "XI_K", L = "XII_L",
    M = "XIII_M", N = "XIV_N", O = "XV_O", P = "XVI_P",
    Q = "XVII_Q", R = "XVIII_R",
    S = "XIX_S_T", T = "XIX_S_T",
    V = "XX_V_Y", W = "XX_V_Y", X = "XX_V_Y", Y = "XX_V_Y",
    "Other"
  )
}
parsed$chapter <- sapply(parsed$three, icd_chapter_of)

fl_tree <- unique(rbind(
  data.frame(node_id = "Root", parent_id = NA_character_,
             stringsAsFactors = FALSE),
  data.frame(node_id = unique(parsed$chapter), parent_id = "Root",
             stringsAsFactors = FALSE),
  unique(data.frame(node_id = parsed$three, parent_id = parsed$chapter,
                    stringsAsFactors = FALSE)),
  data.frame(
    node_id   = parsed$code[!parsed$is_three_leaf],
    parent_id = parsed$three[!parsed$is_three_leaf],
    stringsAsFactors = FALSE
  )
))

# Sanity check: every code in fl_deaths is a leaf of fl_tree
parents <- unique(fl_tree$parent_id[!is.na(fl_tree$parent_id)])
leaves  <- setdiff(fl_tree$node_id, parents)
stopifnot(length(setdiff(icd_codes, leaves)) == 0)

c(total_nodes = nrow(fl_tree),
  chapters    = length(unique(parsed$chapter)),
  leaves      = length(leaves))
```

The result is a two-column data.frame `fl_tree` with columns
`node_id` and `parent_id`. The root has `parent_id = NA`. The tree
is acyclic by construction.

Two notes on style:

  * `treeSS` accepts the tree either as this two-column data.frame
    via the `tree =` argument, or as parallel vectors via the
    `tree_node_id =` and `tree_parent_id =` arguments. Both produce
    identical scans.
  * `node_id` is a string here, but `treeSS` is type-agnostic: any
    R type that supports `==` and can be coerced to character will
    work, including integers and factors.

---

## 3. Download polygons and centroids

`fl_deaths` carries county FIPS codes but not coordinates. We fetch
the 2016 county boundaries from \CRANpkg{tigris}, project to WGS84,
and compute centroids:

```{r fl-polygons}
library(sf)
library(tigris)

fl_map <- counties(state = "FL", cb = TRUE, year = 2016)
fl_map <- st_transform(fl_map, 4326)

centroids   <- st_centroid(st_geometry(fl_map))
coords      <- st_coordinates(centroids)
fl_map$x    <- coords[, "X"]
fl_map$y    <- coords[, "Y"]
```

Two technical points worth stating explicitly:

  * **Centroids of `sf` objects.** `st_centroid()` is exact under
    `sf` and respects the polygon geometry; if you use
    `st_centroid()` after `st_transform()`, the centroid is computed
    in WGS84 coordinates, which is what `treespatial_scan()` expects
    for the small spatial extents of US counties / Brazilian
    municipalities / London boroughs.
  * **`tigris` requires network access.** A first run downloads
    cached shapefiles to a user-configurable cache directory; the
    second run is instant.

---

## 4. Assemble parallel vectors

`treespatial_scan()` consumes six parallel vectors of identical
length — one row per (region, leaf) observation. We join the case
table to the population (which is constant per county in
`fl_deaths`) and to the coordinates from `fl_map`:

```{r fl-vectors}
# Per-county population: same value across all leaves, so take any one
fl_pop <- aggregate(population ~ county_fips, data = fl_deaths,
                     FUN = max)

# Per-county centroids from the polygon layer
xy <- data.frame(county_fips = fl_map$GEOID,
                  x = fl_map$x, y = fl_map$y,
                  stringsAsFactors = FALSE)

dat <- merge(fl_deaths[, c("county_fips", "icd10_code", "deaths")],
             fl_pop, by = "county_fips")
dat <- merge(dat, xy, by = "county_fips")

# Integer region_id is convenient downstream; here we use the rank
# of county_fips in alphabetical order
dat$region_id <- as.integer(factor(
  dat$county_fips, levels = sort(unique(dat$county_fips))
))

# Keep only rows with at least one death (sparse-by-default scan)
dat <- dat[dat$deaths > 0, ]

c(long_rows = nrow(dat),
  counties  = length(unique(dat$region_id)))
```

At this point `dat` is exactly the same long-format shape as
`rj_mortality` in the companion vignette: one row per
(county, ICD-10) combination, with parallel columns of cases,
population, coordinates, and node identifier.

---

## 5. Run the tree-spatial scan

```{r fl-scan}
result_fl <- treespatial_scan(
  cases       = dat$deaths,
  population  = dat$population,
  region_id   = dat$region_id,
  x           = dat$x,
  y           = dat$y,
  node_id     = dat$icd10_code,
  tree        = fl_tree,
  max_pop_pct = 0.05,
  nsim        = 999, seed = 2016,
  n_cores     = 4L
)
print(result_fl)
```

The most likely cluster typically falls on a specific ICD-10 leaf
code (5-character) concentrated in one or two adjacent counties.
The Florida dataset gives a different pattern from RJ because the
overall mortality counts in Florida are much higher and the
geography is much larger; the joint scan picks out narrow,
cause-specific excess rather than broad geographic concentrations.

`max_pop_pct = 0.05` is more restrictive than the default of
`0.50`. Larger study areas like Florida benefit from a tighter cap
because the default would otherwise allow zones covering half the
state's population, which is usually not the size of cluster the
analyst is looking for.

---

## 6. Distinct secondary clusters (paper-faithful)

```{r fl-filter}
filter_clusters(result_fl)[1:5,
                            c("node_id", "n_regions",
                              "cases", "expected", "llr",
                              "pvalue")]
```

As in the RJ analysis, `filter_clusters()` performs no additional
Monte Carlo; the secondary clusters are read from
`result_fl$secondary_clusters` and filtered for distinctness in
$O(k^2)$ time.

---

## 7. Sequential scan when the MLC may shadow

```{r fl-seq}
seq_fl <- sequential_scan(
  cases       = dat$deaths,
  population  = dat$population,
  region_id   = dat$region_id,
  x           = dat$x,
  y           = dat$y,
  node_id     = dat$icd10_code,
  tree        = fl_tree,
  max_iter    = 5L, alpha = 0.05,
  nsim        = 999, seed = 2016,
  max_pop_pct = 0.05, n_cores = 4L
)
print(seq_fl)
```

Each iteration of `sequential_scan()` runs a fresh Monte Carlo on
the dataset with the previous iteration's cluster regions
removed, so the $p$-value of each detected cluster is computed
against the correct conditional null distribution. The procedure
stops at the first non-significant iteration or `max_iter`,
whichever comes first.

---

## 8. Visualise the clusters

`get_cluster_regions()` returns a tidy data.frame that can be
joined to `fl_map` for plotting:

```{r fl-plot}
library(ggplot2)
library(dplyr)

region_info <- unique(dat[, c("region_id", "county_fips")])

cr_seq <- merge(
  get_cluster_regions(seq_fl, overlap = TRUE),
  region_info, by = "region_id"
)

# Cross-join the polygon layer with every iteration panel so all
# counties appear in every panel (those outside the analysis
# dataset show as NA-coloured rather than as an extra empty panel)
panels       <- unique(cr_seq$panel)
cr_seq_keys  <- unique(cr_seq[, c("county_fips", "cluster",
                                   "node_id", "llr", "pvalue",
                                   "panel")])
fl_seq_plot <- merge(
  do.call(rbind,
          lapply(panels, function(p) cbind(fl_map, panel = p))),
  cr_seq_keys,
  by.x = c("GEOID", "panel"),
  by.y = c("county_fips", "panel"),
  all.x = TRUE
)

ggplot(fl_seq_plot) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40",
          linewidth = 0.12, alpha = 0.75) +
  scale_fill_manual(
    values = c("#C44E52", "#4C72B0", "#55A868",
               "#8172B2", "#CCB974"),
    na.value = "gray95", na.translate = FALSE
  ) +
  facet_wrap(~ panel, nrow = 1) +
  theme_minimal() +
  theme(legend.position = "none")
```

The cross-join trick on `panels` is what lets every county appear
in every iteration panel; without it, a simple
`merge(fl_map, cr_seq, all.x = TRUE)` would leave one extra
"NA" facet for the two Florida counties not present in the
mortality dataset.

The complete script with full plot annotations, file output and
summary tables is at
`inst/examples/example_florida.R`.

---

## Recap

The Florida workflow demonstrates the three preprocessing steps that
the RJ vignette skips because `rj_mortality` arrives pre-assembled:

  1. Build the tree from the codes in the data (no hand-coded
     hierarchy).
  2. Pull polygons and centroids on demand from a spatial source
     (\CRANpkg{tigris} for US, \CRANpkg{geobr} for Brazil, etc.).
  3. Join everything into parallel long-form vectors.

After step 3, the rest of the analysis is identical to the RJ
vignette: a single `treespatial_scan()` call, optional
`filter_clusters()` or `sequential_scan()` post-processing, and
visualisation via `get_cluster_regions()`.

---

## References

Cançado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L. H.
(2025). A tree-spatial scan statistic. *Environmental and Ecological
Statistics*, 32, 953–978.
[doi:10.1007/s10651-025-00670-w](https://doi.org/10.1007/s10651-025-00670-w)

Kulldorff, M. (1997). A spatial scan statistic. *Communications in
Statistics — Theory and Methods*, 26(6), 1481–1496.

Zhang, Z., Assunção, R., & Kulldorff, M. (2010). Spatial scan
statistics adjusted for multiple clusters. *Journal of Probability
and Statistics*, 2010, 642379.

Centers for Disease Control and Prevention. *CDC WONDER Compressed
Mortality File 1999–2016*. National Center for Health Statistics.

---
title: "Introduction to treeSS"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to treeSS}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE, comment = "#>",
  eval = FALSE
)
```

## Overview

The **treeSS** package implements the tree-spatial scan statistic of
Cançado, Oliveira, Quadros and Duczmal (2025), which detects clusters
that are simultaneously anomalous in geographic space and on a
hierarchical classification tree. It generalises Kulldorff's (1997)
circular spatial scan by searching jointly over circular spatial
zones $z$ and tree branches $g$, returning the (zone, branch) pair
that maximises the likelihood ratio.

The package targets epidemiological and surveillance applications
where the events to be clustered (deaths, hospitalisations, crimes,
collisions) carry both a *location* and a *categorical hierarchy*
(ICD-10 chapter $\to$ subchapter $\to$ leaf, NAICS sector, road type,
etc.). Detecting clusters jointly in both dimensions can identify
specific cause-area combinations that pure spatial or pure tree scans
would miss.

The package provides:

| Function | Purpose |
|:---------|:--------|
| `treespatial_scan()` | Tree-spatial scan (main entry point) |
| `circular_scan()`    | Kulldorff's circular spatial scan (tree-free) |
| `tree_scan()`        | Tree-based scan (space-free) |
| `iterative_scan()`   | Sequential conditional scan with Holm–Bonferroni correction |
| `filter_clusters()`  | Distinct secondary clusters per Cançado et al. (2025), Sec. 5.1.1 |
| `get_cluster_regions()` | Tidy region-by-cluster table for plotting |
| `aggregate_tree()`   | Roll counts up the tree |

All scans accept the Poisson and Binomial models of the original paper
and run the Monte Carlo step under OpenMP via the `n_cores` argument.

The package ships four example datasets:

| Dataset | Country | Domain | Regions | Tree |
|:--------|:--------|:-------|:--------|:-----|
| `rj_mortality` + `rj_tree` | Brazil | Public health | 89 municipalities | ICD-10 (622 nodes) |
| `chicago_crimes` + `chicago_tree` | USA | Crime | 77 community areas | Type × Description × Location (2841 nodes) |
| `fl_deaths` | USA | Public health | 65 counties | (built by user from raw data) |
| `london_collisions` + `london_tree` | UK | Road safety | 33 boroughs | Light × Road × Junction (81 nodes) |

This vignette walks through `rj_mortality` (reproducing the published
analysis) and `chicago_crimes` (a different-domain illustration), then
shows how to use `iterative_scan()` for sequential cluster detection.
The Florida and London examples are shipped in `inst/examples/` and
discussed in the companion paper.

---

## Example 1 — Infant mortality in Rio de Janeiro

This reproduces Section 5.2 of Cançado et al. (2025): all live births
and infant deaths in the 89 municipalities of Rio de Janeiro state in
2016, classified by the ICD-10 hierarchy. The pre-built dataset
combines deaths, live births, ICD-10 leaf codes and centroid
coordinates in long format.

### Run the scan

```{r rj-data}
library(treeSS)
data(rj_mortality)
data(rj_tree)

str(rj_mortality, max = 1)
#> 'data.frame': 1440 obs. of 9 variables: region_id, ibge_code,
#>   name, live_births, x, y, node_id, cases, ...
```

```{r rj-scan}
result_rj <- treespatial_scan(
  cases       = rj_mortality$cases,
  population  = rj_mortality$live_births,
  region_id   = rj_mortality$region_id,
  x           = rj_mortality$x,
  y           = rj_mortality$y,
  node_id     = rj_mortality$node_id,
  tree        = rj_tree,
  max_pop_pct = 0.50,
  nsim        = 999, seed = 2016,
  n_cores     = 4L
)
print(result_rj)
```

Running this produces the most likely cluster `P209` with 18
municipalities, log-likelihood ratio $LR \approx 38.83$, $p < 0.001$,
27 observed deaths against 3.34 expected — closely matching the
published Table 7 ($LR = 38.48$, 18 municipalities, 27 deaths,
3.37 expected). The minor LR difference comes from a SIM database
update between the paper's analysis and the dataset shipped here;
the conclusions are identical.

### Multiple distinct clusters (paper-faithful)

To extract more than one cluster from a single scan, the paper's
Section 5.1.1 specifies a filtering rule: a candidate (zone, branch)
pair is retained only if its branch is disjoint from every previously
retained branch (no ancestor/descendant relation), or its zone is
disjoint from every previously retained zone (no region in common).
This is implemented in `filter_clusters()`:

```{r rj-filter}
filter_clusters(result_rj)[1:5, c("node_id", "n_regions",
                                   "cases", "expected", "llr")]
```

For visualisation, `get_cluster_regions()` returns a tidy data.frame
ready to merge with a polygon layer:

```{r rj-getregions, eval = FALSE}
# Most likely cluster only (single map)
cr1 <- get_cluster_regions(result_rj, n_clusters = 1, overlap = FALSE)

# Top-2 distinct clusters (faceted map)
cr2 <- get_cluster_regions(result_rj, n_clusters = 2, overlap = TRUE)
```

A complete plotting workflow with `geobr` and `ggplot2`:

```{r rj-plot}
library(ggplot2)
library(geobr)
library(sf)

mun <- read_municipality(code_muni = "RJ", year = 2016)
mun$code6 <- as.integer(substr(mun$code_muni, 1, 6))

cr2 <- merge(get_cluster_regions(result_rj, n_clusters = 2, overlap = TRUE),
             unique(rj_mortality[, c("region_id", "ibge_code")]),
             by = "region_id")
mun_facet <- merge(mun, cr2, by.x = "code6", by.y = "ibge_code")

ggplot(mun_facet) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40", alpha = 0.75) +
  scale_fill_manual(values = c("1" = "#C44E52", "2" = "#4C72B0"),
                    na.value = "gray95", na.translate = FALSE) +
  facet_wrap(~ panel, nrow = 1) +
  theme_minimal() +
  theme(legend.position = "none")
```

A complete worked example with detailed annotation is shipped at
`system.file("examples", "example_brazil_rj.R", package = "treeSS")`.

---

## Example 2 — Crime in Chicago

The Chicago dataset illustrates the package on a non-medical domain
with a much larger tree (2841 nodes covering crime type, description
and location). It also demonstrates a recurring choice users face
in surveillance applications: **which population should be the
denominator?**

`chicago_crimes` ships with two denominator columns:

  * `population` — the total incidents in each community area. This
    is a *compositional* denominator: it expresses each cell as a
    share of the citywide incident count and answers the question
    *"which crime types over-occur in which areas, relative to the
    citywide profile?"*
  * `pop_residential` — the residential population of each community
    area (ACS 2020 5-year, aggregated to community areas by CMAP
    Community Data Snapshots). This is the appropriate denominator
    for an *incidence-rate* analysis (incidents per resident),
    analogous to deaths per live birth in Example 1.

The choice changes both the model and the interpretation. We use the
incidence-rate denominator below.

### Run the scan

```{r chi-scan}
data(chicago_crimes)
data(chicago_tree)

result_chi <- treespatial_scan(
  cases       = chicago_crimes$cases,
  population  = chicago_crimes$pop_residential,
  region_id   = chicago_crimes$region_id,
  x           = chicago_crimes$x,
  y           = chicago_crimes$y,
  node_id     = chicago_crimes$node_id,
  tree        = chicago_tree,
  max_pop_pct = 0.25,
  nsim        = 999, seed = 2023,
  n_cores     = 4L
)
print(result_chi)
```

The most likely cluster covers a contiguous group of community areas
on Chicago's South Side (Englewood, West Englewood, Auburn Gresham,
Roseland and neighbouring CCAs), driven by an over-occurrence of
violent and property crimes against residents. The relative risk for
this cluster is approximately 1.58 — a 58 % excess over the citywide
rate.

### Visualise the most likely cluster

`chicago_crimes` joins to the bundled `chicago_map` polygon layer via
the `area_number` column:

```{r chi-plot}
data(chicago_map)

cr1 <- merge(get_cluster_regions(result_chi, n_clusters = 1, overlap = FALSE),
             unique(chicago_crimes[, c("region_id", "area_number", "name")]),
             by = "region_id")
chi <- merge(chicago_map, cr1, by.x = "AREA_NUM", by.y = "area_number",
              all.x = TRUE)

ggplot(chi) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40", alpha = 0.75) +
  scale_fill_manual(values = c("1" = "#C44E52"), na.value = "gray95",
                    name = "Cluster") +
  theme_minimal()
```

---

## Multi-cluster detection: `iterative_scan()`

`filter_clusters()` (used in Example 1) returns distinct clusters
extracted from a *single* run of the scan. An alternative — popular
in the spatial-epidemiology literature since Zhang, Assunção and
Kulldorff (2010) — is to run the scan, remove the cases assigned to
the detected cluster, and re-run on the residual data. Repeating this
yields a sequence of clusters whose subsequent components are
*conditionally most likely*, given that the prior clusters have been
explained away.

The function `iterative_scan()` implements this procedure. **Note that
it is not part of Cançado et al. (2025);** users seeking strict
fidelity to that paper should use `filter_clusters()`. The iterative
procedure is offered as a useful extension and is documented as such.

Because each iteration is a separate hypothesis test on data modified
by the previous iteration, the raw p-values overstate significance.
`iterative_scan()` collects the raw p-values from all iterations and
applies the Holm–Bonferroni correction at the end:

```{r rj-iter}
iter_rj <- iterative_scan(
  cases       = rj_mortality$cases,
  population  = rj_mortality$live_births,
  region_id   = rj_mortality$region_id,
  x           = rj_mortality$x,
  y           = rj_mortality$y,
  node_id     = rj_mortality$node_id,
  tree        = rj_tree,
  max_iter    = 5, alpha = 0.05,
  nsim        = 999, seed = 2016,
  max_pop_pct = 0.50, n_cores = 4L
)
print(iter_rj)
```

The returned `clusters` data.frame has one row per iteration with the
raw p-value, the Holm-adjusted p-value, and a `significant` flag:

```{r rj-iter-table}
iter_rj$clusters[, c("iteration", "node_id", "n_regions",
                      "llr", "pvalue", "pvalue_adjusted",
                      "significant")]
```

Mapping the iterative result uses the same S3 dispatch:

```{r rj-iter-map}
cr_it <- merge(get_cluster_regions(iter_rj, overlap = TRUE),
               unique(rj_mortality[, c("region_id", "ibge_code")]),
               by = "region_id")
mun_it <- merge(mun, cr_it, by.x = "code6", by.y = "ibge_code",
                all.x = TRUE)

ggplot(mun_it) +
  geom_sf(aes(fill = factor(cluster)), color = "gray40", alpha = 0.75) +
  scale_fill_manual(values = c("1" = "#C44E52", "2" = "#4C72B0",
                                "3" = "#55A868", "4" = "#8172B2",
                                "5" = "#CCB974"),
                    na.value = "gray95", na.translate = FALSE) +
  facet_wrap(~ panel) +
  theme_minimal() + theme(legend.position = "none")
```

When to use which:

  * Use `filter_clusters()` / `get_cluster_regions(n_clusters = N)`
    when you want to follow Cançado et al. (2025) exactly.
  * Use `iterative_scan()` when residual clustering is plausible and
    you want a sequence of conditionally distinct clusters with
    proper multiple-testing control.

---

## Computational notes

The Monte Carlo step (replicates of the null Poisson/Binomial counts
under proportional intensities) dominates the runtime. All scan
functions accept `n_cores`, which parallelises the Monte Carlo step
via OpenMP. On a 4-core laptop the Rio de Janeiro analysis runs in
under 20 seconds with `nsim = 999`; the Chicago analysis (much larger
tree) takes around two minutes.

For tasks where the case counts are concentrated near the leaves and
the tree is deep, `aggregate_tree()` can be used to compute the
internal-node sums once and reuse them across calls.

---

## Other examples

The `inst/examples/` directory of the installed package contains
fully annotated worked examples for all four datasets:

```{r examples-dir}
list.files(system.file("examples", package = "treeSS"))
#> [1] "example_brazil_rj.R"  "example_chicago.R"
#> [2] "example_florida.R"    "example_london.R"
```

The Florida script demonstrates the full workflow from raw long
data — building the ICD-10 tree from scratch, downloading county
polygons via `tigris`, assembling parallel input vectors and
plotting with `ggplot2`. The London script illustrates the use of
a non-medical tree (light × road × junction) on traffic collisions,
with an interactive `leaflet` map.

These two examples are also the basis of the detailed use cases in
the companion R Journal paper.

---

## References

Cançado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L. H.
(2025). A tree-spatial scan statistic. *Environmental and Ecological
Statistics*, 32, 953–978.
[doi:10.1007/s10651-025-00670-w](https://doi.org/10.1007/s10651-025-00670-w)

Kulldorff, M. (1997). A spatial scan statistic. *Communications in
Statistics — Theory and Methods*, 26(6), 1481–1496.

Kulldorff, M., Fang, Z., & Walsh, S. J. (2003). A tree-based scan
statistic for database disease surveillance. *Biometrics*, 59(2),
323–331.

Zhang, Z., Assunção, R., & Kulldorff, M. (2010). Spatial scan
statistics adjusted for multiple clusters. *Journal of Probability
and Statistics*, 2010, 642379.

Holm, S. (1979). A simple sequentially rejective multiple test
procedure. *Scandinavian Journal of Statistics*, 6(2), 65–70.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.treespatial_scan}
\alias{print.treespatial_scan}
\title{Print Method for treespatial_scan Objects}
\usage{
\method{print}{treespatial_scan}(x, max_show = 10L, ...)
}
\arguments{
\item{x}{An object of class \code{"treespatial_scan"}.}

\item{max_show}{Integer. Maximum number of leaf IDs and region
IDs to display in full before truncating with "... and N more".
The default of \code{10L} keeps console output compact when the
most likely cluster spans many leaves (for example, when the
root node maximises the likelihood ratio for an aggregated
denominator) or many regions; set \code{max_show = -1L} to
print every value. Mirrors the convention used by \pkg{tibble}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{x} (the input object of class
  \code{"treespatial_scan"}), unchanged. Called for the side
  effect of printing a human-readable summary of the scan result
  to the console: the total population, the number of regions
  and tree nodes, the number of Monte Carlo simulations, and the
  contents of the most likely cluster (tree node, leaf IDs,
  region IDs, cases, expected count, population, relative risk,
  log-likelihood ratio, and p-value).
}
\description{
Print Method for treespatial_scan Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{london_collisions}
\alias{london_collisions}
\title{London Road Collisions (2022)}
\format{
A data.frame in long format with columns:
\describe{
  \item{region_id}{Integer identifier of the borough.}
  \item{borough}{Borough name.}
  \item{population}{Total collisions in the borough (used as population
    denominator).}
  \item{x}{Longitude of the borough centroid.}
  \item{y}{Latitude of the borough centroid.}
  \item{node_id}{Character leaf identifier from \code{\link{london_tree}}.}
  \item{cases}{Integer count of collisions.}
}
}
\source{
UK Department for Transport, STATS19 data via the \code{stats19} R package.
}
\usage{
london_collisions
}
\description{
Combined long-format dataset of road traffic collisions in the 33 London
boroughs, 2022. Each row represents a (borough, collision-category)
combination with at least one collision. Use together with
\code{\link{london_tree}}.
}
\examples{
data(london_collisions)
head(london_collisions)
cat("Total collisions:", sum(london_collisions$cases), "\n")
}
\seealso{
\code{\link{london_tree}}, \code{\link{london_boroughs_map}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iterative_scan.R
\name{iterative_scan}
\alias{iterative_scan}
\title{Iterative Scan}
\usage{
iterative_scan(
  cases = NULL,
  population = NULL,
  region_id = NULL,
  x = NULL,
  y = NULL,
  node_id = NULL,
  tree = NULL,
  tree_node_id = NULL,
  tree_parent_id = NULL,
  max_iter = 5L,
  alpha = 0.05,
  nsim = 999L,
  max_pop_pct = 0.5,
  model = c("poisson", "binomial"),
  seed = NULL,
  verbose = TRUE,
  n_cores = 1L
)
}
\arguments{
\item{cases}{Numeric vector. For tree-spatial scan: one entry per
(region, leaf) row. For circular: one entry per region. For tree-only:
one entry per leaf.}

\item{population}{Numeric vector parallel to \code{cases}.}

\item{region_id, x, y}{Vectors parallel to \code{cases} (omit for
tree-only scan).}

\item{node_id}{Vector parallel to \code{cases} (omit for circular and
tree-only scans).}

\item{tree}{Tree as a 2-column data.frame (\code{node_id, parent_id}),
or use \code{tree_node_id}/\code{tree_parent_id}. Omit for circular
scan.}

\item{tree_node_id, tree_parent_id}{Optional. Parallel vectors as an
alternative to \code{tree}.}

\item{max_iter}{Integer. Maximum number of iterations.}

\item{alpha}{Numeric. Significance threshold applied to Holm-Bonferroni
adjusted p-values.}

\item{nsim}{Integer. MC simulations per iteration.}

\item{max_pop_pct}{Numeric. Passed to inner scans.}

\item{model}{Character. \code{"poisson"} or \code{"binomial"}.}

\item{seed}{Integer or \code{NULL}.}

\item{verbose}{Logical.}

\item{n_cores}{Integer. OpenMP threads.}
}
\value{
An object of class \code{"iterative_scan"} with components:
  \describe{
    \item{clusters}{A data.frame with one row per iteration, containing
      the raw p-value (\code{pvalue}), the Holm-Bonferroni adjusted
      p-value (\code{pvalue_adjusted}), a logical \code{significant}
      column, plus the cluster's node, regions, cases, expected, RR, and
      LLR.}
    \item{iterations}{A list with the full scan result of each iteration.}
    \item{regions, tree, alpha, n_iter, scan_type}{Bookkeeping fields.}
  }
}
\description{
Runs the scan statistic iteratively, removing the cases of each detected
cluster before the next iteration. Supports tree-spatial, circular, and
tree-only scans, dispatched by which arguments are supplied.
}
\details{
\strong{Note on methodology.} This iterative procedure is \emph{not} part
of the original tree-spatial scan statistic of Cancado et al. (2025). It is
an extension inspired by the conditional scan approach of Zhang et al.
(2010), which sequentially removes cases attributed to detected clusters
before re-running the scan to find additional, distinct anomalies. To
reproduce the secondary-cluster procedure described in Section 5.1.1 of
Cancado et al. (2025), use \code{\link{filter_clusters}} or
\code{\link{get_cluster_regions}} with \code{n_clusters > 1} on a single
scan result instead.

\strong{Multiple-testing correction.} Because each iteration is a separate
hypothesis test on data that has been modified by the previous iteration,
the raw p-values overstate significance. This function collects raw p-values
from all iterations performed and applies the Holm-Bonferroni correction
(\code{\link[stats]{p.adjust}} with \code{method = "holm"}) at the end. The
returned \code{clusters} data.frame includes both the raw p-value
(\code{pvalue}) and the adjusted p-value (\code{pvalue_adjusted}), plus a
logical \code{significant} column indicating whether the cluster is
significant after correction at level \code{alpha}.

The loop stops early only when the residual signal is exhausted (the most
likely cluster has \code{LR = 0} or zero cases), not based on raw p-values,
since the multiple-testing correction depends on the full set of tests.
}
\examples{
\donttest{
data(london_collisions); data(london_tree)
result <- iterative_scan(
  cases       = london_collisions$cases,
  population  = london_collisions$population,
  region_id   = london_collisions$region_id,
  x           = london_collisions$x,
  y           = london_collisions$y,
  node_id     = london_collisions$node_id,
  tree        = london_tree,
  max_iter = 3, nsim = 99, seed = 42
)
print(result)
}
}
\references{
Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L. H.
(2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
Statistics}, 32, 953-978.

Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
Statistics - Theory and Methods}, 26(6), 1481-1496.

Zhang, Z., Assuncao, R., & Kulldorff, M. (2010). Spatial scan statistics
adjusted for multiple clusters. \emph{Journal of Probability and
Statistics}, 2010, 642379.

Holm, S. (1979). A simple sequentially rejective multiple test procedure.
\emph{Scandinavian Journal of Statistics}, 6(2), 65-70.
}
\seealso{
\code{\link{treespatial_scan}}, \code{\link{circular_scan}},
  \code{\link{tree_scan}}, \code{\link{filter_clusters}},
  \code{\link{get_cluster_regions}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{summary.circular_scan}
\alias{summary.circular_scan}
\title{Summary Method for circular_scan Objects}
\usage{
\method{summary}{circular_scan}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"circular_scan"}.}

\item{...}{Further arguments passed to \code{print.circular_scan()}.
In particular, \code{max_show} controls how many region IDs of
the most-likely cluster are printed in full before truncating
with \code{"... and N more"}; see
\code{\link{print.circular_scan}} for details.}
}
\value{
Invisibly returns \code{object} (the input object of
  class \code{"circular_scan"}), unchanged. Called for the side
  effect of printing the same fields as
  \code{print.circular_scan()}, followed by a numeric summary
  (min, quartiles, median, mean, max) of the simulated
  log-likelihood-ratio distribution under the null.
}
\description{
Prints the same fields as \code{print.circular_scan()}, followed
by a numeric summary of the simulated log-likelihood-ratio
distribution under the null. Useful for checking that the Monte
Carlo replicates produced a reasonable spread relative to the
observed test statistic.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treespatial_scan.R
\name{treespatial_scan}
\alias{treespatial_scan}
\title{Tree-Spatial Scan Statistic}
\usage{
treespatial_scan(
  data,
  cases,
  population,
  region_id,
  x,
  y,
  node_id,
  tree = NULL,
  tree_node_id = NULL,
  tree_parent_id = NULL,
  max_pop_pct = 0.5,
  nsim = 999L,
  alpha = 0.05,
  model = c("poisson", "binomial"),
  seed = NULL,
  n_cores = 1L
)
}
\arguments{
\item{data}{A \code{data.frame} in long format with one row per
(region, tree-leaf) observation. The \code{cases}, \code{population},
\code{region_id}, \code{x}, \code{y} and \code{node_id} arguments name
columns of this data.frame.}

\item{cases}{Column of \code{data} giving the number of cases observed
for each (region, leaf) row. Given as an unquoted column name (a string
or an expression on columns also works).}

\item{population}{Column of \code{data} giving the population (or
denominator) of the region for each row. The same value should be
repeated across all rows of a given region; if it varies, the first
occurrence per region is used and a warning is issued.}

\item{region_id}{Column of \code{data} giving the region identifier.}

\item{x, y}{Columns of \code{data} giving the region centroid coordinates.
Like \code{population}, these should be constant within region.}

\item{node_id}{Column of \code{data} giving the tree leaf identifier for
each row. Each value must match a leaf of the tree.}

\item{tree}{A \code{data.frame} with columns \code{node_id} and
\code{parent_id}. The root node(s) must have \code{parent_id = NA}.
As an alternative, pass \code{tree_node_id} and \code{tree_parent_id}
as parallel vectors instead of this argument.}

\item{tree_node_id, tree_parent_id}{Optional. Parallel vectors describing
the tree edges, used as an alternative to \code{tree}. If both
\code{tree} and these vectors are supplied, an error is raised.}

\item{max_pop_pct}{Numeric. Maximum proportion of total population
allowed inside a zone. Default \code{0.5}.}

\item{nsim}{Integer. Number of Monte Carlo simulations. Default \code{999}.}

\item{alpha}{Numeric. Significance level. Default \code{0.05}.}

\item{model}{Character. \code{"poisson"} (default) or \code{"binomial"}.}

\item{seed}{Integer or \code{NULL}. Random seed for the Monte
Carlo loop. When non-\code{NULL}, the user's pre-existing RNG
state is saved on entry and restored on exit, so the seed
argument affects only this call and does not leak into
subsequent draws in the user's session.}

\item{n_cores}{Integer. OpenMP threads for the Monte Carlo loop.
Default \code{1L} (serial).}
}
\value{
An object of class \code{"treespatial_scan"}.
}
\description{
Performs the tree-spatial scan statistic, combining Kulldorff's circular
scan (spatial clusters) with the tree-based scan (hierarchical data
mining). Searches all combinations of spatial zones and tree branches
to identify pairs \eqn{(z, g)} with significantly more cases than
expected.
}
\details{
Inputs are supplied through a single \code{data} data.frame in long
format (one row per (region, tree-leaf) observation); the case, population,
region, coordinate and leaf columns are referenced by name. The user is
responsible for choosing which column to use as \code{population} (e.g.,
total population, live births, person-years), making the choice of
denominator explicit.

\strong{Secondary clusters.} The returned object contains the most likely
cluster as well as the full set of evaluated (zone, branch) pairs in
\code{secondary_clusters}. To obtain the distinct secondary clusters as
described in Section 5.1.1 of Cancado et al. (2025) (filtering out pairs
that overlap in regions or branches with already-retained clusters), use
\code{\link{filter_clusters}} or \code{\link{get_cluster_regions}} with
\code{n_clusters > 1}.
}
\examples{
set.seed(123)
n_regions <- 10
tree <- data.frame(
  node_id   = c(1, 2, 3, 4, 5, 6, 7),
  parent_id = c(NA, 1, 1, 2, 2, 3, 3)
)
# Long-format data.frame: one row per (region, leaf) combination
crimes <- expand.grid(region_id = 1:n_regions, node_id = c(4, 5, 6, 7))
xy <- data.frame(region_id = 1:n_regions,
                 x = runif(n_regions, 0, 10),
                 y = runif(n_regions, 0, 10))
crimes <- merge(crimes, xy, by = "region_id")
crimes$population <- 1000
crimes$cases <- rpois(nrow(crimes), lambda = 5)
crimes$cases[crimes$node_id == 4 & crimes$region_id \%in\% 1:3] <- rpois(3, 30)

result <- treespatial_scan(
  crimes,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = tree,
  nsim        = 99
)
print(result)
}
\references{
Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
(2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
Statistics}, 32, 953--978. \doi{10.1007/s10651-025-00670-w}
}
\seealso{
\code{\link{circular_scan}}, \code{\link{tree_scan}},
  \code{\link{aggregate_tree}}, \code{\link{filter_clusters}},
  \code{\link{get_cluster_regions}}, \code{\link{sequential_scan}}
}

% Generated manually for treeSS 0.1.44; keep in sync with roxygen comments
% in R/iterative_scan.R. devtools::document() will regenerate this file from
% those comments without manual intervention.
\name{print.iterative_scan}
\alias{print.iterative_scan}
\title{Print Method for iterative_scan Objects}
\usage{
\method{print}{iterative_scan}(x, max_show = 10L, ...)
}
\arguments{
\item{x}{An object of class \code{"iterative_scan"}.}

\item{max_show}{Integer. Accepted for API consistency with
\code{print.treespatial_scan()}, \code{print.tree_scan()}, and
\code{print.circular_scan()}. Currently the only field affected
is the per-iteration \code{leaf_ids} column of \code{x$clusters},
which is omitted from the printed table by default to keep the
summary compact; pass \code{max_show = -1L} to include both
\code{leaf_ids} and \code{region_ids} in the printed table.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{x} (the input object of class
\code{"iterative_scan"}), unchanged. Called for the side
effect of printing a human-readable summary of the iterative
scan result to the console: the scan type, the number of
iterations performed, the significance threshold (applied to
Holm-Bonferroni adjusted p-values), the number of significant
clusters after correction, and a per-iteration table of
detected clusters.
}
\description{
Print Method for iterative_scan Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tree_scan}
\alias{print.tree_scan}
\title{Print Method for tree_scan Objects}
\usage{
\method{print}{tree_scan}(x, max_show = 10L, ...)
}
\arguments{
\item{x}{An object of class \code{"tree_scan"}.}

\item{max_show}{Integer. Maximum number of leaf IDs to display
in full before truncating with "... and N more". The default of
\code{10L} keeps console output compact when a node spans
hundreds or thousands of leaves (such as the root of a deep
tree); set \code{max_show = -1L} to print every leaf.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{x} (the input object of class
  \code{"tree_scan"}), unchanged. Called for the side effect of
  printing a human-readable summary of the scan result to the
  console: the total cases and population, the number of tree
  nodes, the number of Monte Carlo simulations, the contents of
  the most likely cluster (node ID, leaf IDs, cases, expected
  count, log-likelihood ratio, and p-value), and the top
  significant cuts at the chosen significance threshold.
}
\description{
Print Method for tree_scan Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rj_mortality}
\alias{rj_mortality}
\title{Rio de Janeiro Infant Mortality (2016)}
\format{
A data.frame in long format with columns:
\describe{
  \item{region_id}{Integer identifier of the municipality.}
  \item{ibge_code}{6-digit IBGE municipality code.}
  \item{name}{Municipality name.}
  \item{population}{IBGE municipal population estimate for 2016.}
  \item{live_births}{Number of live births in 2016 (SINASC). Used as the
    population denominator in Section 5.2 of Cancado et al. (2025).}
  \item{x}{Longitude of the municipality centroid.}
  \item{y}{Latitude of the municipality centroid.}
  \item{node_id}{Character ICD-10 leaf code (4 characters), matching a
    leaf of \code{\link{rj_tree}}.}
  \item{cases}{Integer count of infant deaths.}
}
}
\source{
Population: IBGE municipal estimates. Live births: DATASUS/SINASC via
TabNet. Deaths: DATASUS/SIM microdata via OpenDATASUS
(\url{https://opendatasus.saude.gov.br}). Centroids: \code{geobr}.
}
\usage{
rj_mortality
}
\description{
Combined long-format dataset of infant mortality in the 92 municipalities
of Rio de Janeiro state, Brazil, 2016. Each row represents a (municipality,
ICD-10 leaf) combination with at least one death; missing combinations
are implicitly zero. Use together with \code{\link{rj_tree}}.
}
\details{
To reproduce Section 5.2 of Cancado et al. (2025), use \code{live_births}
as the population denominator:
\preformatted{
  data <- rj_mortality
  data$population <- data$live_births
  treespatial_scan(data, cases, population, region_id, x, y, node_id,
                   tree = rj_tree, ...)
}

Municipality polygons are provided as the \code{rj_map} dataset.
}
\examples{
data(rj_mortality)
head(rj_mortality)
cat("Total deaths:", sum(rj_mortality$cases), "\n")
}
\references{
Cancado, A.L.F., Oliveira, G.S., Quadros, A.V.C., Duczmal, L. (2025).
A tree-spatial scan statistic. \emph{Environmental and Ecological
Statistics}, 32, 953--978. \doi{10.1007/s10651-025-00670-w}
}
\seealso{
\code{\link{rj_tree}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rj_map}
\alias{rj_map}
\title{Rio de Janeiro Municipal Boundaries}
\format{
An \code{sf} data.frame with 92 rows and 3 columns:
\describe{
  \item{ibge_code}{6-digit IBGE municipality code; join key to
    \code{rj_mortality$ibge_code}.}
  \item{NAME}{Municipality name.}
  \item{geometry}{MULTIPOLYGON geometry in WGS84 (EPSG:4326).}
}
}
\source{
Municipal boundaries (Malhas Municipais) from the Brazilian Institute of
Geography and Statistics (IBGE).
}
\usage{
rj_map
}
\description{
Polygon boundaries for the 92 municipalities of Rio de Janeiro state,
used to map clusters from the \code{rj_mortality} analysis.
}
\examples{
data(rj_map)
cat("Municipalities:", nrow(rj_map), "\n")
}
\seealso{
\code{\link{rj_mortality}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{summary.sequential_scan}
\alias{summary.sequential_scan}
\title{Summary Method for sequential_scan Objects}
\usage{
\method{summary}{sequential_scan}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"sequential_scan"}.}

\item{...}{Passed to \code{print.sequential_scan()}.}
}
\value{
Invisibly returns \code{object}. Called for the side effect
  of printing the same fields as \code{print.sequential_scan()}
  followed by, for each iteration, the cluster composition
  (region IDs or leaf IDs).
}
\description{
Summary Method for sequential_scan Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_clusters.R
\name{filter_clusters}
\alias{filter_clusters}
\title{Filter Overlapping Secondary Clusters}
\usage{
filter_clusters(result, alpha = NULL)
}
\arguments{
\item{result}{An object of class \code{"treespatial_scan"},
\code{"circular_scan"}, or \code{"tree_scan"}.}

\item{alpha}{Numeric. Significance level for filtering. Default is
\code{NULL}, which uses the alpha from the original analysis.}
}
\value{
A \code{data.frame} of distinct significant clusters ordered by
  descending LLR.
}
\description{
Removes overlapping secondary clusters from the output of
\code{\link{treespatial_scan}}, \code{\link{circular_scan}}, or
\code{\link{tree_scan}}.
}
\details{
For \code{treespatial_scan} objects, two clusters overlap if they have
BOTH tree overlap (one node is ancestor/descendant of the other) AND
spatial overlap (same center or identical set of regions). This follows
Cancado et al. (2025).

For \code{circular_scan} objects, the criterion follows Kulldorff (1997):
a secondary cluster is distinct if its center is not contained in any
previously retained cluster's zone, and no previously retained cluster's
center is contained in the candidate zone.

For \code{tree_scan} objects, a secondary cluster is distinct if its node
is NOT an ancestor or descendant of any previously retained node. This
follows Kulldorff et al. (2003).

When a dominant signal saturates the candidate pool (making distinct
secondary clusters hard to find), or when clusters shadow each other,
consider using \code{\link{sequential_scan}} (Zhang, Assuncao and
Kulldorff, 2010), which re-runs the scan after removing each detected
cluster.
}
\examples{
data(london_collisions); data(london_tree)
result <- treespatial_scan(
  london_collisions,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = london_tree,
  nsim        = 99, seed = 42
)
filter_clusters(result)

}
\references{
Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
Statistics - Theory and Methods}, 26(6), 1481-1496.

Kulldorff, M., Fang, Z., & Walsh, S. J. (2003). A tree-based scan
statistic for database disease surveillance. \emph{Biometrics}, 59(2),
323-331.

Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
(2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
Statistics}, 32, 953-978. \doi{10.1007/s10651-025-00670-w}
}
\seealso{
\code{\link{treespatial_scan}}, \code{\link{circular_scan}},
  \code{\link{tree_scan}}, \code{\link{sequential_scan}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/circular_scan.R
\name{circular_scan}
\alias{circular_scan}
\title{Kulldorff's Circular Spatial Scan Statistic}
\usage{
circular_scan(
  data,
  cases,
  population,
  region_id,
  x,
  y,
  max_pop_pct = 0.5,
  nsim = 999L,
  alpha = 0.05,
  n_secondary = 1000L,
  model = c("poisson", "binomial"),
  seed = NULL,
  n_cores = 1L
)
}
\arguments{
\item{data}{A \code{data.frame} with one row per region. The
\code{cases}, \code{population}, \code{region_id}, \code{x} and \code{y}
arguments name columns of this data.frame.}

\item{cases}{Column of \code{data} giving the total cases in each region.
Given as an unquoted column name (a string or expression also works).}

\item{population}{Column of \code{data} giving the population (or
denominator) of each region.}

\item{region_id}{Column of \code{data} giving the region identifier.}

\item{x, y}{Columns of \code{data} giving the region centroid coordinates.}

\item{max_pop_pct}{Numeric. Default \code{0.5}.}

\item{nsim}{Integer. Number of MC simulations. Default \code{999}.}

\item{alpha}{Numeric. Significance level. Default \code{0.05}.}

\item{n_secondary}{Integer. Default \code{1000}.}

\item{model}{Character. \code{"poisson"} or \code{"binomial"}.}

\item{seed}{Integer or \code{NULL}. Random seed for the Monte
Carlo loop. When non-\code{NULL}, the user's pre-existing RNG
state is saved on entry and restored on exit, so the seed
argument affects only this call and does not leak into
subsequent draws in the user's session.}

\item{n_cores}{Integer. OpenMP threads.}
}
\value{
An object of class \code{"circular_scan"}.
}
\description{
Performs Kulldorff's circular spatial scan statistic for detecting
spatial clusters. Inputs are supplied through a single \code{data}
data.frame with one row per region (cases must already be aggregated to
the region level); the relevant columns are referenced by name.
}
\examples{
set.seed(42)
n <- 20
regions <- data.frame(
  region_id  = 1:n,
  cases      = rpois(n, lambda = 10),
  population = rep(1000, n),
  x          = runif(n, 0, 10),
  y          = runif(n, 0, 10)
)
regions$cases[1:5] <- rpois(5, lambda = 30)

result <- circular_scan(
  regions,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  nsim        = 99
)
print(result)
}
\references{
Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
Statistics - Theory and Methods}, 26(6), 1481-1496.
}
\seealso{
\code{\link{filter_clusters}}, \code{\link{tree_scan}},
  \code{\link{treespatial_scan}}, \code{\link{get_cluster_regions}},
  \code{\link{sequential_scan}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{london_boroughs_map}
\alias{london_boroughs_map}
\title{London Borough Boundaries}
\format{
An \code{sf} data.frame with 33 rows and 3 columns:
\describe{
  \item{NAME}{Borough name, e.g. \code{"Westminster"}.}
  \item{GSS_CODE}{ONS geography code, e.g. \code{"E09000033"}.}
  \item{geometry}{MULTIPOLYGON geometry in WGS84 (EPSG:4326).}
}
}
\source{
London Datastore, Statistical GIS Boundary Files for London
(see \url{https://data.london.gov.uk/dataset/statistical-gis-boundary-files-london/}).
Contains National Statistics data (c) Crown copyright and database
right 2014.
}
\usage{
london_boroughs_map
}
\description{
Simplified polygon boundaries for the 33 London boroughs. Use with
\code{\link{london_collisions}} for map-based visualization of cluster
results.
}
\details{
Geometries are simplified (\code{st_simplify(dTolerance = 0.001)}) to
reduce file size. For full-resolution boundaries, download from the
source URL.

To merge with cluster results from \code{\link{get_cluster_regions}},
use: \code{merge(london_boroughs_map, cr, by.x = "NAME", by.y = "borough")}.
}
\examples{
data(london_boroughs_map)
cat("Boroughs:", nrow(london_boroughs_map), "\n")
}
\seealso{
\code{\link{london_collisions}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treeSS-package.R
\docType{package}
\name{treeSS-package}
\alias{treeSS}
\alias{treeSS-package}
\title{treeSS: Tree-Spatial Scan Statistic for Cluster Detection}
\description{
Implements the tree-spatial scan statistic for detecting clusters that
combine both spatial and hierarchical structures, as proposed by Cançado
et al. (2025). The method extends Kulldorff's (1997) circular spatial scan
statistic and the tree-based scan statistic (Kulldorff et al. 2003) by
searching for anomalies in both geographic regions and branches of
hierarchical trees simultaneously.
}
\section{Main functions}{

\describe{
  \item{\code{\link{treespatial_scan}}}{Performs the tree-spatial scan
    statistic, detecting clusters defined by pairs of spatial zones and
    tree branches.}
  \item{\code{\link{circular_scan}}}{Performs Kulldorff's circular spatial
    scan statistic.}
  \item{\code{\link{tree_scan}}}{Performs the tree-based scan statistic.}
}
}

\section{Helper functions}{

\describe{
  \item{\code{\link{build_zones}}}{Constructs candidate spatial zones using
    circular windows centered on region centroids.}
  \item{\code{\link{aggregate_tree}}}{Aggregates case counts from leaves to
    internal nodes of the hierarchical tree.}
  \item{\code{\link{filter_clusters}}}{Removes overlapping secondary
    clusters from a single-pass scan output (Cançado et al. 2025,
    Sec. 5.1.1).}
  \item{\code{\link{sequential_scan}}}{Sequential adjustment for secondary
    clusters: removes the most likely cluster's regions and re-runs the
    scan with a fresh Monte Carlo simulation (Zhang, Assunção &
    Kulldorff 2010).}
  \item{\code{\link{get_cluster_regions}}}{Returns a tidy
    region-by-cluster data.frame suitable for plotting with any mapping
    package.}
}
}

\references{
Cançado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
(2025). A tree-spatial scan statistic. \emph{Environmental and Ecological
Statistics}, 32, 953–978. \doi{10.1007/s10651-025-00670-w}

Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
Statistics - Theory and Methods}, 26(6), 1481–1496.

Kulldorff, M., Fang, Z., & Walsh, S. J. (2003). A tree-based scan
statistic for database disease surveillance. \emph{Biometrics}, 59(2),
323–331.

Zhang, Z., Assunção, R., & Kulldorff, M. (2010). Spatial scan statistics
adjusted for multiple clusters. \emph{Journal of Probability and
Statistics}, 2010, 642379.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/allanvc/treeSS}
  \item Report bugs at \url{https://github.com/allanvc/treeSS/issues}
}

}
\author{
\strong{Maintainer}: Allan Quadros \email{allanvcq@gmail.com} (\href{https://orcid.org/0000-0003-3250-5380}{ORCID})

Authors:
\itemize{
  \item Andre L. F. Cançado (\href{https://orcid.org/0000-0002-7019-8521}{ORCID})
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_cluster_regions.R
\name{get_cluster_regions}
\alias{get_cluster_regions}
\alias{get_cluster_regions.default}
\alias{get_cluster_regions.sequential_scan}
\title{Extract Cluster Membership for Each Region}
\usage{
get_cluster_regions(result, n_clusters = 1L, overlap = TRUE, ...)

\method{get_cluster_regions}{default}(result, n_clusters = 1L, overlap = TRUE, ...)

\method{get_cluster_regions}{sequential_scan}(result, n_clusters = 1L, overlap = TRUE, ...)
}
\arguments{
\item{result}{A \code{"treespatial_scan"} or \code{"circular_scan"}
object (single-pass scan), or a \code{"sequential_scan"} object
(Zhang, Assuncao and Kulldorff, 2010).}

\item{n_clusters}{Integer. (Single-pass methods only.) Number of
clusters to extract. \code{1} returns only the most likely cluster.
Values greater than 1 use \code{\link{filter_clusters}} internally
to identify distinct secondary clusters. Default is \code{1}.
Ignored for sequential scan inputs (all detected iterations are
returned).}

\item{overlap}{Logical. If \code{TRUE} (default), returns a facet-ready
data.frame: all regions are replicated for each cluster panel, with a
\code{panel} column for faceting and \code{cluster} marked only for
member regions. If \code{FALSE}, returns one row per region assigned
to its highest-ranked cluster (for single-panel maps).}

\item{...}{Further arguments passed to methods.}
}
\value{
A \code{data.frame} with columns from \code{result$regions} plus:
  \describe{
    \item{cluster}{Integer cluster number (1 = most likely / first
      iteration, 2 = first secondary / second iteration, etc.), or
      \code{NA} if the region is not in the cluster.}
    \item{node_id}{The tree node of the cluster, or \code{NA}.}
    \item{llr}{The log-likelihood ratio of the cluster, or \code{NA}.}
    \item{pvalue}{The p-value of the cluster, or \code{NA}.}
    \item{panel}{(Only when \code{overlap = TRUE}) A two-line label
      for \code{facet_wrap}, with the cluster identifier on the first
      line and the test statistic on the second. For single-pass
      scans the label looks like \code{"#1 P209\\n(LR=39.6)"}; for
      sequential scans it looks like
      \code{"Iter 1: P209\\n(LR=39.6, p=0.005)"}. The newline keeps
      long node identifiers from overflowing the strip in multi-panel
      layouts.}
  }
}
\description{
Returns a data.frame indicating which cluster (if any) each region
belongs to. This is the primary output for visualization — use it with
any mapping package (ggplot2, leaflet, tmap, etc.).
}
\details{
This is a generic with methods for objects returned by
\code{\link{treespatial_scan}}, \code{\link{circular_scan}}, and
\code{\link{sequential_scan}}.
}
\examples{
data(london_collisions); data(london_tree)
result <- treespatial_scan(
  london_collisions,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = london_tree,
  nsim        = 99, seed = 42
)

# Long format suitable for merging with a polygon layer.
cr <- get_cluster_regions(result, n_clusters = 2L, overlap = TRUE)
head(cr)

}
\seealso{
\code{\link{treespatial_scan}}, \code{\link{circular_scan}},
  \code{\link{sequential_scan}}, \code{\link{filter_clusters}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rj_tree}
\alias{rj_tree}
\title{Rio de Janeiro Infant Mortality - ICD-10 Tree}
\format{
A data.frame with 622 rows and 2 columns:
\describe{
  \item{node_id}{Character identifier of each ICD-10 node.}
  \item{parent_id}{Character identifier of the parent node. \code{NA}
    for the root.}
}
}
\source{
Constructed from the leaf-level codes in \code{\link{rj_mortality}}, which
in turn come from the Brazilian Mortality Information System (SIM)
maintained by DATASUS. See \code{data-raw/} in the GitHub repository for
the build script.
}
\usage{
rj_tree
}
\description{
ICD-10 hierarchy used by the \code{\link{rj_mortality}} dataset.
Three levels: Chapter -> 3-character category -> 4-character subcategory.
}
\details{
The tree has 410 leaves (4-character ICD-10 codes) and 622 total nodes.
}
\examples{
data(rj_tree)
head(rj_tree)
}
\seealso{
\code{\link{rj_mortality}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{london_tree}
\alias{london_tree}
\title{London Road Collisions - Hierarchy}
\format{
A data.frame with 81 rows and 2 columns:
\describe{
  \item{node_id}{Character identifier of each node.}
  \item{parent_id}{Character identifier of the parent node. \code{NA}
    for the root.}
}
}
\source{
Constructed from the leaf-level codes in \code{\link{london_collisions}}.
See \code{data-raw/} in the GitHub repository for the build script.
}
\usage{
london_tree
}
\description{
Hierarchical classification of road collisions by light conditions,
road type, and junction detail.
}
\details{
3-level tree: Root -> Light conditions (Daylight/Darkness) -> Road type
(5 types) -> Junction detail (7 types). 68 leaf categories.
}
\examples{
data(london_tree)
head(london_tree)
}
\seealso{
\code{\link{london_collisions}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{summary.tree_scan}
\alias{summary.tree_scan}
\title{Summary Method for tree_scan Objects}
\usage{
\method{summary}{tree_scan}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"tree_scan"}.}

\item{...}{Further arguments passed to \code{print.tree_scan()}.
In particular, \code{max_show} controls how many leaf IDs of
the most-likely cluster are printed in full before truncating
with \code{"... and N more"}; see \code{\link{print.tree_scan}}
for details.}
}
\value{
Invisibly returns \code{object} (the input object of
  class \code{"tree_scan"}), unchanged. Called for the side
  effect of printing the same fields as
  \code{print.tree_scan()}, followed by the full table of all
  candidate cuts (not just significant ones) ordered by
  decreasing log-likelihood ratio.
}
\description{
Prints the same fields as \code{print.tree_scan()}, followed by
the full table of all candidate cuts (not just significant ones)
ordered by decreasing log-likelihood ratio.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate_tree.R
\name{aggregate_tree}
\alias{aggregate_tree}
\title{Aggregate Case Counts from Leaves to All Nodes in a Hierarchical Tree}
\usage{
aggregate_tree(
  data,
  cases,
  region_id,
  node_id,
  tree = NULL,
  tree_node_id = NULL,
  tree_parent_id = NULL
)
}
\arguments{
\item{data}{A \code{data.frame} in long format with one row per
(region, leaf) observation. The \code{cases}, \code{region_id} and
\code{node_id} arguments name columns of this data.frame.}

\item{cases}{Column of \code{data} giving case counts. Given as an
unquoted column name (a string or expression also works).}

\item{region_id}{Column of \code{data} giving the region identifier.}

\item{node_id}{Column of \code{data} giving the leaf identifier.}

\item{tree}{A \code{data.frame} with columns \code{node_id} and
\code{parent_id}. As an alternative, pass \code{tree_node_id} and
\code{tree_parent_id}.}

\item{tree_node_id, tree_parent_id}{Optional. Parallel vectors as an
alternative to \code{tree}.}
}
\value{
A matrix of dimensions \eqn{m \times k} (nodes x regions), with
  rows ordered by \code{tree$node_id} and columns by region.
}
\description{
Given case counts at the leaf level (supplied as columns of a long-format
\code{data} data.frame) and a tree, aggregates case counts upward from
child nodes to parent nodes, producing a complete case matrix indexed by
all nodes (rows) and regions (columns).
}
\details{
This function is exposed for inspection and pedagogical use; the scan
functions call it internally on the matrix they build from your input
data.
}
\examples{
tree <- data.frame(
  node_id   = c(1, 2, 3, 4, 5, 6, 7),
  parent_id = c(NA, 1, 1, 2, 2, 3, 3)
)
# Leaves are 4, 5, 6, 7; long-format data.frame
dat <- data.frame(
  cases     = c(10, 5, 3, 8,  2, 7, 4, 1,  6, 3, 9, 2),
  region_id = rep(1:3, each = 4),
  node_id   = rep(c(4, 5, 6, 7), times = 3)
)
aggregate_tree(dat, cases = cases, region_id = region_id,
               node_id = node_id, tree = tree)
}
\references{
Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal, L.
(2025). A tree-spatial scan statistic.
\emph{Environmental and Ecological Statistics}, 32, 953-978.
\doi{10.1007/s10651-025-00670-w}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fl_deaths}
\alias{fl_deaths}
\title{Florida General Mortality Data (2016)}
\format{
A data.frame with 3,066 rows and 6 columns:
\describe{
  \item{county_fips}{5-digit FIPS code (character), e.g. \code{"12086"}.}
  \item{county_name}{County name, e.g. \code{"Miami-Dade County"}.}
  \item{icd10_code}{ICD-10 cause of death code, e.g. \code{"I25.1"}.}
  \item{icd10_desc}{Description of the ICD-10 code, e.g.
    \code{"Atherosclerotic heart disease"}.}
  \item{deaths}{Number of deaths (integer).}
  \item{population}{County population estimate (integer).}
}
}
\source{
CDC WONDER Compressed Mortality File 1999--2016
(\url{https://wonder.cdc.gov/cmf-icd10.html}).
}
\usage{
fl_deaths
}
\description{
Death counts by county and ICD-10 cause of death for the state of Florida,
USA, 2016. This is \strong{raw data} -- the user builds the ICD-10 tree
and the cases matrix in the analysis script, making it a pedagogical
example of the full workflow.
}
\details{
Covers 65 of Florida's 67 counties (2 small counties excluded by CDC
suppression rules), 253 ICD-10 codes, and 157,000 total deaths. All
ages, all causes.

To use with \code{\link{treespatial_scan}}, rename and merge with
centroids:
\preformatted{
  # Centroids from tigris
  data <- transform(fl_deaths,
                    region_id = county_fips,
                    node_id   = icd10_code,
                    cases     = deaths)
  # Add x, y from a centroids table
  data <- merge(data, centroids, by = "county_fips")
  treespatial_scan(data, cases, population, region_id, x, y, node_id,
                   tree = fl_tree, ...)
}

County polygons and centroids can be obtained via
\code{tigris::counties(state = "FL", cb = TRUE)}.

The ICD-10 descriptions in \code{icd10_desc} can be used to build a
lookup table: \code{unique(fl_deaths[, c("icd10_code", "icd10_desc")])}.
}
\examples{
data(fl_deaths)
head(fl_deaths)
cat("Counties:", length(unique(fl_deaths$county_fips)), "\n")
cat("ICD-10 codes:", length(unique(fl_deaths$icd10_code)), "\n")
cat("Total deaths:", sum(fl_deaths$deaths), "\n")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{summary.treespatial_scan}
\alias{summary.treespatial_scan}
\title{Summary Method for treespatial_scan Objects}
\usage{
\method{summary}{treespatial_scan}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"treespatial_scan"}.}

\item{...}{Further arguments passed to
\code{print.treespatial_scan()}. In particular, \code{max_show}
controls how many leaf IDs and region IDs of the most-likely
cluster are printed in full before truncating with \code{"...
and N more"}; see \code{\link{print.treespatial_scan}} for
details.}
}
\value{
Invisibly returns \code{object} (the input object of
  class \code{"treespatial_scan"}), unchanged. Called for the
  side effect of printing the same fields as
  \code{print.treespatial_scan()}, followed by the top-10
  branches by total case count and a numeric summary (min,
  quartiles, median, mean, max) of the simulated
  log-likelihood-ratio distribution under the null.
}
\description{
Prints the same fields as \code{print.treespatial_scan()}, followed
by the top-10 branches by total case count and a numeric summary
of the simulated log-likelihood-ratio distribution under the null.
Useful for diagnosing how heavily one branch of the tree dominates
the dataset and whether the observed maximum LR is a clear outlier
against the simulated null.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{chicago_map}
\alias{chicago_map}
\title{Chicago Community Area Boundaries}
\format{
An \code{sf} data.frame with 77 rows and 3 columns:
\describe{
  \item{NAME}{Community area name.}
  \item{AREA_NUM}{Community area number (integer, 1--77).}
  \item{geometry}{MULTIPOLYGON geometry in WGS84 (EPSG:4326).}
}
}
\source{
City of Chicago Data Portal, Community Area Boundaries.
}
\usage{
chicago_map
}
\description{
Simplified polygon boundaries for the 77 Chicago community areas.
}
\examples{
data(chicago_map)
cat("Community areas:", nrow(chicago_map), "\n")
}
\seealso{
\code{\link{chicago_crimes}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sequential_scan.R
\name{sequential_scan}
\alias{sequential_scan}
\title{Sequential Scan for Secondary Clusters}
\usage{
sequential_scan(
  data,
  cases,
  population,
  region_id = NULL,
  x = NULL,
  y = NULL,
  node_id = NULL,
  tree = NULL,
  tree_node_id = NULL,
  tree_parent_id = NULL,
  max_iter = 5L,
  alpha = 0.05,
  nsim = 999L,
  max_pop_pct = 0.5,
  buffer_size = 0L,
  model = c("poisson", "binomial"),
  seed = NULL,
  verbose = TRUE,
  n_cores = 1L
)
}
\arguments{
\item{data}{A \code{data.frame} whose columns are named by the
\code{cases}, \code{population} and (depending on scan type)
\code{region_id}, \code{x}, \code{y}, \code{node_id} arguments.}

\item{cases}{Column of \code{data} giving case counts. Given as an
unquoted column name (a string or expression also works).}

\item{population}{Column of \code{data} giving the denominator.}

\item{region_id, x, y}{Columns of \code{data} (omit for tree-only scan).}

\item{node_id}{Column of \code{data} (omit for circular scan).}

\item{tree}{Tree as a 2-column data.frame
(\code{node_id, parent_id}), or use
\code{tree_node_id}/\code{tree_parent_id}. Omit for circular scan.}

\item{tree_node_id, tree_parent_id}{Optional. Parallel vectors as an
alternative to \code{tree}.}

\item{max_iter}{Integer. Maximum number of iterations (including the
primary MLC). Default 5.}

\item{alpha}{Numeric. Significance level. Iteration stops when the
MLC p-value of the current reduced data exceeds \code{alpha}.
Default 0.05.}

\item{nsim}{Integer. Monte Carlo replications per iteration.
Default 999.}

\item{max_pop_pct}{Numeric. Maximum zone-size constraint, passed to
the inner scans. Default 0.5.}

\item{buffer_size}{Integer. Number of nearest-neighbour regions to
remove together with each detected cluster, computed by Euclidean
distance from each cluster region to the remaining ones. Default
0. Zhang et al. (2010) report that the type I error and power are
essentially insensitive to the buffer size and that
\code{buffer_size = 0} is generally adequate. Ignored for
tree-only scans.}

\item{model}{Character. \code{"poisson"} (default) or
\code{"binomial"}.}

\item{seed}{Integer or \code{NULL}. Seed for the first iteration;
subsequent iterations advance the RNG state to avoid using the
same seed across iterations.}

\item{verbose}{Logical. Print progress at each iteration.}

\item{n_cores}{Integer. OpenMP threads passed to inner scans.}
}
\value{
An object of class \code{"sequential_scan"} with components:
  \describe{
    \item{clusters}{A data.frame with one row per iteration performed,
      with columns \code{iteration}, \code{node_id} (only for
      tree-spatial / tree-only scans), \code{n_regions} (only for
      scans with a spatial component), \code{cases}, \code{expected},
      \code{population}, \code{rr} (only for scans with a spatial
      component), \code{llr}, \code{pvalue}, and a logical
      \code{significant} flag (\code{pvalue <= alpha}). The list-
      columns \code{region_ids} and/or \code{leaf_ids} carry the
      cluster composition.}
    \item{iterations}{A list with the full scan result of each
      iteration.}
    \item{regions, tree, alpha, nsim, buffer_size, n_iter,
          scan_type}{Bookkeeping fields. \code{n_iter} is the number
      of iterations actually performed.}
  }
}
\description{
Implements the sequential adjustment of Zhang, Assuncao and Kulldorff
(2010) for detecting secondary clusters. After the most likely cluster
(MLC) is detected and found significant, the regions composing it
(optionally together with a buffer of nearest-neighbour regions) are
\emph{removed} from the dataset -- treated as a \dQuote{lake} with no
population and no cases -- and the scan is re-run on the reduced data
with a fresh Monte Carlo simulation. The procedure is iterated until
the MLC of the current reduced data is no longer significant, or
\code{max_iter} is reached.
}
\details{
This differs from \code{\link{filter_clusters}}, which scans the
single-pass candidate pool for non-overlapping clusters (the original
Cancado et al. 2025 criterion). The sequential approach computes a
fresh, unconservative p-value for each secondary cluster, conditional
on the prior detections.

Inputs are supplied through a single \code{data} data.frame; the
relevant columns are referenced by name. Which columns are supplied
selects one of three scan types:
\itemize{
  \item tree-spatial: all of \code{cases}, \code{population},
        \code{region_id}, \code{x}, \code{y}, \code{node_id},
        \code{tree} are required (\code{data} in long format, one row
        per (region, leaf));
  \item circular: \code{cases}, \code{population}, \code{region_id},
        \code{x}, \code{y} (no \code{node_id} / \code{tree};
        one row per region);
  \item tree-only: \code{cases}, \code{population}, \code{node_id},
        \code{tree} (no spatial inputs). \code{buffer_size} is ignored
        for the tree-only case.
}
}
\examples{
\donttest{
data(london_collisions); data(london_tree)
result <- sequential_scan(
  london_collisions,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = london_tree,
  max_iter = 3, nsim = 99, seed = 42
)
print(result)
}
}
\references{
Zhang, Z., Assuncao, R., & Kulldorff, M. (2010). Spatial scan
statistics adjusted for multiple clusters. \emph{Journal of
Probability and Statistics}, 2010, 642379.

Cancado, A. L. F., Oliveira, G. S., Quadros, A. V. C., & Duczmal,
L. H. (2025). A tree-spatial scan statistic. \emph{Environmental
and Ecological Statistics}, 32, 953-978.
\doi{10.1007/s10651-025-00670-w}

Kulldorff, M. (1997). A spatial scan statistic.
\emph{Communications in Statistics - Theory and Methods}, 26(6),
1481-1496.
}
\seealso{
\code{\link{treespatial_scan}}, \code{\link{circular_scan}},
  \code{\link{tree_scan}}, \code{\link{filter_clusters}},
  \code{\link{get_cluster_regions}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{chicago_tree}
\alias{chicago_tree}
\title{Chicago Crime - Hierarchy}
\format{
A data.frame with 2,841 rows and 2 columns:
\describe{
  \item{node_id}{Character identifier of each node.}
  \item{parent_id}{Character identifier of the parent node. \code{NA}
    for the root.}
}
}
\source{
Constructed from the leaf-level codes in \code{\link{chicago_crimes}}.
See \code{data-raw/} in the GitHub repository for the build script.
}
\usage{
chicago_tree
}
\description{
Hierarchical classification of crime incidents by type, description and
location group.
}
\details{
4-level tree: Root -> Crime Type -> Description -> Location Group.
2,486 leaf nodes. Leaf node IDs follow the pattern
\code{"<TYPE> | <DESCRIPTION> | <LOCATION>"}.
}
\examples{
data(chicago_tree)
head(chicago_tree)
}
\seealso{
\code{\link{chicago_crimes}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_zones.R
\name{build_zones}
\alias{build_zones}
\title{Build Candidate Spatial Zones Using Circular Windows}
\usage{
build_zones(regions, max_pop = NULL)
}
\arguments{
\item{regions}{A \code{data.frame} with columns \code{region_id},
\code{population}, \code{x}, and \code{y}. Coordinates \code{x} and
\code{y} represent centroids of each region.}

\item{max_pop}{A numeric value specifying the maximum population allowed
inside a zone. If \code{NULL} (default), it is set to 50\% of the total
population.}
}
\value{
A \code{list} of zones. Each zone is a list with elements:
  \describe{
    \item{center}{Integer index of the center region.}
    \item{region_idx}{Integer vector of region indices inside the zone.}
    \item{population}{Total population inside the zone.}
  }
}
\description{
Constructs the set of candidate spatial zones for the circular scan
statistic. Each zone is defined by a circular window centered on a region's
centroid, containing all regions whose centroids fall within the circle.
}
\details{
For each region \eqn{j}, regions are sorted by distance from \eqn{j}. Regions
are added incrementally to the zone as long as the total population does
not exceed \code{max_pop}. This generates a nested set of circular windows
centered on each region.
}
\examples{
regions <- data.frame(
  region_id = 1:5,
  population = c(1000, 2000, 1500, 800, 1200),
  x = c(0, 1, 2, 0.5, 1.5),
  y = c(0, 0, 0, 1, 1)
)
zones <- build_zones(regions, max_pop = 3000)
length(zones) # number of candidate zones
}
\references{
Kulldorff, M. (1997). A spatial scan statistic. \emph{Communications in
Statistics - Theory and Methods}, 26(6), 1481–1496.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.circular_scan}
\alias{print.circular_scan}
\title{Print Method for circular_scan Objects}
\usage{
\method{print}{circular_scan}(x, max_show = 10L, ...)
}
\arguments{
\item{x}{An object of class \code{"circular_scan"}.}

\item{max_show}{Integer. Maximum number of region IDs to display
in full before truncating with "... and N more". The default of
\code{10L} keeps console output compact for large clusters; set
\code{max_show = -1L} (or any value larger than the cluster
size) to print every region. Mirrors the convention used by
\pkg{tibble} for printing wide tables.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{x} (the input object of class
  \code{"circular_scan"}), unchanged. Called for the side effect
  of printing a human-readable summary of the scan result to the
  console: the total cases and population, the number of regions
  scanned, the number of Monte Carlo simulations, and the
  contents of the most likely cluster (region IDs, cases,
  expected count, population, relative risk, log-likelihood
  ratio, and p-value).
}
\description{
Print Method for circular_scan Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{chicago_crimes}
\alias{chicago_crimes}
\title{Chicago Crime (2023)}
\format{
A data.frame in long format with columns:
\describe{
  \item{region_id}{Integer identifier of the community area.}
  \item{area_number}{Official community area number (1--77).}
  \item{name}{Community area name.}
  \item{population}{Total incidents in the area. This is a compositional
    denominator (sums to the total incident count of the city) and is
    useful for analyses that ask "which crime types over-occur in which
    areas relative to the citywide profile". Not a population at risk.}
  \item{pop_residential}{Residential population of the community area.
    Use this as the denominator for incidence-rate analyses (incidents
    per resident). Source: U.S. Census Bureau, ACS 2020 5-year estimates,
    aggregated to community areas by CMAP Community Data Snapshots.
    The 77 values sum to approximately 2.71M, matching the published
    Chicago population.}
  \item{x}{Longitude of the centroid.}
  \item{y}{Latitude of the centroid.}
  \item{node_id}{Character leaf identifier from \code{\link{chicago_tree}}.}
  \item{cases}{Integer count of crime incidents.}
}
}
\source{
Crime incidents: City of Chicago Data Portal, Crimes -- 2001 to Present.

Residential population: U.S. Census Bureau ACS 2020 5-year estimates,
aggregated to community areas by the Chicago Metropolitan Agency for
Planning (CMAP), Community Data Snapshots (2023 release).
\url{https://cmap.illinois.gov/data/community-data-snapshots/}
}
\usage{
chicago_crimes
}
\description{
Combined long-format dataset of crime incidents in the 77 community areas
of Chicago, 2023. Use together with \code{\link{chicago_tree}}.
}
\examples{
data(chicago_crimes)
head(chicago_crimes)
cat("Total incidents:", sum(chicago_crimes$cases), "\n")
cat("Total residents:",
    sum(unique(chicago_crimes[, c("area_number", "pop_residential")])$pop_residential),
    "\n")
}
\seealso{
\code{\link{chicago_tree}}, \code{\link{chicago_map}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_scan.R
\name{tree_scan}
\alias{tree_scan}
\title{Tree-Based Scan Statistic}
\usage{
tree_scan(
  data,
  cases,
  node_id,
  tree = NULL,
  population = NULL,
  tree_node_id = NULL,
  tree_parent_id = NULL,
  nsim = 999L,
  alpha = 0.05,
  model = c("poisson", "binomial"),
  seed = NULL,
  n_cores = 1L
)
}
\arguments{
\item{data}{A \code{data.frame} with one row per leaf node. The
\code{cases}, \code{node_id} and (optional) \code{population} arguments
name columns of this data.frame.}

\item{cases}{Column of \code{data} giving case counts at the leaf level.
Given as an unquoted column name (a string or expression also works).}

\item{node_id}{Column of \code{data} giving the leaf identifier for each
row. Each value must match a leaf of the tree.}

\item{tree}{A \code{data.frame} with columns \code{node_id} and
\code{parent_id}. Root node(s) must have \code{parent_id = NA}.
Alternatively, pass the tree as parallel vectors via
\code{tree_node_id} and \code{tree_parent_id}.}

\item{population}{Optional column of \code{data} giving the population at
the leaf level. For the binomial model, \code{population} is the number
of trials (cases + controls) per leaf and is required. For the
Poisson model, defaults to \code{1} per leaf when omitted.}

\item{tree_node_id, tree_parent_id}{Optional parallel vectors describing
the tree as an alternative to the \code{tree} data.frame. Both must
have the same length, and the root node(s) must have
\code{tree_parent_id = NA}. Ignored when \code{tree} is supplied.}

\item{nsim}{Integer. Number of Monte Carlo simulations. Default is
\code{999}.}

\item{alpha}{Numeric. Significance level. Default is \code{0.05}.}

\item{model}{Character. Likelihood model: either \code{"poisson"}
(default) or \code{"binomial"}.}

\item{seed}{Integer or \code{NULL}. Random seed for the Monte
Carlo loop. When non-\code{NULL}, the user's pre-existing RNG
state is saved on entry and restored on exit, so the seed
argument affects only this call and does not leak into
subsequent draws in the user's session.}

\item{n_cores}{Integer. Number of OpenMP threads for the Monte Carlo
loop. Default is \code{1L} (serial). Set higher to parallelize.}
}
\value{
An object of class \code{"tree_scan"} (see package help for
  details).
}
\description{
Performs the tree-based scan statistic for detecting clusters in
hierarchical data. Uses a Poisson or binomial model with Monte Carlo
simulation (implemented in C++ via Rcpp) for significance testing.
}
\details{
Inputs are supplied through a single \code{data} data.frame with one row
per leaf (rows are matched to the tree by \code{node_id}, so they need
not be pre-ordered); counts are summed within leaf.
}
\examples{
tree <- data.frame(
  node_id   = c(1, 2, 3, 4, 5, 6, 7, 8),
  parent_id = c(NA, 1, 1, 2, 2, 3, 3, 3)
)
# One row per leaf (leaves are 4, 5, 6, 7, 8)
leaf_data <- data.frame(
  node_id = c(4, 5, 6, 7, 8),
  cases   = c(50, 5, 3, 2, 4),
  pop     = c(100, 100, 100, 100, 100)
)

result <- tree_scan(leaf_data, cases = cases, node_id = node_id,
                    tree = tree, population = pop, nsim = 99)
print(result)
}
\references{
Kulldorff, M., Fang, Z., & Walsh, S. J. (2003). A tree-based scan
statistic for database disease surveillance. \emph{Biometrics}, 59(2),
323–331.
}
\seealso{
\code{\link{circular_scan}}, \code{\link{treespatial_scan}},
  \code{\link{aggregate_tree}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.sequential_scan}
\alias{print.sequential_scan}
\title{Print Method for sequential_scan Objects}
\usage{
\method{print}{sequential_scan}(x, max_show = 10L, ...)
}
\arguments{
\item{x}{An object of class \code{"sequential_scan"}.}

\item{max_show}{Integer. Maximum number of region IDs (or leaf IDs,
for tree-only scans) to display in full per iteration before
truncating with \code{"... and N more"}. Default \code{10L}; set
\code{max_show = -1L} to print every value.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
Invisibly returns \code{x} (the input object of class
  \code{"sequential_scan"}), unchanged. Called for the side effect
  of printing a human-readable summary of the sequential scan
  result to the console: the scan type, the buffer size, the
  number of iterations performed, the significance threshold, the
  number of significant clusters, and a per-iteration table of
  detected clusters.
}
\description{
Print Method for sequential_scan Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_example_data.R
\name{generate_example_data}
\alias{generate_example_data}
\title{Generate Example Data for Tree-Spatial Scan}
\usage{
generate_example_data(
  n_regions = 50L,
  pop_per_region = 1000,
  cluster_regions = 1:7,
  cluster_leaves = c(3, 4),
  rr = 2,
  Cg = 200L,
  seed = NULL
)
}
\arguments{
\item{n_regions}{Integer. Default 50.}

\item{pop_per_region}{Numeric. Default 1000.}

\item{cluster_regions}{Integer vector. Default \code{1:7}.}

\item{cluster_leaves}{Integer vector. Default \code{c(3, 4)}.}

\item{rr}{Numeric. Relative risk. Default 2.0.}

\item{Cg}{Integer. Cases per branch. Default 200.}

\item{seed}{Integer or \code{NULL}. Random seed. When non-\code{NULL},
the user's pre-existing RNG state is saved on entry and restored on
exit, so the \code{seed} argument affects only the result of the
call. Default \code{NULL} (the user's session-level RNG state is
used as-is and is not modified by the function).}
}
\value{
A list with vector components ready to feed into
  \code{\link{treespatial_scan}}: \code{cases}, \code{population},
  \code{region_id}, \code{x}, \code{y}, \code{node_id}, plus the
  \code{tree} (data.frame) and a \code{true_cluster} list describing the
  injected cluster.
}
\description{
Creates a synthetic dataset for demonstrating and testing the tree-spatial
scan statistic. Returns parallel vectors (\code{cases}, \code{population},
\code{region_id}, \code{x}, \code{y}, \code{node_id}) and a \code{tree},
matching the input format expected by \code{\link{treespatial_scan}}.
}
\examples{
ex <- generate_example_data(seed = 42)
dat <- data.frame(
  cases = ex$cases, population = ex$population, region_id = ex$region_id,
  x = ex$x, y = ex$y, node_id = ex$node_id
)
result <- treespatial_scan(
  dat,
  cases       = cases,
  population  = population,
  region_id   = region_id,
  x           = x,
  y           = y,
  node_id     = node_id,
  tree        = ex$tree,
  nsim        = 99
)
print(result)
}
